<?php

namespace WPEP\API\System;

use WPEP\API\Utils\Report as API_Utils_Report;
use WPEP\API\Utils\User as API_Utils_User;
use WPEP\Entity\EmailNotification as WPEP_Entity_EmailNotification;

class Email {

  public static function activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    $first_slot_key = key( $time_slots );
    $current_slot = $time_slots[ key( $time_slots ) ];
    unset( $time_slots[ $first_slot_key ] );

    foreach( $time_slots as $next_slot ) {
      $email_sent_count = WPEP_Entity_EmailNotification::get_delivery_count( [
        'time_from'  => $current_slot,
        'time_to'    => $next_slot
      ] );

      $email_open_count = WPEP_Entity_EmailNotification::get_open_count( [
        'time_from'  => $current_slot,
        'time_to'    => $next_slot
      ] );


      $current_time_slot_data = [];

      if( $email_sent_count != 0 )
        $current_time_slot_data[ 'Emails - Sent' ] = $email_sent_count;

      if( $email_open_count != 0 )
        $current_time_slot_data[ 'Emails - Opened' ] = $email_open_count;

      $response[ $current_slot ] = $current_time_slot_data;

      $current_slot = $next_slot;
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

  public static function entries( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $pagination_limit = 20;
    $current_page     = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );

    $get_entries_args = [
      'count_total' => true,
      'limit'       => $pagination_limit,
      'offset'      => ( $current_page - 1 ) * $pagination_limit
    ];

    if( isset( $request_data[ 'report_time_from' ] ) && isset( $request_data[ 'report_time_to' ] ) )
      $get_entries_args[ 'created_at' ] = [ $request_data[ 'report_time_from' ] => $request_data[ 'report_time_to' ]  ];

    $get_entries_response = WPEP_Entity_EmailNotification::get_entries( $get_entries_args );

    $entries = [];
    $entries_user_objects = [];

    if( !empty( $get_entries_response[ 'results' ] ) ) {
      $entries_user_objects = wpep_utilities_map_object( get_users( [ 'ids' => wpep_utility_map_to_object_assoc(  $get_entries_response[ 'results' ], 'user_id', 'user_id') ] ), 'ID' );

      foreach( $get_entries_response[ 'results' ] as $entry_key => $entry ) {

        $entries[] = [
          'user_id'          => $entry->user_id,
          'email_title'      => $entry->title,
          'email_created_at' => wpep_format_date_with_time( $entry->created_at ),
          'email_opened_at'  => ( $entry->opened_at === NULL ? '-' : wpep_format_date_with_time( $entry->opened_at ) )
        ];
      }
    }

    $response = [
      'status'          =>  'ok',
      'entries_header'  => [
        'email_title'      => __( "Email Title", 'wpep' ),
        'email_created_at' => __( "Sent At", "wpep" ),
        'email_opened_at'  => __( "Opened At", "wpep" )
      ],
      'entries'         => $entries,
      'pagination'      => [
        'current'       => $current_page,
        'total'         => ceil( $get_entries_response[ 'total' ] / $pagination_limit ),
        'total_entries' => $get_entries_response[ 'total' ]
      ]
    ];

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function entries_analytics( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $pagination_limit = 20;
    $current_page     = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );

    $get_entries_args = [
      'count_total' => true,
      'limit'       => $pagination_limit,
      'offset'      => ( $current_page - 1 ) * $pagination_limit
    ];


    if( isset( $request_data[ 'report_time_from' ] ) && isset( $request_data[ 'report_time_to' ] ) )
      $get_entries_args[ 'created_at' ] = [ $request_data[ 'report_time_from' ] => $request_data[ 'report_time_to' ]  ];

    $get_entries_response = WPEP_Entity_EmailNotification::get_entries_analyze( $get_entries_args );

    $entries = [];

    foreach( $get_entries_response[ 'results' ] as $entry_key => $entry ) {
      $entries[] = [
        'title'             => $entry->title,
        'sent_open_rate'    => wpep_get_percentage( $entry->open_count, $entry->sent_count ),
        'sent_opened'       => $entry->open_count,
        'sent_total'        => $entry->sent_count,
        'sent_last_date'    => wpep_format_date_with_time( $entry->last_date ),
        'sent_first_date'   => wpep_format_date_with_time( $entry->first_date ),
      ];
    }

    return [
      'status'          =>  'ok',
      'entries_header'  => [
        'title'           => __( "Title", 'wpep' ),
        'sent_open_rate'  => __( "Open Rate", "wpep" ),
        'sent_opened'     => __( "Emails Opened", "wpep" ),
        'sent_total'      => __( "Emails Sent", "wpep" ),
        'sent_last_date'  => __( "Last Sent", "wpep" ),
        'sent_first_date' => __( "First Sent", "wpep" ),
      ],
      'entries'         => $entries,
      'pagination'      => [
        'current'       => $current_page,
        'total'         => ceil( $get_entries_response[ 'total' ] / $pagination_limit ),
        'total_entries' => $get_entries_response[ 'total' ]
      ]
    ];
  }

}