<?php

namespace WPEP\API\System;

class Misc {

  public static function template_authentication( $request_data ) {
    wpep_request()->set_current_post_id( isset( $request_data[ 'post_id' ] ) ? intval( $request_data[ 'post_id' ] ) : 0 );

    return wpep_template_get( 'authentication.php', [
      'include_header' => false,
      'include_footer' => false,
      'post_id'        => ( isset( $request_data[ 'post_id' ] ) ? intval( $request_data[ 'post_id' ] ) : 0 )
    ] );
  }

  public static function menu_notifications() {
    $total = wpep_platform_health()->service->get_notification_count();

    return [
      'status'  => 'ok',
      'total'   => $total,
      'map'     => [
        'admin.php?page=wpep-platform-health' => $total
      ]
    ];
  }

  public static function update_course_user_data( $request_data ) {
    if( !isset( $request_data['course_id'] ) || !isset( $request_data['user_id'] ) )
      return false;

    $key   = sanitize_text_field( $request_data['data_key'] );
    $value = ( isset( $request_data[ 'data_value' ] ) ? sanitize_text_field( $request_data['data_value'] ) : sanitize_text_field( $request_data['field_value'] ) );
    $course_id = intval( $request_data['course_id'] );
    $user_id   = intval( $request_data['user_id'] );

    if( in_array( $key, [ WPEP_USER_COURSE_STARTED, WPEP_USER_COURSE_COMPLETED_AT, WPEP_USER_COURSE_LATEST_INTERACTION ] ) )
      $value = strtotime( $value );

    wpep_content_type_course_instance()->user->data->set( $key, $value, $course_id, 0,0, $user_id );

    return true;
  }

  public static function reset_course_user_progress( $request_data ) {
    if( !isset( $request_data['course_id'] ) || !isset( $request_data['user_id'] ) )
      return false;

    $course_id = intval( $request_data['course_id'] );
    $user_id   = intval( $request_data['user_id'] );

    wpep_content_type_course_instance()->user->reset_progress( $course_id, $user_id );

    return true;
  }

  public static function reset_assessment_user_progress( $request_data ) {
    if( !isset( $request_data['assessment_id'] ) || !isset( $request_data['user_id'] ) )
      return false;

    $assessment_id = intval( $request_data['assessment_id'] );
    $user_id   = intval( $request_data['user_id'] );

    wpep_content_type_assessment_instance()->user->data->delete(
      false,
      $assessment_id,
      false,
      false,
      $user_id
    );

    return true;
  }

  public static function submit_assessment_review( $request_data ) {
    if( !isset( $request_data['assessment_id'] ) || !isset( $request_data['user_id'] ) )
      return false;

    $assessment_id = intval( $request_data['assessment_id'] );
    $user_id   = intval( $request_data['user_id'] );

    foreach( $request_data['question_is_correct'] as $question_id => $is_correct )
      wpep_content_type_assessment_instance()->user->data->set( 'is_correct', intval( $is_correct ), $assessment_id, $question_id, 0, $user_id );

    wpep_content_type_assessment_instance()->user->refresh_submission_grading( $assessment_id, $user_id );

    if( wpep_get_setting_flag( 'assessment-review-completed-email-notification-status' ) )
      wpep_controller()->userEngagement->assessment->send_student_email_notification( $assessment_id, $user_id );

    return true;
  }

}