<?php

namespace WPEP\API\System;

use WPEP\API\Utils\Report as API_Utils_Report;

class Statistics {

  public static function configuration_report( $request_data ) {
    $analyze_activity_posts = [];
    $list_activity_posts = [];

    foreach( wpep_content_type()->contentTypes as $contentType ) {
      $analyze_activity_posts[ 'activity-' . $contentType->get_post_type() ] = [
        'title'            =>  $contentType->get_plural_name(),
        'type'             => 'data-view-analyze',
        'data_action'      => 'wpep/system/user/post-type-activity-report',
        'data_action_args' => [
          'post_type' => $contentType->get_post_type()
        ]
      ];

      $list_activity_posts[ 'list-' . $contentType->get_post_type() ] = [
        'title'            =>  $contentType->get_plural_name(),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/system/user/post-type-activity-list',
        'data_action_args' => [
          'post_type' => $contentType->get_post_type()
        ]
      ];
    }

    $report_data_sources = [
      'activity-metrics' => [
        'title'       => __( "Activity Metrics", "wpep" ),
        'type'        => 'group',
        'group'       => [
          'overview' => [
            'title'       =>  __( "General", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/system/user/activity-report'
          ],
          'authentication' => [
            'title'       =>  __( "Authentication", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/system/user/authentication-activity-report'
          ],
          'discussion' => [
            'title'       =>  __( "Discussion", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/system/discussion/activity-report'
          ],
          'post-content-activity' => [
            'title'       => __( "Content Administration", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/system/post/content-activity-report'
          ],
          'email' => [
            'title'       =>  __( "Email", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/system/email/activity-report'
          ],
        ] + $analyze_activity_posts
      ],
      'activity-list' => [
        'title'       => __( "Activity List", "wpep" ),
        'type'        => 'group',
        'group'       => [
          'user-list' => [
            'title'       => __( "User Activity", "wpep" ),
            'type'        => 'data-view-list',
            'data_action' => 'wpep/system/user/entries'
          ],
          'email-analytics-list' => [
            'title'       =>  __( "Email Analytics", "wpep" ),
            'type'        => 'data-view-list',
            'data_action' => 'wpep/system/email/entries-analytics'
          ],
          'email-list' => [
            'title'       =>  __( "Delivered Emails", "wpep" ),
            'type'        => 'data-view-list',
            'data_action' => 'wpep/system/email/entries'
          ],
        ] + $list_activity_posts
      ],
    ];

    if( wpep_get_setting_flag( 'enable-student-supervisor' ) ) {
      $supervisor_users = wpep_get_users_by_role( WPEP_USER_ROLE_STUDENT_SUPERVISOR );

      if( !empty( $supervisor_users ) ) {
        $report_data_sources[ 'student-supervisor-list' ] = [
          'title'       =>  __( "Student Supervisor List", "wpep" ),
          'type'        => 'group',
          'group'       => []
        ];

        foreach( $supervisor_users as $supervisor_user ) {
          $report_data_sources[ 'student-supervisor-list' ][ 'group' ][ 'supervisor-analytics-' . $supervisor_user->ID ] = [
            'title'            => wpep_content_user_profile_format_name( $supervisor_user ),
            'type'             => 'analytics-panel',
            'data_action'      => 'wpep/system/statistics-current-user/configuration-report',
            'data_action_args' => [
              'scu_user_id' => $supervisor_user->ID
            ]
          ];
        }
      }
    }

    return [
        'status'            => 'ok',
        'report_data_sources' => $report_data_sources,
      ] + API_Utils_Report::configuration_for_statistics( $request_data );
  }

  /**
   * ACL Methods are handled by app/App.php
   * @param $request_data
   * @return bool|array
   */
  public static function user_id_configuration_report( $request_data ) {
    if( !isset( $request_data[ 'user_id' ] ) )
      return false;

    $user = get_user_by( 'ID', intval( $request_data[ 'user_id' ] ) );

    if( false === $user )
      return false;

    $analyze_activity_posts = [];
    $list_activity_posts = [];

    foreach( wpep_content_type()->contentTypes as $contentType ) {
      $analyze_activity_posts[ 'activity-' . $contentType->get_post_type() ] = [
        'title'            =>  $contentType->get_plural_name(),
        'type'             => 'data-view-analyze',
        'data_action'      => 'wpep/system/user/post-type-activity-report',
        'data_action_args' => [
          'post_type' => $contentType->get_post_type(),
          'user_id'   => $user->ID
        ]
      ];

      $list_activity_posts[ 'list-' . $contentType->get_post_type() ] = [
        'title'            =>  $contentType->get_plural_name(),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/system/user/post-type-activity-list',
        'data_action_args' => [
          'post_type' => $contentType->get_post_type(),
          'user_id'   => $user->ID
        ]
      ];
    }

    $report_data_sources = [
      'activity-list' => [
        'title'       => __( "Activity List", "wpep" ),
        'type'        => 'group',
        'group'       => $list_activity_posts
      ],
      'activity-metrics' => [
        'title'       => __( "Activity Metrics", "wpep" ),
        'type'        => 'group',
        'group'       => [
          'overview' => [
            'title'            =>  __( "General", "wpep" ),
            'type'             => 'data-view-analyze',
            'data_action'      => 'wpep/system/user/activity-report',
            'data_action_args' => [
              'user_id' => $user->ID
            ]
          ],
          'authentication' => [
            'title'            =>  __( "Authentication", "wpep" ),
            'type'             => 'data-view-analyze',
            'data_action'      => 'wpep/system/user/authentication-activity-report',
            'data_action_args' => [
              'user_id' => $user->ID
            ]
          ],
        ] + $analyze_activity_posts
      ]
    ];

    $request_data[ 'start_time' ] = ( is_numeric( $user->user_registered ) ? $user->user_registered : strtotime( $user->user_registered ) );

    return [
        'status'            => 'ok',
        'report_data_sources' => $report_data_sources,
      ] + API_Utils_Report::configuration_for_statistics( $request_data );
  }

  /**
   * ACL Methods are handled by app/App.php
   * @param $request_data
   * @return bool|array
   */
  public static function post_id_configuration_report( $request_data ) {
    if( !isset( $request_data[ 'post_id' ] ) )
      return false;

    $post = get_post( intval( $request_data[ 'post_id' ] ) );

    if( false === $post )
      return false;

    $report_data_sources = [
      'overview' => [
        'title'            =>  __( "Activity Report", "wpep" ),
        'type'             => 'data-view-analyze',
        'data_action'      => 'wpep/system/user/post-id-activity-report',
        'data_action_args' => [
          'post_id' => $post->ID
        ]
      ],
      'post-id-activity-list' => [
        'title'            =>  __( "Activity List", "wpep" ),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/system/user/post-id-activity-list',
        'data_action_args' => [
          'post_id' => $post->ID
        ]
      ]
    ];

    $request_data[ 'start_time' ] = ( is_numeric( $post->post_date_gmt ) ? $post->post_date_gmt : strtotime( $post->post_date_gmt ) );

    return [
        'status'            => 'ok',
        'report_data_sources' => $report_data_sources,
      ] + API_Utils_Report::configuration_for_statistics( $request_data );
  }

  public static function user_id_post_id_configuration_report( $request_data ) {
    if( !isset( $request_data[ 'post_id'] ) || !isset( $request_data[ 'user_id'] ) )
      return false;

    $request_data[ 'post_id' ] = intval( $request_data[ 'post_id'] );

    $post = get_post( intval( $request_data[ 'post_id'] ) );

    if( !wpep_content_type()->is_enabled( $post->post_type ) || !in_array( $post->post_status, wpep_post_monitored_post_status_list() ) )
      return false;

    $request_data[ 'user_id' ] = intval( $request_data[ 'user_id' ] );

    $report_data_sources = [
      'single-activity-report' => [
        'title'            =>  __( "Activity Report", "wpep" ),
        'type'             => 'data-view-analyze',
        'data_action'      => 'wpep/system/user/user-id-post-id-activity-report',
        'data_action_args' => [
          'post_id' => $post->ID,
          'user_id' => $request_data[ 'user_id']
        ]
      ],
      'action-tracking-list' => [
        'title'            =>  __( "Action Tracking - List", "wpep" ),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/system/user/user-id-post-id-action-tracking-list',
        'data_action_args' => [
          'post_id' => $post->ID,
          'user_id' => $request_data[ 'user_id']
        ]
      ]
    ];

    $request_data[ 'start_time' ] = ( is_numeric( $post->post_date_gmt ) ? $post->post_date_gmt : strtotime( $post->post_date_gmt ) );

    return [
        'status'            => 'ok',
        'report_data_sources' => $report_data_sources,
      ] + API_Utils_Report::configuration_for_statistics( $request_data );
  }

}