<?php

namespace WPEP\API\User;

use WPEP\API\Utils\User as API_Utils_User;
use WP_User_Query;

class AutoLogin {

  public static function token_entries( $request_data ) {
    $pagination_limit = 20;
    $current_page     = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );

    $user_search = new WP_User_Query( [
      'role__not_in'  => [ WPEP_USER_ROLE_AUDIENCE_MANAGER, WPEP_USER_ROLE_PLATFORM_MANAGER, 'administrator' ],
      'number'        => $pagination_limit,
      'paged'         => $current_page
    ] );

    $entries_user_objects = wpep_utilities_map_object( (array) $user_search->get_results(), 'ID' );
    $total_entries = $user_search->get_total();
    $entries = [];

    $salt           = wpep_get_setting( 'auto-login-link-salt' );
    $url_identifier = wpep_get_setting( 'auto-login-link-token' );
    $site_url       = get_site_url();

    foreach( $entries_user_objects as $entries_user_object ) {
      if( user_can( $entries_user_object, WPEP_CAP_MANAGE_SETTINGS ) ) {
        $user_token = '-';
      } else {
        $user_token = get_user_meta( $entries_user_object->ID, WPEP_USER_META_AUTO_LOGIN_TOKEN, true );

        if( $user_token === '' ) {
          $user_token = sha1( $salt . wpep_generate_random_token( 32 ) );

          update_user_meta( $entries_user_object->ID, WPEP_USER_META_AUTO_LOGIN_TOKEN, $user_token );
        }
      }

      $entries[] = [
        'user_id'           => $entries_user_object->ID,
        'auto_login_token'  => $user_token,
        'auto_login_home'   => ( $user_token === '-' ? '-' : add_query_arg( $url_identifier, $user_token, $site_url ) ),
      ];
    }

    wpep_set_setting( 'auto-login-link-last-generate', time() );

    return API_Utils_User::entries( [
      'status'          =>  'ok',
      'entries_header'  => [
        'auto_login_token'  => __( "Auto Login Token", "wpep" ),
        'auto_login_home'   => __( "Home Auto-Login URL", "wpep" )
      ],
      'entries'         => $entries,
      'pagination'      => [
        'current'       => $current_page,
        'total'         => ceil( $total_entries / $pagination_limit ),
        'total_entries' => $total_entries
      ]
    ], $request_data, $entries_user_objects );
  }

  public static function token_reset( $request_data ) {
    wpep_wpdb()->delete( wpep_wpdb()->usermeta, [
      'meta_key'  => WPEP_USER_META_AUTO_LOGIN_TOKEN
    ] );

    wpep_set_setting( 'auto-login-link-last-generate', 0 );

    return true;
  }

}