<?php

namespace WPEP\API\Utils;

use WPEP\ACL\User as WPEP_ACL_User;
use WPEP\ACL\Post as WPEP_ACL_Post;

class Post {

  /**
   * @param [] $response
   * @param [] $request_data
   * @return mixed
   */
  public static function entries( $response, $request_data ) {
    if( !isset( $response[ 'entries' ] ) || empty( $response[ 'entries' ] ) )
      return $response;

    $has_user_id = false;

    foreach( $response[ 'entries' ] as $entry ) {
      if( isset( $entry[ 'user_id' ] ) ) {
        $has_user_id = true;
        break;
      }
    }

    $single_post_id = isset( $request_data[ 'post_ids' ] ) && is_numeric( $request_data[ 'post_ids' ] );

    if( !$has_user_id && $single_post_id )
      return $response;

    $has_sid_support = wpep_has_sid_support();
    $has_cid_identifier = false;
    $has_cid_identifier_version = false;

    foreach( $response[ 'entries' ] as $entry_index => $entry ) {

      if( !$single_post_id ) {
        $attach_data = [
          'post_title' => [
            'label'          => get_the_title( $entry[ 'post_id' ] ),
            'type'           => 'component_trigger',
            'trigger'        => 'analytics_attach',
            'args' => [
              'component'       => 'analytics-panel',
              'component_title' => get_the_title( $entry[ 'post_id' ] ),
              'post_id'     => $entry[ 'post_id' ],
              'action'      => 'wpep/system/statistics/post-id-configuration-report'
            ]
          ],
        ];

        if( $has_sid_support && get_post_type( $entry[ 'post_id' ] ) === WPEP_POST_TYPE_COURSE ) {
          $attach_data[ 'wpep_post_cid_identifier' ] = get_post_meta( $entry[ 'post_id'], 'wpep_course_identifier', true );
          $attach_data[ 'wpep_post_cid_identifier_version' ] = get_post_meta( $entry[ 'post_id'], 'wpep_course_identifier_version', true );

          if( $attach_data[ 'wpep_post_cid_identifier' ] !== '' )
            $has_cid_identifier = true;

          if( $attach_data[ 'wpep_post_cid_identifier_version' ] !== '' )
            $has_cid_identifier_version = true;
        }

        $response[ 'entries' ][ $entry_index ] = $attach_data + $response[ 'entries' ][ $entry_index ];
      }

      if( !$has_user_id )
        continue;

      if( !isset( $entry[ 'user_id' ] ) ) {

        if( !WPEP_ACL_Post::analytics( $entry[ 'post_id' ], 'view' ) && !WPEP_ACL_User::analytics( $entry[ 'user_id' ], 'view' ) )
          $response[ 'entries' ][ $entry_index ][ 'view_user_ct_data' ] = '-';
        continue;
      }

      $response[ 'entries' ][ $entry_index ][ 'view_user_ct_data' ] = [
        'label'          => __( "Analyze", "wpep" ),
        'type'           => 'component_trigger',
        'trigger'        => 'analytics_attach',
        'button_type'    => 'primary',
        'args' => [
          'component'       => 'analytics-panel',
          'component_title' => sprintf(
            __( "Report : %s - %s", "wpep" ),
            wpep_content_user_profile_format_name( $entry[ 'user_id' ] ),
            get_the_title( $entry[ 'post_id' ] )
          ),
          'component_img'   => wpep_user_profile_picture_url( $entry[ 'user_id' ], 48 ),
          'action'          => 'wpep/system/statistics/user-id-post-id-configuration-report',
          'post_id'         => $entry[ 'post_id' ],
          'user_id'         => $entry[ 'user_id' ]
        ]
      ];
    }

    if( !$single_post_id ) {
      $attach_data = [];

      if( $has_cid_identifier )
        $attach_data[ 'wpep_post_cid_identifier' ] = __( "CID", "wpep" );

      if( $has_cid_identifier_version )
        $attach_data[ 'wpep_post_cid_identifier_version' ] = __( "CID Version", "wpep" );

      $attach_data[ 'post_title' ] = __( "Post Title", "wpep" );

      $response[ 'entries_header' ] = $attach_data + $response[ 'entries_header' ];
    }

    if( $has_user_id )
      $response[ 'entries_header' ][ 'view_user_ct_data' ] = __( "Analyze", "wpep" );

    return $response;
  }

}