<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Controller as WPEP_Controller;

class EmailMarketingCourseTags {

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_email_marketing_course_tags', [ $this, '_handle_request' ] );
  }

  public function _handle_request() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    if( !wpep_controller()->emailMarketing->has_text_based_tag_integrations()  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "No text based tagging Email Marketing integration is available.", "wpep" ) . '</div>');

    if( isset( $_POST[ 'wpep_emct_source' ] ) ) {
      $source            = sanitize_text_field( $_POST[ 'wpep_emct_source' ] );
      $tag_prefix        = sanitize_text_field( $_POST[ 'wpep_emct_tags_prefix' ] );
      $override_existent = ( isset( $_POST[ 'wpep_emct_override_existent' ] ) ? intval( $_POST[ 'wpep_emct_override_existent' ] ) : 0 );
      $retroactive       = ( isset( $_POST[ 'wpep_emct_retroactive' ] ) ? intval( $_POST[ 'wpep_emct_retroactive' ] ) : 0 );
      $course_tag_verification_completed  = ( isset( $_POST[ 'wpep_emct_course_tag_verification_completed' ] ) ? intval( $_POST[ 'wpep_emct_course_tag_verification_completed' ] ) : 0 );

      $emailMarketingIntegration = WPEP_Controller::instance()->emailMarketing->get_integration( $source );

      if( $emailMarketingIntegration === false )
        exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Invalid Email Marketing Integration provided.", "wpep" ) . '</div>');

      if( $retroactive && $course_tag_verification_completed ) {
        $is_completed = $this->_handle_retroactive_tagging( $emailMarketingIntegration );

        if( !$is_completed ) {
          $this->_output_request_request_information( $source, $tag_prefix, $override_existent, $retroactive, $course_tag_verification_completed  );

          exit;
        }

        echo '<div class="wpep-title-container">';
        echo    '<h3 class="wpep-title">' . __( "Completed Quick Tag Setup for Courses & Retroactive Tagging", "wpep" ) . '</h3>';
        echo    '<input class="wpep-button wpep-button-primary" 
                        type="submit" 
                        data-switch-sync-step="dynamic"
                        value="' . __( "Start Over", "wpep" ) . '"/>';
        echo '</div>';

        exit;
      }

      $course_tag_verification_completed = $this->_handle_course_started_and_completed_tags( $emailMarketingIntegration, $tag_prefix, $override_existent );

      if( !$course_tag_verification_completed ) {
        $this->_output_request_request_information( $source, $tag_prefix, $override_existent, $retroactive, $course_tag_verification_completed );

        exit;
      }

      if( $course_tag_verification_completed && !$retroactive ) {
        echo '<div class="wpep-title-container">';
        echo    '<h3 class="wpep-title">' . __( "Completed Quick Tag Setup for Courses", "wpep" ) . '</h3>';
        echo    '<input class="wpep-button wpep-button-primary" 
                          type="submit" 
                          data-switch-sync-step="dynamic"
                          value="' . __( "Start Over", "wpep" ) . '"/>';
        echo '</div>';

        exit;
      } else {
        $this->_output_request_request_information( $source, $tag_prefix, $override_existent, $retroactive, $course_tag_verification_completed );

        echo '<div class="wpep-notification wpep-notification-success">' . __( "Completed Quick Tag Setup for Courses, preparing to tag / re-tag students.", "wpep" ) . '</div>';

        exit;
      }
    }

    $this->_display_dashboard();

    exit;
  }

  private function _output_request_request_information( $source, $tag_prefix, $override_existent, $retroactive, $course_tag_verification_completed  ) {
    echo '<input type="hidden" name="wpep_emct_source" value="' . $source . '"/>';
    echo '<input type="hidden" name="wpep_emct_tags_prefix" value="' . $tag_prefix . '"/>';
    echo '<input type="hidden" name="wpep_emct_override_existent" value="' . $override_existent . '"/>';
    echo '<input type="hidden" name="wpep_emct_retroactive" value="' . $retroactive . '"/>';
    echo '<input type="hidden" name="wpep_emct_course_tag_verification_completed" value="' . $course_tag_verification_completed . '"/>';
  }

  private function _display_dashboard() {
    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Configure Quick Setup", "wpep" ) . '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Start", "wpep" ) . '"/>';
    echo '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="wpep-ph-ds-emct-source">' . __( "Source", "wpep" ) . '</label>';
    echo    '<select id="wpep-ph-ds-emct-source" name="wpep_emct_source">';
    foreach( wpep_controller()->emailMarketing->get_text_based_tag_integrations() as $text_based_email_marketing_integration )
      echo      '<option value="' . $text_based_email_marketing_integration->get_alias() . '">' . $text_based_email_marketing_integration->get_name() . '</option>';
    echo    '</select>';
    echo  '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="wpep-ph-ds-emct-tags-prefix">' . __( "Tag Prefix", "wpep" ) . '</label>';
    echo    '<input id="wpep-ph-ds-emct-tags-prefix"
                    type="text" 
                    name="wpep_emct_tags_prefix" 
                    class="regular-text" 
                    value="' . wpep_label_to_prefix( get_bloginfo( 'name' ) ) . '"/>';

    echo    '<div class="wpep-description">' . __( "Tags will be set on the following pattern : [tag-prefix]-[course-slug]-[started|completed].", "wpep" ) . '</div>';
    echo  '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="wpep-ph-ds-emct-override-existent">' . __( "Mode", "wpep" ) . '</label>';

    echo    '<select id="wpep-ph-ds-emct-override-existent" name="wpep_emct_override_existent">';
    echo      '<option value="0">' . __( "New Only - if there's existent tags for courses, pattern based ones will not be set.", "wpep" ) . '</option>';
    echo      '<option value="1">' . __( "All - if there's existent tags for courses, pattern based ones will be set.", "wpep" ) . '</option>';
    echo    '</select>';
    echo  '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="wpep-ph-ds-emct-retroactive">' . __( "Retroactive", "wpep" ) . '</label>';

    echo    '<select id="wpep-ph-ds-emct-retroactive" name="wpep_emct_retroactive">';
    echo      '<option value="0">' . __( "No - students that have already started or completed the course will not be tagged.", "wpep" ) . '</option>';
    echo      '<option value="1">' . __( "Yes - students that have already started or completed the course will be tagged / re-tagged.", "wpep" ) . '</option>';
    echo    '</select>';

    echo    '<div class="wpep-description">';
    echo      __( "Retroactive tagging will happen once all the courses have been verified.", "wpep" );
    echo    '</div>';
    echo  '</div>';
  }

  private function _handle_course_started_and_completed_tags( $emailMarketingIntegration, $tag_prefix, $override_existent ) {
    $course_ids        = ( isset( $_POST[ 'wpep_emct_course_ids'] )
      ? array_map( 'absint', explode( ',', $_POST[ 'wpep_emct_course_ids'] ) )
      : get_posts( [
        'numberposts' => -1,
        'post_type'   => WPEP_POST_TYPE_COURSE,
        'post_status' => wpep_post_monitored_post_status_list(),
        'fields'      => 'ids'
      ] ) );

    $table_data = [];

    $request_remaining_entries = 5; foreach( $course_ids as $course_id_index => $course_id ) {
      if( $request_remaining_entries <= 0 )
        break;

      $course_information = get_post( $course_id );

      if( $course_information !== null ) {
        $post_identifier = ( $course_information->post_name !== '' ? $course_information->post_name : wpep_label_to_alias( $course_information->post_title ) );

        $current_table_data = [
          'course_title'         => $course_information->post_title,
          'course_started_tag'   => '',
          'course_completed_tag' => ''
        ];

        $dynamic_started_tag   = $tag_prefix . '-' . $post_identifier . '-started';
        $dynamic_completed_tag = $tag_prefix . '-' . $post_identifier . '-completed';

        $current_post_meta_started   = get_post_meta( $course_id, $emailMarketingIntegration->post_meta_started, true );
        $current_post_meta_completed = get_post_meta( $course_id, $emailMarketingIntegration->post_meta_completed, true );

        if( $current_post_meta_started === '' || $override_existent )
          update_post_meta( $course_id, $emailMarketingIntegration->post_meta_started, $dynamic_started_tag );

        if( $current_post_meta_started === '' )
          $current_table_data[ 'course_started_tag' ] = $dynamic_started_tag;
        elseif( $override_existent )
          $current_table_data[ 'course_started_tag' ] = $dynamic_started_tag;
        else
          $current_table_data[ 'course_started_tag' ] = $current_post_meta_started;

        if( $current_post_meta_completed === '' || $override_existent )
          update_post_meta( $course_id, $emailMarketingIntegration->post_meta_completed, $dynamic_completed_tag );

        if( $current_post_meta_started === '' )
          $current_table_data[ 'course_completed_tag' ] = $dynamic_completed_tag;
        elseif( $override_existent )
          $current_table_data[ 'course_completed_tag' ] = $dynamic_completed_tag;
        else
          $current_table_data[ 'course_completed_tag' ] = $current_post_meta_started;

        $table_data[] = $current_table_data;
      }

      unset( $course_ids[ $course_id_index ] );

      $request_remaining_entries--;
    }

    if( empty( $course_ids ) )
      return true;

    echo '<textarea style="display:none !important;" name="wpep_emct_course_ids">' . implode(',', $course_ids ). '</textarea>';

    echo '<div class="wpep-notification wpep-notification-info">' . sprintf( __( "%s remaining courses to verify & set the tags for", "wpep" ), count( $course_ids ) ) . '</div>';

    wpep_admin_display_table(
      false,
      [
        'course_title'         => __( "Course Title", "wpep" ),
        'course_started_tag'   => __( "Course Started", "wpep" ),
        'course_completed_tag' => __( "Course Completed", "wpep" )
      ],
      $table_data
    );

    return false;
  }

  /**
   * @param \WPEP_Integration_Email_Marketing $emailMarketingIntegration
   * @return bool
   */
  private function _handle_retroactive_tagging( $emailMarketingIntegration ) {
    $user_ids        = ( isset( $_POST[ 'wpep_emct_user_ids'] )
      ? array_map( 'absint', explode( ',', $_POST[ 'wpep_emct_user_ids'] ) )
      : get_users( [ 'paged' => -1, 'orderby' => 'ID', 'fields' => 'ID' ] ) );

    if( empty( $user_ids ) )
      return true;

    $current_user_id = array_shift( $user_ids );

    $current_user_started_courses_ids = wpep_content_type_course_instance()->user->get_started_course_list( $current_user_id );

    while( empty( $current_user_started_courses_ids ) && !empty( $user_ids ) ) {
      $current_user_id = array_shift( $user_ids );

      $current_user_started_courses_ids = wpep_content_type_course_instance()->user->get_started_course_list( $current_user_id );
    }

    $current_user_completed_courses_ids = wpep_content_type_course_instance()->user->get_completed_course_list( $current_user_id );

    $user_tags = [];
    $table_data = [];

    foreach( $current_user_started_courses_ids as $current_user_started_courses_id ) {
      $current_tag = get_post_meta(
        $current_user_started_courses_id,
        ( in_array( $current_user_started_courses_id, $current_user_completed_courses_ids )
          ? $emailMarketingIntegration->post_meta_completed
          : $emailMarketingIntegration->post_meta_started
        ),
        true
      );

      if( $current_tag !== '' ) {
        $user_tags[] = $current_tag;
        $table_data[] = [
          'tag' => $current_tag
        ];
      }
    }

    if( !empty( $user_tags ) )
      $emailMarketingIntegration->add_tag( $user_tags, $current_user_id );

    if( empty( $user_ids ) )
      return true;

    echo '<textarea style="display:none !important;" name="wpep_emct_user_ids">' . implode(',', $user_ids ). '</textarea>';

    echo '<div class="wpep-notification wpep-notification-success">';
    echo    sprintf( __( "Set %s tags for %s, email address : %s", "wpep" ),
                     count( $user_tags ),
                     wpep_content_user_profile_format_name( $current_user_id ),
                     get_user_by( 'ID', $current_user_id )->user_email
                    );
    echo '</div>';
    echo '<div class="wpep-notification wpep-notification-info">' . sprintf( __( "%s remaining students to tag / re-tag", "wpep" ), count( $user_ids ) ) . '</div>';

    wpep_admin_display_table(
      false,
      [
        'tag'         => __( "Tag", "wpep" )
      ],
      $table_data
    );

    return false;
  }

}
