<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

class PlatformHealth {

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_platform_health', [ $this, 'handle_request' ] );
  }

  public function handle_request() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    if( isset( $_POST[ 'sync_stage' ] ) ) {
      $sync_stage = $_POST[ 'sync_stage' ];

      if( $sync_stage === 'update_licenses' && isset( $_POST[ 'wpep_license_management' ] ) ) {

        foreach( $_POST[ 'wpep_license_management' ] as $license_product_alias => $license_key ) {
          if( $license_key === '' )
            continue;

          $integration = wpep_controller()->licenseLibrary->get_integration( $license_product_alias );

          if( false === $integration )
            continue;

          $license_key = sanitize_text_field( $license_key );

          $valid = $integration->activate_license( $license_key );

          if( $valid )
            echo '<div class="wpep-notification wpep-notification-success">' . sprintf( __( "%s has been licensed using %s", "wpep" ), $integration->get_product_name(), $license_key ) . '</div>';
          else
            echo '<div class="wpep-notification wpep-notification-danger">' . sprintf( __( "%s is not a valid license for %s", "wpep" ), $license_key, $integration->get_product_name() ) . '</div>';
        }
      } else {
        $sync_stage = explode( "__", $sync_stage );

        if( in_array( $sync_stage[ 0 ], [ 'service', 'maintenance' ] ) ) {
          $report_data = ( $sync_stage[ 0 ] === 'service' ? wpep_platform_health()->service->get_report() : wpep_platform_health()->maintenance->get_report() );

          if( isset( $report_data[ $sync_stage[ 1 ] ] ) && isset( $report_data[ $sync_stage[ 1 ] ][ 'operation'] ) && is_callable( $report_data[ $sync_stage[ 1 ] ][ 'operation'] ) ) {
            $system_log = WPEP_Entity_SystemLog::instance( 'platform_health_' . $sync_stage[ 0 ], $sync_stage[ 1 ], time() );

            $response = call_user_func( $report_data[ $sync_stage[ 1 ] ][ 'operation'] );

            if( is_array( $response ) ) {
              $system_log->completed_arguments = $response;

              if( isset( $response[ 'notification' ] ) )
                echo '<div class="wpep-notification wpep-notification-' . ( isset( $response[ 'notification_type' ] ) ? $response[ 'notification_type' ] : 'info' ) . '">' . $response[ 'notification' ] . '</div>';
            }

            $system_log->completed_at = time();
            $system_log->save();
          }
        }
      }

      wpep_platform_health()->service->reset_notification_count_transient();
    }

    wpep_template_load( 'admin/platform-health/data-system-platform-health.php' );
    exit;
  }

}
