<?php

namespace WPEP\Admin\Coordinator\DataSystem;

class ThirdPartyImport {

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_third_party_import', [ $this, 'import_process' ] );
  }

  public function import_process() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    if( !wpep_data_system()->is_third_party_import_available()  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "No Third-Party Import is available.", "wpep" ) . '</div>');

    $active_importer = false;

    foreach( wpep_data_system()->get_third_party_import_instances() as $instance ) {
      $instance->setup();

      if( $instance->import_available() && $instance->get_import_status() !== 'pending' ) {
        $active_importer = $instance;
        break;
      }
    }

    if( $active_importer !== false ) {
      if( $active_importer->get_import_status() === 'processing' || ( isset( $_POST[ 'confirm_third_party_import' ] ) && intval( $_POST[ 'confirm_third_party_import' ] ) ) ) {
        $active_importer->import_process();

        $importer_que = $active_importer->get_que();
        $importer_map = $active_importer->get_import_map();

        if( !is_array( $importer_que ) || count( $importer_que ) === 0 ) {
          echo '<div class="wpep-title-container">';
          echo    '<h3 class="wpep-title">' . __( "Import Completed", "wpep" ) . '</h3>';
          echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "New Import", "wpep" ) . '"/>';
          echo '</div>';

          $table_preview = $active_importer->implementation_import_success_preview_table_data( $importer_map );

          if( isset( $table_preview[ 'head' ] ) && isset( $table_preview[ 'body' ] ) && !empty( $table_preview[ 'body' ] ) ) {
            wpep_admin_display_table(
              false,
              $table_preview[ 'head' ],
              $table_preview[ 'body' ]
            );
          }

          $active_importer->import_end();

          exit;
        }

        echo '<div class="wpep-notification wpep-notification-success">' .
                $active_importer->implementation_import_progressing_notification_text( $importer_map, $importer_que ) .
             '</div>';

        exit;
      }

      $this->_display_import_preview( $active_importer );

      exit;
    }

    if( isset( $_POST[ 'wpep_import_source' ] ) && isset( $_POST[ 'wpep_configuration' ][ $_POST[ 'wpep_import_source' ] ] ) ) {
      $active_importer  = wpep_data_system()->get_third_party_import_instance( $_POST[ 'wpep_import_source' ] );

      if( $active_importer === false )
        exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Invalid Request.", "wpep" ) . '</div>');

      $active_importer->import_start();
      $active_importer->set_preferences_field_map( $_POST[ 'wpep_configuration' ][ $_POST[ 'wpep_import_source' ] ] );
      $active_importer->sync_settings();

      $this->_display_import_preview( $active_importer );

      exit;
    }

    $this->_display_dashboard();

    exit;
  }

  private function _display_import_preview( $active_importer = false ) {
    if( $active_importer === false )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Invalid Request.", "wpep" ) . '</div>');

    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Step 2: Confirm Import", "wpep" ) . '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Start Import", "wpep" ) . '"/>';
    echo    '<input type="hidden" name="confirm_third_party_import" value="1"/>';
    echo '</div>';

    echo $active_importer->implementation_import_confirmation_content();
  }

  private function _display_dashboard() {
    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Step 1: Configure Import", "wpep" ) . '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Preview Import", "wpep" ) . '"/>';
    echo '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="wpep-ph-ds-tpm-source">' . __( "Source", "wpep" ) . '</label>';
    echo    '<select id="wpep-ph-ds-tpm-source" name="wpep_import_source" data-wpep-conditional-display="data-wpep-ph-ds-tpm-source">';
    foreach( wpep_data_system()->get_third_party_import_instances() as $importer_instance ) {
      if( empty( $importer_instance->import_available() ) )
        continue;

      echo      '<option value="' . $importer_instance->get_alias() . '">' . $importer_instance->get_name() . '</option>';
    }
    echo    '</select>';
    echo  '</div>';

    foreach( wpep_data_system()->get_third_party_import_instances() as $importer_instance ) {
      foreach( $importer_instance->get_preferences_field_map() as $field_key => $field_information ) {
        echo  '<div class="wpep-utility-data-sync-row" data-wpep-ph-ds-tpm-source="' . $importer_instance->get_alias() . '">';
        echo    '<label for="wpep-' . $importer_instance->get_alias() . '-' . $field_key . '">' . $field_information['name'] . '</label>';

        if( $field_information['type'] === "text" ) {
          echo    '<input id="wpep-' . $importer_instance->get_alias() . '-' . $field_key . '"
                          type="text" 
                          name="wpep_configuration[' . $importer_instance->get_alias() . '][' . $field_key . ']" 
                          class="regular-text" 
                          value="' . $importer_instance->preferences_import_map[ $field_key ] . '"/>';
        } elseif( $field_information['type'] === "select" ) {
          echo    '<select id="#wpep-' . $importer_instance->get_alias() . '-' . $field_key . '" 
                           name="wpep_configuration[' . $importer_instance->get_alias() . '][' . $field_key . ']">';

          foreach( $field_information[ 'options' ] as $option_key => $option_value )
            echo      '<option value="' . $option_key . '">' . $option_value . '</option>';

          echo    '</select>';
        } else {
          echo '<div class="wpep-notification wpep-notification-danger">' . __( "Field Type not supported", "wpep" ) . '</div>';
        }

        if( isset( $field_information['description'] ) )
          echo '<div class="wpep-description">' . $field_information['description'] . '</div>';

        echo  '</div>';
      }
    }
  }

}
