<?php

namespace WPEP\Admin\Coordinator;

class FieldLibrary {

  public $post;
  public $term;

  public function __construct() {
    $this->post = new FieldLibrary\Post();
    $this->term = new FieldLibrary\Term();
  }

  public function setup() {
    $this->post->setup();
    $this->term->setup();
  }

  /**
   * @param \WPEP\Container\FormFields $containerConfigurationInstance
   * @param $field_list
   */
  public function display_fields( $containerConfigurationInstance, $field_list ) {
    $remaining_fields = count( $field_list );

    echo '<div class="wpep-meta-box-container">';

    foreach( $field_list as $field_information ) {
      echo '<div data-wpep-field-type-container="' . $field_information['type'] . '">';

      $input_name = isset( $field_information[ 'input_name' ] ) ? $field_information[ 'input_name' ] : false;

      if( isset( $field_information[ 'content_before' ] ) )
        echo $field_information[ 'content_before' ];

      if( $field_information['type'] == 'checkbox' ) {
        $containerConfigurationInstance->display_checkbox_field(
          $field_information['id'],
          null,
          $input_name,
          isset( $field_information['default'] ) ? $field_information['default'] : 0
        );
      }

      if( isset( $field_information['name'] ) )
        echo '<p class="label">' . $field_information['name'] . '</p>';

      if( isset( $field_information['top_description'] ) || isset( $field_information['top_desc'] ) )
        echo '<div class="description">' . wpautop( isset( $field_information['top_description'] ) ? $field_information['top_description'] : $field_information['top_desc'] ) . '</div>';

      if( $field_information['type'] == 'radio_inline' ) {
        $containerConfigurationInstance->display_radio_fields(
          $field_information['id'],
          isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'],
          isset( $field_information['default'] ) ? $field_information['default'] : 0
        );
      } else if( $field_information['type'] == 'select' ) {
        $containerConfigurationInstance->display_select_field(
          $field_information['id'],
          isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'],
          $input_name,
          null,
          isset( $field_information['class'] ) ? $field_information['class'] : 'wpep-select-field',
          isset( $field_information['default'] ) ? $field_information['default'] : '',
          isset( $field_information['attributes'] ) ? $field_information['attributes'] : []
        );
      } else if( $field_information['type'] == 'multiple_select' ) {
        $containerConfigurationInstance->display_multiple_select_field(
          $field_information['id'],
          isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'],
          $input_name,
          null,
          'wpep-select-field',
          isset( $field_information['attributes'] ) ? $field_information['attributes'] : []
        );
      } else if( $field_information['type'] == 'wysiwyg' || $field_information['type'] == 'editor' ) {
        $containerConfigurationInstance->display_editor_field( $field_information['id'] );
      } else if( $field_information['type'] == 'textarea' ) {
        $containerConfigurationInstance->display_textarea_field(
          $field_information['id'],
          isset( $field_information['input_name'] ) ? $field_information['input_name'] : false,
          isset( $field_information['height'] )     ? $field_information['height']     : false
        );
      } else if( $field_information['type'] == 'colorpicker' || $field_information['type'] == 'input_color' ) {
        $containerConfigurationInstance->display_color_field( $field_information['id'] );
      } else if( $field_information['type'] == 'image' ) {
        $containerConfigurationInstance->display_image_field( $field_information['id'], $field_information['name'] );
      } else if( $field_information['type'] == 'file' ) {
        $containerConfigurationInstance->display_file_field( $field_information['id'] );
      } else if( $field_information['type'] == 'numeric' ) {
        $containerConfigurationInstance->display_numeric_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['class'] ) ? $field_information['class'] : false,
          isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : ''
        );
      } else if( $field_information['type'] == 'input' || $field_information['type'] == 'text' ) {
        $containerConfigurationInstance->display_text_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['class'] ) ? $field_information['class'] : false,
          isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : '',
          isset( $field_information['data'] ) ? $field_information['data'] : [],
          isset( $field_information['default'] ) ? $field_information['default'] : ''
        );
      } else if( $field_information['type'] == 'date' ) {
        $containerConfigurationInstance->display_date_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['class'] ) ? $field_information['class'] : false,
          isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : '',
          isset( $field_information['data_attributes'] ) ? $field_information['data_attributes'] : []
        );
      } else if( $field_information['type'] == 'attributes' ) {
        $containerConfigurationInstance->display_attributes_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : []
        );
      } else if( $field_information['type'] == 'attributes_map' ) {
        $containerConfigurationInstance->display_attributes_map_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : [],
          isset( $field_information['map_labels'] ) ? $field_information['map_labels'] : [],
          isset( $field_information['map_options'] ) ? $field_information['map_options'] : []
        );
      } else if( $field_information['type'] == 'list' ) {
        $containerConfigurationInstance->display_list_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : []
        );
      } else if( $field_information['type'] == 'learning_journey' ) {
        $containerConfigurationInstance->display_select_learning_journey_sequence_field(
          $field_information['id'],
          $input_name,
          isset( $field_information['value'] ) ? $field_information['value'] : false
        );
      } else if( $field_information['type'] == 'attributes_field_filters' ) {
        $containerConfigurationInstance->display_attributes_field_filters(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : []
        );
      }

      if( isset( $field_information['description'] ) || isset( $field_information['desc'] ) )
        echo '<div class="description">' . wpautop( isset( $field_information['description'] ) ? $field_information['description'] : $field_information['desc'] ) . '</div>';

      if( $remaining_fields > 1 && $field_information['type'] != 'none' )
        echo '<hr/>';

      if( isset( $field_information[ 'content_after' ] ) )
        echo $field_information[ 'content_after' ];

      if( $field_information['type'] != 'none' )
        $remaining_fields--;

      echo '</div>';
    }

    echo '</div>';
  }

  /**
   * @param \WPEP\Container\FormFields $containerConfigurationInstance
   * @param $field_list
   */
  public function display_inner_table( $containerConfigurationInstance, $field_list ) {
    foreach( $field_list as $field_information ) {
      $current_table_row_attr = $containerConfigurationInstance->get_table_row_data_attributes();

      $label       = ( isset( $field_information['name'] ) ? $field_information['name'] : '' );
      $field_key   = ( isset( $field_information['id'] ) ? $field_information['id'] : ( isset( $field_information['alias'] ) ? $field_information['alias'] : '' ) );
      $description = ( isset( $field_information['description'] ) ? $field_information['description'] : ( isset( $field_information['desc'] ) ? $field_information['desc'] : '' ) );
      $description = ( is_string( $description ) ? $description : ( $description !== false && is_callable( $description ) ? call_user_func( $description ) : $description ) );

      if( isset( $field_information[ 'row_data_attributes' ] ) )
        $containerConfigurationInstance->set_table_row_data_attributes( $field_information[ 'row_data_attributes' ] );

      if( in_array( $field_information['type'], [ 'select', 'multiple_select', 'select_sequence' ] ) ) {
        $field_options = ( isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'] );
        $field_options = ( is_array( $field_options ) ? $field_options : ( is_callable( $field_options ) ? call_user_func( $field_options ) : [] ) );

        if( $field_information['type'] == 'select' ) {
          $containerConfigurationInstance->display_table_select_field(
            $label,
            $field_key,
            $field_options,
            $description,
            ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
            null,
            ( isset( $field_information['class'] ) ? $field_information['class'] : 'wpep-select-field' ),
            ( isset( $field_information['attributes'] ) ? $field_information['attributes'] : [] )
          );
        } else if( $field_information['type'] == 'select_sequence' ) {
          $containerConfigurationInstance->display_table_select_sequence_field( $label, $field_key, $description, false, [], $field_options );
        } else {
          $containerConfigurationInstance->display_table_multiple_select_field(
            $label,
            $field_key,
            $field_options,
            $description
          );
        }
      } else if( $field_information['type'] == 'checkbox' ) {
        $containerConfigurationInstance->display_table_checkbox_field(
          $label,
          $field_key,
          $description,
          null,
          ( isset( $field_information[ 'checkbox_force_checked'] ) ? $field_information[ 'checkbox_force_checked'] : null )
        );
      } else if( $field_information['type'] == 'checkbox_array' ) {
        $containerConfigurationInstance->display_table_checkbox_array_field(
          $label,
          $field_key,
          $field_information[ 'checkbox_value' ]
        );
      } else if( $field_information['type'] == 'wysiwyg' || $field_information['type'] == 'editor' ) {
        $containerConfigurationInstance->display_table_editor_field(
          $label,
          $field_key,
          $description
        );
      } else if( $field_information['type'] == 'editor_translated' ) {
        $containerConfigurationInstance->display_table_editor_translated_field(
          $label,
          $field_key,
          $description
        );
      } else if( $field_information['type'] == 'textarea' ) {
        $containerConfigurationInstance->display_table_textarea_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information['input_name'] ) ? $field_information['input_name'] : false ),
          ( isset( $field_information['height'] )     ? $field_information['height']     : false )
        );
      } else if( in_array( $field_information['type'], [ 'color', 'colorpicker', 'input_color' ] ) ) {
        $containerConfigurationInstance->display_table_color_field(
          $label,
          $field_key,
          $description
        );
      } else if( $field_information['type'] == 'image' || $field_information['type'] == 'file' ) {
        $containerConfigurationInstance->display_table_image_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          null,
          ( isset( $field_information[ 'image_css_attributes' ] ) ? $field_information[ 'image_css_attributes'] : [] ),
          ( isset( $field_information[ 'image_allow_remove' ] ) ? $field_information[ 'image_allow_remove'] : true )
        );
      } else if( $field_information['type'] == 'input' || $field_information['type'] == 'text' ) {
        $containerConfigurationInstance->display_table_text_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          ( isset( $field_information['value'] ) ? $field_information['value'] : null ),
          ( isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : null )
        );
      } else if( $field_information['type'] == 'numeric' ) {
        $containerConfigurationInstance->display_table_numeric_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          ( isset( $field_information['value'] ) ? $field_information['value'] : null ),
          ( isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : null )
        );
      } else if( $field_information['type'] == 'date' ) {
        $containerConfigurationInstance->display_table_date_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          isset( $field_information['value'] ) ? $field_information['value'] : null,
          isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : '',
          isset( $field_information['data_attributes'] ) ? $field_information['data_attributes'] : []
        );
      } else if( $field_information['type'] == 'text_translated'  ) {
        $containerConfigurationInstance->display_table_text_translated_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false )
        );
      } else if( $field_information['type'] == 'attributes' ) {
        $containerConfigurationInstance->display_table_attributes_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          null,
          ( isset( $field_information['default'] ) ? $field_information['default'] : [] )
        );
      } else if( $field_information['type'] == 'attributes_content_editor' ) {
        $containerConfigurationInstance->display_table_attributes_field_content_editor(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          null,
          ( isset( $field_information['default'] ) ? $field_information['default'] : [] )
        );
      } else if( $field_information['type'] == 'list' ) {
        $containerConfigurationInstance->display_table_list_field(
          $label,
          $field_key,
          $description,
          ( isset( $field_information[ 'field_name' ] ) ? $field_information[ 'field_name' ] : false ),
          [],
          ( isset( $field_information['default'] ) ? $field_information['default'] : [] )
        );
      } else if( $field_information['type'] == 'html' ) {
        $content = !is_string( $field_information['html'] ) && is_callable( $field_information['html'] ) ? call_user_func( $field_information['html'] ) : $field_information['html'];

        $containerConfigurationInstance->display_table_html( $content, ( isset( $field_information[ 'html_container_class' ] ) ? $field_information[ 'html_container_class' ] : '' ) );
      } else if( $field_information['type'] == 'html_separator' ) {
        $content = !is_string( $field_information['html'] ) && is_callable( $field_information['html'] ) ? call_user_func( $field_information['html'] ) : $field_information['html'];

        $containerConfigurationInstance->display_table_separator( $label, $content, ( isset( $field_information[ 'html_container_class' ] ) ? $field_information[ 'html_container_class' ] : '' ) );
      } else if( $field_information['type'] == 'content_ordering' ) {
        $field_options = ( is_array( $field_information['options'] ) ? $field_information['options'] : ( is_callable( $field_information['options'] ) ? call_user_func( $field_information['options'] ) : [] ) );

        $containerConfigurationInstance->display_table_content_ordering( $field_options, $field_information[ 'tab_base_url' ], $field_information[ 'tab_index' ], $field_information[ 'field_prefix' ] );
      } else if( $field_information[ 'type'] == 'wrapper_options' ) {
        $containerConfigurationInstance->display_table_wrapper_options();
      } else {
        $containerConfigurationInstance->display_table_separator(
          "<hr/>",
          sprintf( __( "Field Type %s is not available", "wpep" ), $field_information['type'] )
        );
      }

      if( !empty( $current_table_row_attr ) && isset( $field_information[ 'row_data_attributes' ] ) )
        $containerConfigurationInstance->set_table_row_data_attributes( $current_table_row_attr );
    }
  }

}