<?php

namespace WPEP\Admin\Coordinator\FieldLibrary;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FormFields as WPEP_FormFields;

/**
 * Class Post
 * @package WPEP\Admin\Coordinator
 */
class Term {

  use WPEP_FormFields;

  private $_term_id = 0;

  public function __construct() {
    $this->set_field_prefix( 'wpep_term_meta_value' );
  }

  public function setup() {
    add_action( WPEP_TAXONOMY_CATEGORY . '_add_form_fields', [ $this, '_form_action' ], 20 );
    add_action( WPEP_TAXONOMY_CATEGORY . '_edit_form_fields', [ $this, '_form_table_action' ], 20 );

    add_action( 'create_' . WPEP_TAXONOMY_CATEGORY, [ $this, '_save_action' ], 20 );
    add_action( 'edited_' . WPEP_TAXONOMY_CATEGORY, [ $this, '_save_action' ], 20 );
  }

  public function _form_action() {
    WPEP_Controller::instance()->adminController->fieldLibrary->display_fields( $this, $this->_get_field_information() );
  }

  public function _form_table_action( $term ) {
    $this->_term_id = ( isset( $term->term_id ) ? $term->term_id : 0 );

    WPEP_Controller::instance()->adminController->fieldLibrary->display_inner_table( $this, $this->_get_field_information() );
  }

  private function _get_field_information() {
    $information = [
      'icon'  => [
        'name'         => __( 'Icon', "wpep"),
        'description'  => __( 'Recommended size is : 100x100.', "wpep"),
        'id'           => 'icon',
        'type'         => 'image',
      ],
      'url'   => [
        'name'         => __( 'Link', "wpep" ),
        'description'  => __( 'If you want this category to redirect the user to a specific page, add a link here.', "wpep"),
        'id'           => 'url',
        'type'         => 'text'
      ],
    ];

    if( wpep_content_primary_has_category_card_labels() ) {

      $information['wpep_show_as_tag'] = [
        'name'    => __( 'Show as Tag', "wpep"),
        'desc'    => __( 'Display as Tag at the top of the Course Title, if this category is assigned.', "wpep"),
        'id'      => 'wpep_show_as_tag',
        'type'    => 'select',
        'options' => wpep_flag_setting_field_options()
      ];
      $information['wpep_tag_icon'] = [
        'name'         => __( 'Tag Icon', "wpep"),
        'description'  => __( 'Recommended size is 36x36, in some cases it will be displayed smaller.', "wpep"),
        'id'           => 'wpep_tag_icon',
        'type'         => 'image',
      ];

    }

    return $information;
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    $field_value = ( $this->_term_id === 0 ? ( $single ? '' : [] ) : get_term_meta( $this->_term_id, $field_key, $single ) );

    return $field_value == '' ? $field_default : $field_value;
  }

  public function _save_action( $term_id ) {
    if ( isset( $_REQUEST['action'] ) && $_REQUEST['action'] == 'inline-save-tax' )
      return;

    if( !isset( $_POST[ $this->field_prefix ] ) )
      return;

    $information = $_POST[ $this->field_prefix ];

    foreach( $information as $field_key => $field_value ) {
      if( is_array( $field_value ) ) {
        $single_meta = ( strpos( $field_key, '_' ) === 0 ? false : true );

        if( $single_meta ) {
          delete_term_meta( $term_id, $field_key );

          foreach( $field_value as $field_val ) {
            // We have our tricks here.
            if( $field_val == 'a:0:{}' )
              continue;

            add_post_meta( $term_id, $field_key, $field_val );
          }
          continue;
        }
      }

      if( $field_value === '' ) {
        delete_term_meta( $term_id, $field_key );

        continue;
      }

      update_term_meta( $term_id, $field_key, $field_value );
    }
  }

}