<?php

namespace WPEP\Container\Migration;

use WPEP\Controller as WPEP_Controller;

class Admin {

  /**
   * @var \WPEP\Container\Migration
   */
  public $controller;

  public function __construct( $controller ) {
    $this->controller = $controller;

    add_action( 'wp_ajax_wpep_migration_handler_' . $this->controller->namespace, [ $this, 'migrate_single' ] );

    WPEP_Controller::instance()->adminController->migration->register( $this );
  }

  public function _init() {
    add_action( 'admin_menu', [ $this, '_register_menu' ] );

    if( isset( $_GET['page'] ) && ( strpos($_GET['page'], 'wpep-migration-' ) === 0 ) )
      return;

    if( ( isset( $_GET['post_type'] ) && isset( wpep_content_type()->standardContentTypes[ $_GET['post_type']] ) )
        || ( isset( $_GET['post'] ) && isset( wpep_content_type()->standardContentTypes[ get_post_type( absint( $_GET['post'] ) ) ] ) )
        || ( isset( $_GET['page'] ) && strpos( $_GET['page'], 'wpep-' ) === 0 ) ) {
      wpep_redirect( wpep_admin_url( 'wpep-migration-' . $this->controller->namespace ) );
      exit;
    }

    add_action('admin_notices', [ $this, 'admin_notices' ] );
  }

  public function _register_menu() {
    wpep_admin_register_menu(
      sprintf( __("%s Migration", 'wpep'), WPEP_PLUGIN_NAME ),
      __( "Migration", 'wpep'),
      'manage_options',
      "wpep-migration-" . $this->controller->namespace,
      [ $this, "display" ]
    );
  }

  public function display() {
    $plugin_data = get_plugin_data( $this->controller->plugin_file_path );

    echo '<div class="wrap">';
    echo '<h2>' . sprintf( __( "%s Migration", 'wpep'), $this->controller->get_name() ) . '</h2>';
    echo '<hr/>';

    echo '<p>' . sprintf( __( "You're about to upgrade to %s version %s this version brings in database upgrades.", 'wpep' ),
                  $this->controller->get_name(),
                  isset( $plugin_data['Version'] ) ? $plugin_data['Version'] : WPEP_VERSION
                 ) .
         '</p>';

    echo '<p>' . __( "Please do not close this screen after pressing continue, until the update is completed.", "wpep" ) . '</p>';

    echo '<a class="wpep-button wpep-button-primary wpep-migration-continue">' . __( "Continue", "wpep" ) . '</a>';

    echo '<div id="migration-wpep-spinner-loader-container" style="display:none;">';
    echo    wpep_spinner_loader( __( "Processing..." ) );
    echo '</div>';

    echo '<div id="wpep-migration-helper-log"></div>';

    echo '</div>';

    echo '
      <script type="text/javascript">
        var WPEP_Migration = {
        
          processing : false,
        
          init : function() {
            var objectInstance = this;
            
            jQuery("#wpbody-content").on( "click", ".wpep-migration-continue", function() {
              jQuery(this).fadeOut("slow");
              
              objectInstance.migrateToNext();
            });
          },
          
          migrateToNext : function( request_data, last_response ) {
            if( this.processing === true )
              return;
            
            last_response = ( typeof last_response === "undefined" ? "" : last_response );
            
            jQuery("#migration-wpep-spinner-loader-container").slideDown("slow");
              
            var objectInstance = this;
            
            request_data = ( request_data !== undefined && request_data !== null && request_data.constructor === Object ? request_data : {} );
            request_data.action  = "wpep_migration_handler_' . $this->controller->namespace . '";
            request_data.attempt = ( typeof request_data.attempt !== "undefined" ? request_data.attempt : 0 );
            
            this.processing = true;
            
            jQuery.post(ajaxurl, request_data )
                  .done( function (response) {
                    objectInstance.processing = false;

                    if( last_response !== "" ) {
                      if( jQuery( "#wpep-migration-attempt-helper" ).length === 0 )
                        jQuery( "body" ).append( \'<div id="wpep-migration-attempt-helper" style="display: none !important;"></div>\' );
                      
                      var helperObject = jQuery( "#wpep-migration-attempt-helper" );
                      
                      helperObject.html( response );
                      helperObject.find( "[data-wpep-migration-attempt]" ).remove();
                      
                      var current_response_clean = helperObject.html();
                      
                      helperObject.html( last_response );
                      helperObject.find( "[data-wpep-migration-attempt]" ).remove();
                      
                      if( helperObject.html() === current_response_clean )
                        request_data.attempt++;
                      else 
                        request_data.attempt = 0;
                    }
                                                            
                    jQuery("#wpep-migration-helper-log").prepend( response ).show("slow");
                    
                    if( jQuery( response ).find(".wpep-migration-continue , .wpep-migration-complete").length === 0 && response.indexOf("wpep-migration-complete") === -1 )
                      objectInstance.migrateToNext( { 
                        attempt        : request_data.attempt
                      }, response );
                    else 
                      jQuery("#migration-wpep-spinner-loader-container").slideUp("slow");
                  } ).fail( function() {
                    objectInstance.processing = false;
                    
                    objectInstance.migrateToNext();
                  });
          }
        
        };
        
        jQuery( document ).ready( function() {
          WPEP_Migration.init();
        });
      </script>
    ';
  }

  public function admin_notices($hook) {

    $url = wpep_admin_url( 'wpep-migration-' . $this->controller->namespace );

    echo '<div class="notice notice-success is-dismissible">';
    echo    '<h2>' . sprintf(__( '%s Database Upgrade', "wpep" ), $this->controller->get_name() ) . '</h2>';
    echo    '<p>' . sprintf( 'Before you continue to use %s you need to migrate to the newest version.', $this->controller->get_name() ) . '</p>';
    echo    '<p><a class="button button-primary" href="' . $url .'">' . __( "Click here to continue", "wpep" ) . '</a></p>';
    echo '</div>';

  }

  public function migrate_single() {
    if( !current_user_can( 'manage_options') )
      exit;

    if( $this->controller->current_version >= $this->controller->current_version_available ) {
      echo '<p class="wpep-migration-complete">' . __( "Database successfully updated. Continue to : ", 'wpep' ) . '</p>';
      echo '<p>';
      echo    '<a href="' . wpep_admin_url_dashboard() . '" class="button button-primary">' . __( "Dashboard", "wpep" ) . '</a> ';
      echo    '<a href="' . wpep_admin_url_settings() . '" class="button button-primary">' . __( "Manage Settings", "wpep" ) . '</a> ';
      echo '</p>';
      exit;
    }

    $version = $this->controller->get_current_migration_version();

    if( $version === false ) {
      echo '<p class="wpep-notification wpep-notification-danger">' . __( "Could not determine the migration version.", "wpep" ). '</p>';
      echo '<p><a class="button button-primary wpep-migration-continue">' . __( "Retry", "wpep" ) . '</a></p>';
      exit;
    }

    $attempt = ( isset( $_POST[ 'attempt'] ) ? intval( $_POST[ 'attempt'] ) : 0 );

    echo '<p data-wpep-migration-version="' . $version . '">';
    echo    sprintf( __( "Processing File : %s", "wpep" ), $version );

    if( $attempt !== 0 ) {
      echo '<span data-wpep-migration-attempt="' . $attempt . '">';
      echo   '( ' . sprintf( __( "Retry Attempt : %s; Current Version : %s" , "wpep"), $attempt, $this->controller->current_version ) . ' )';
      echo '</span>';
    }
    echo '</p>';

    if( $attempt >= 5 ) {
      $this->controller->current_version = $version;
      delete_option( $this->controller->option_name );
      update_option( $this->controller->option_name, $version );

      echo '<p class="wpep-notification wpep-notification-danger">';
      echo    sprintf( __( "Skipped Migration file for version : %s, check your Platform Health Report for possible issues.", "wpep" ), $version );
      echo '</p>';
      exit;
    }

    echo $this->controller->migrate_to_version( $version );

    wpep_set_setting( 'configuration-setup-rules-flushed', 0, true );
    exit;
  }

}