<?php

namespace WPEP\ContentType;

use WPEP\API as WPEP_API;
use WPEP\ContentType\Course\Container\PostDependency as Course_PostDependency;
use WPEP\ContentType\Assessment\Container\Data as Assessment_Data;

class Assessment
  extends Skeleton\Standard
  implements Course_PostDependency {

  use Assessment_Data;

  /**
   * @var Assessment\Admin
   */
  public $admin;

  /**
   * @var Assessment\Block
   */
  public $block;

  /**
   * @var Assessment\Delivery
   */
  public $delivery;

  /**
   * @var Assessment\Privacy
   */
  public $privacy;

  /**
   * @var Assessment\User
   */
  public $user;

  protected $_current_directory;

  /**
   * @var \WPEP\ContentType\Assessment\Integration\Builder
   */
  protected $_builder_instance;

  public function __construct() {
    $this->_current_directory = dirname( __FILE__ );

    require_once( $this->_current_directory . '/Assessment/_inc/_global.php' );
  }

  public function setup() {
    require_once( $this->_current_directory . '/Assessment/_inc/definitions.php' );
    require_once( $this->_current_directory . '/Assessment/_inc/functions.php' );

    $this->_register_api_methods();
  }

  public function init() {
    $this->admin    = new Assessment\Admin();
    $this->block    = new Assessment\Block();
    $this->delivery = new Assessment\Delivery();
    $this->privacy  = new Assessment\Privacy();
    $this->user     = new Assessment\User();

    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
    add_action( 'delete_user', [ $this, "_on_delete_user" ] );
  }

  public function has_duplicate() {
    return false;
  }

  public function _on_delete_post( $post_id ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_ASSESSMENT )
      return;

    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION, [ 'post_id' => $post_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER, [ 'post_id' => $post_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA, [ 'post_id' => $post_id ] );
  }

  public function _on_delete_user( $user_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA, [ 'user_id' => $user_id ] );
  }

  /**
   * @return Assessment\Integration\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new Assessment\Integration\LearningJourney( $this );
  }

  /**
   * @return \WPEP\ContentType\Assessment\Integration\Builder
   */
  public function get_builder_instance() {
    if( $this->_builder_instance === null )
      $this->_builder_instance = new Assessment\Integration\Builder( $this );

    return $this->_builder_instance;
  }

  public function get_settings_defaults() {
    return [
      'assessment-course-page-embed'           => 0,
      'assessment-course-protection-rules'     => 1,
      'assessment'        => 'Assessment',
      'assessment-time-text' => 'You have a limited amount of time to complete this Assessment.',
      'assessment-time-start'=> 'Start Now',
      'assessment-view'   => 'View Assessment',
      'assessment-questions'         => 'Questions',
      'assessment-questions-correct' => 'Questions Correct',
      'assessment-simple-score'      => 'Score',
      'assessment-view-questions' => 'View Questions',
      'assessment-retake'         => 'Retake Assessment',
      'assessment-status'         => 'Status',
      'assessment-status-label-pending'         => 'Pending',
      'assessment-status-label-pending_review'  => 'Pending Review',
      'assessment-status-label-completed'       => 'Completed',
      'assessment-status-label-failed'          => 'Failed',
      'assessment-success-message'  => "<p>Completing this assessment enables you to continue the course.</p>
                                        <p>You can continue the Course by clicking the 'Continue to Course' button.</p>",
      'assessment-time-remaining'   => 'Time left : ',
      'assessment-time-allowed'     => 'Assessment Time',
      'assessment-part-prev'        => 'Previous Question',
      'assessment-part-next'        => 'Next Question',
      'assessment-submit'           => 'Submit Answers',
      'assessment-submit-singular'  => 'Submit Answer',
      'assessment-save'             => 'Save Answers',
      'assessment-return-to-course' => 'Return to Course',
      'assessment-go-to-course'     => 'Continue to Course',
      'assessment-score'            => 'Assessment Score',
      'assessment-score-requirement' => 'Required to Pass',
      'assessment-file-upload'      => 'Upload File',
      'assessment-input-placeholder'=> 'Write the answer to the question...',
      'assessment-status-pending'        => 'You have not submitted your answers for this assessment yet.',
      'assessment-status-pending-review' => 'The Assessment is currently being reviewed.',
      'assessment-status-completed'      => 'The Assessment has been completed successfully.',
      'assessment-status-failed'         => 'The Assessment has not been successfully completed',

      'assessment-assignment-status-pending-review' => 'The Assignment is currently being reviewed.',
      'assessment-assignment-status-completed'      => 'The Assignment has been completed successfully.',
      'assessment-assignment-status-failed'         => 'The Assignment has not been successfully completed',

      'assessment-question-status-correct'     => 'The answer provided is correct.',
      'assessment-question-status-incorrect'   => 'The answer provided is incorrect.',
      'assessment-answer-breakdown-separator'  => ' of ',
      'assessment-review-completed-email-notification-status'  => 0,
      'assessment-review-completed-email-notification-title'   => 'Assessment has been reviewed : [assessment_name]',
      'assessment-review-completed-email-notification-content' => '<p>[assessment_student_name] you have [assessment_student_status] the [assessment_name] assessment, with a score of [assessment_student_score].</p>' .
        '<p>[condition_assessment_has_course]<a href="[assessment_course_link]">Continue Course</a> | [/condition_assessment_has_course]<a href="[assessment_link]">View Assessment</a></p>' .
        '[assessment_student_answers]<br/>',
      'assessment-admin-email-notification-deliver-to' => 0,
      'assessment-admin-email-notification-title'       => 'New Submission : [assessment_name]',
      'assessment-admin-email-notification-content'    => '<br/>' .
        '<p>[assessment_student_name] has [assessment_student_status] the [assessment_name] assessment, with a score of [assessment_student_score].</p>' .
        '<p>[assessment_student_answers]</p><br/>' .
        '<p><a href="[assessment_student_submission_link]">Manage Assessment Submission</a></p>',

      'user-overview-assessment-single-view-list'      => 'View List',
      'user-overview-assessment-single-table-question-title'  => 'Title',
      'user-overview-assessment-single-table-question-answer' => 'Answer',
      'user-overview-assessment-single-table-title'    => '%s Assessment Report : %s - %s',
      'user-overview-assessment-single-label-status'   => 'Status',
      'user-overview-assessment-single-label-score'        => 'Score',
      'user-overview-assessment-single-label-started-at'   => 'Started At',
      'user-overview-assessment-single-label-completed-at' => 'Completed At',
      'user-overview-assessment-list-table-empty'      => 'There are no assessment records to display.',
      'user-overview-assessment-list-table-title'      => '%s Assessments Report',
      'user-overview-assessment-list-table-name'       => 'Name',
      'user-overview-assessment-list-table-status'     => 'Status',
      'user-overview-assessment-list-table-score'      => 'Score',
      'user-overview-assessment-list-table-started-at' => 'Started At',
      'user-overview-assessment-list-table-completed-at' => 'Completed At',
      'user-overview-assessment-list-table-view-assessment-report' => 'View Report',
    ];
  }

  public function get_post_supports() {
    return [ 'title' ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_ASSESSMENT;
  }

  public function get_cap_name() {
    return WPEP_POST_CAP_ASSESSMENT;
  }

  public function get_singular_name() {
    return __( 'Assessment', "wpep" );
  }

  public function get_plural_name() {
    return __( 'Assessments', "wpep" );
  }

  public function get_label_all_items() {
    return __( 'All Assessments', "wpep" );
  }

  public function register_post_type() {
    $post_labels = [
      'name'               => _x( $this->get_singular_name(), 'Post Type General Name', "wpep"),
      'singular_name'      => _x( $this->get_singular_name(), 'Post Type Singular Name', "wpep"),
      'menu_name'          => $this->get_label_all_items(),
      'name_admin_bar'     => __( $this->get_singular_name(), "wpep"),
      'parent_item_colon'  => __('Parent ' . $this->get_singular_name() . ':', "wpep"),
      'add_new'            => __('New ' . $this->get_singular_name(), "wpep"),
      'add_new_item'       => __('Add New ' . $this->get_singular_name(), "wpep"),
      'new_item'           => __('New ' . $this->get_singular_name(), "wpep"),
      'edit_item'          => __('Edit ' . $this->get_singular_name(), "wpep"),
      'update_item'        => __('Update ' . $this->get_singular_name(), "wpep"),
      'view_item'          => __('View ' . $this->get_singular_name(), "wpep"),
      'all_items'          => $this->get_label_all_items(),
      'search_items'       => __('Search ' . $this->get_singular_name(), "wpep"),
      'not_found'          => __('No ' . $this->get_singular_name() . ' found', "wpep"),
      'not_found_in_trash' => __('No ' . $this->get_singular_name() . ' found in Trash', "wpep")
    ];

    $post_params = [
      'supports'            => $this->get_post_supports(),
      'taxonomies'          => [],
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => 3,
      'menu_icon'           => '',
      'can_export'          => true,
      'has_archive'         => false,
      'exclude_from_search' => false,
      'publicly_queryable'  => true,
      'capability_type'     => WPEP_POST_CAP_ASSESSMENT,
      'map_meta_cap'        => true,
      'rewrite' => [
        'slug'  => wpep_get_setting( 'content-type-assessment-rewrite-slug', WPEP_POST_TYPE_ASSESSMENT )
      ],
      'label'               => __( strtolower( $this->get_singular_name() ), "wpep" ),
      'labels'              => $post_labels
    ];

    $post_params = apply_filters( 'wpep_content_type_' . $this->get_post_type() . '_register_args', $post_params );

    register_post_type( $this->get_post_type(), $post_params);
  }

  public function is_enabled() {
    if( !wpep_get_setting( 'content-type-' . WPEP_POST_TYPE_COURSE . '-enabled', false ) )
      return false;

    return wpep_get_setting( 'enable-course-assessments', 0 );
  }

  public function data_export_array() {
    return false;
  }

  public function data_asset_map() {
    return [
      'meta'              => [
        'wpep_success_message',
        'wpep_failed_message'
      ],
      'question_list'     => [
        'content',
        'answers' => [
          'content'
        ]
      ],
      'student_activity'  => [
        'value'
      ]
    ];
  }

  public function on_course_save( $course_post_id, $entity_id_map = [] ) {
    $this->admin->_handle_request_delete_questions_or_answers();

    if( isset( $_POST['wpep_assessment'] ) ) {
      foreach( $_POST['wpep_assessment'] as $assessment_section_id => $assessment_information ) {
        if( !isset( $assessment_information['title'] ) || $assessment_information['title'] == '' )
          continue;

        if( !is_numeric( $assessment_section_id ) && isset( $entity_id_map['section'][$assessment_section_id] ) )
          $assessment_section_id = $entity_id_map['section'][$assessment_section_id];
        else if( !in_array( $assessment_section_id, $entity_id_map['section'] ) && $assessment_section_id != 0 ) {
          // Delete the assessment if it existed ?
          continue;
        }

        $assessment_id         = wpep_get_course_assessment_id( $course_post_id, $assessment_section_id );
        $assessment_status     = ( isset( $assessment_information['post_status'] ) ? $assessment_information['post_status'] : 'publish' );

        if( $assessment_id == 0 ) {
          $assessment_id = wp_insert_post( [
            'post_title'      => $assessment_information['title'],
            'post_type'       => WPEP_POST_TYPE_ASSESSMENT,
            'post_status'     => $assessment_status,
            'comment_status'  => 'closed'
          ] );
        } else {
          wp_update_post( [
            'ID'          => $assessment_id,
            'post_status' => $assessment_status,
            'post_title'  => $assessment_information['title']
          ] );
        }

        update_post_meta( $assessment_id, 'wpep_course_id', $course_post_id );
        update_post_meta( $assessment_id, 'wpep_section_id', $assessment_section_id );

        foreach( $assessment_information['meta_information'] as $assessment_meta_key => $assessment_meta_value )
          update_post_meta( $assessment_id, $assessment_meta_key, $assessment_meta_value );

        if( !isset( $assessment_information['question'] ) )
          continue;

        $entity_id_map = $this->admin->save_assessment_questions( $assessment_information['question'], $assessment_id, $entity_id_map );
      }
    }

    return $entity_id_map;
  }

  /**
   * @param $tab_list
   * @param $builder_integration
   * @return mixed
   */
  public function attach_course_builder_tabs( $tab_list, $builder_integration ) {
    if ( get_current_screen()->action != 'add' )
      $assessment_ids = wpep_content_type_assessment_instance()->delivery->course->preload_for_post_status_list( get_the_ID(), wpep_post_monitored_post_status_list() );

    $tab_list[ 'final_assessment' ] = [
      'name'    => __( "Assess", "wpep" ),
      'slug'    => 'course-final-assessment',
      'content' => [ $this->admin, 'course_builder_tab_content' ]
    ];

    if( isset( $builder_integration->section_course_type[ 'assessment_management' ] ) ) {
      $tab_list[ 'final_assessment' ][ 'navigation_attributes' ] = [
        'data-wpep-course-type-information-block' => $builder_integration->section_course_type[ 'assessment_management' ]
      ];
    }

    return $tab_list;
  }

  private function _register_api_methods() {
    add_filter( 'wpep_api_request_response_system/statistics', [ $this, '_api_response_system_statistics' ], 50, 3 );
    add_filter( 'wpep_api_request_response_system/statistics-current-user', [ $this, '_api_response_system_statistics_current_user' ], 50, 3 );

    WPEP_API::instance()->register_content_type( $this->get_post_type(), [
      'template-interactive-single-content' => [
        'type'     => WPEP_API_METHOD_TYPE_PUBLIC,
        'callback' => '\WPEP\ContentType\Assessment\API::template_interactive_single_content'
      ],

      'user-id-post-id-data-view-plain' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_user_id_post_id',
        'callback'  => '\WPEP\ContentType\Assessment\API::user_id_post_id_data_view_plain'
      ],

      'submission-user-overview-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Assessment\API::submission_user_overview_list'
      ],

      'submission-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Assessment\API::submission_list'
      ],

      'post-id-submission-answers-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Assessment\API::post_id_submission_answers_list'
      ]
    ] );
  }

  public function _api_response_system_statistics( $response, $action, $request_data ) {
    if( $action === 'user-id-post-id-configuration-report' ) {
      if( !isset( $request_data[ 'post_id' ] ) || !isset( $request_data[ 'user_id' ] ) )
        return $response;

      if( \get_post_type( $request_data[ 'post_id' ] ) !== WPEP_POST_TYPE_ASSESSMENT )
        return $response;

      $response[ 'report_data_sources' ] = [
          'entity-assessment-user-view' => [
            'title'            => __( "Report", "wpep" ),
            'type'             => 'data-view-plain',
            'data_action'      => 'wpep/content-type/' . WPEP_POST_TYPE_ASSESSMENT . '/user-id-post-id-data-view-plain',
            'data_action_args' => [
              'user_id' => $request_data[ 'user_id' ],
              'post_id' => $request_data[ 'post_id' ]
            ]
          ]
        ] + $response[ 'report_data_sources' ];

      return $response;
    }

    if( !in_array( $action, [ 'configuration-report', 'user-id-configuration-report', 'post-id-configuration-report' ] ) )
      return $response;

    if( !isset( $response[ 'report_data_sources' ] ) )
      return $response;

    $base_args = [
      'type'        => 'data-view-list',
      'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_ASSESSMENT . '/submission-list'
    ];

    if( $action === 'post-id-configuration-report' ) {
      if( !isset( $request_data[ 'post_id' ] ) )
        return $response;

      if( get_post_type( $request_data[ 'post_id' ] ) !== WPEP_POST_TYPE_ASSESSMENT ) {
        if( get_post_type( $request_data[ 'post_id' ] ) !== WPEP_POST_TYPE_COURSE )
          return $response;

        $base_args[ 'data_action_args' ] = [
          'post_ids' => wpep_get_course_assessment_ids( $request_data[ 'post_id' ] )
        ];

        if( empty( $base_args[ 'data_action_args' ][ 'post_ids' ] ) )
          return $response;
      } else {
        $base_args[ 'data_action_args' ] = [
          'post_id' => intval( $request_data[ 'post_id' ] )
        ];
      }
    }

    if( $action === 'user-id-configuration-report' ) {
      if( !isset( $request_data[ 'user_id' ] ) )
        return $response;

      $base_args[ 'data_action_args' ] = [
        'user_id' => intval( $request_data[ 'user_id' ] )
      ];
    }

    $group_items = [
      'all' => [
        'title'       => __( "All", "wpep" ),
      ],
      WPEP_USER_ASSESSMENT_STATUS_PENDING => [
        'title'            =>  __( "Pending", "wpep" ),
        'data_action_args' => [
          WPEP_USER_ASSESSMENT_STATUS => WPEP_USER_ASSESSMENT_STATUS_PENDING
        ]
      ],
      WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => [
        'title'            =>  __( "Pending Review", "wpep" ),
        'data_action_args' => [
          WPEP_USER_ASSESSMENT_STATUS => WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW
        ]
      ],
      WPEP_USER_ASSESSMENT_STATUS_COMPLETED => [
        'title'            =>  __( "Completed", "wpep" ),
        'data_action_args' => [
          WPEP_USER_ASSESSMENT_STATUS => WPEP_USER_ASSESSMENT_STATUS_COMPLETED
        ]
      ],
      WPEP_USER_ASSESSMENT_STATUS_FAILED => [
        'title'            =>  __( "Failed", "wpep" ),
        'data_action_args' => [
          WPEP_USER_ASSESSMENT_STATUS => WPEP_USER_ASSESSMENT_STATUS_FAILED
        ]
      ],
      'user_overview' => [
        'title'       =>  __( "User Overview", "wpep" ),
        'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_ASSESSMENT . '/submission-user-overview-list'
      ]
    ];

    foreach( $group_items as $group_item_index => $group_item ) {
      $group_items[ $group_item_index ] += $base_args;

      // This isn't mutated.
      if( isset( $base_args[ 'data_action_args' ] ) )
        foreach( $base_args[ 'data_action_args' ] as $k => $v )
          $group_items[ $group_item_index ][ 'data_action_args' ][ $k ] = $v;

      if( isset( $request_data[ 'user_ids' ] ) && !empty( $request_data[ 'user_ids' ] ) ) {
        if( !isset( $group_items[ $group_item_index ][ 'data_action_args' ] ) )
          $group_items[ $group_item_index ][ 'data_action_args' ] = [];

        $group_items[ $group_item_index ][ 'data_action_args' ][ 'user_ids' ] = $request_data[ 'user_ids' ];
      }
    }

    if( $action === 'post-id-configuration-report' ) {
      $append = [];

      if( get_post_type( $request_data[ 'post_id' ] ) === WPEP_POST_TYPE_ASSESSMENT ) {
        $append['answers'] = wp_parse_args( [
          'title'       => __( "Answers", "wpep" ),
          'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_ASSESSMENT . '/post-id-submission-answers-list'
        ], $base_args );
      }

      foreach( $group_items as $group_item_key => $group_item )
        $append[ 'assessment-submissions-' . $group_item_key ] = [
          'title'       => __( "Submissions", "wpep" ) . ' - ' . $group_item[ 'title' ]
        ] + $group_item;

      $response[ 'report_data_sources' ] += $append;
    } else {
      $response[ 'report_data_sources' ][ 'assessment-submissions' ] = [
        'title'       => __( "Assessment Submissions", "wpep" ),
        'type'        => 'group',
        'group'       => $group_items,
      ];
    }

    return $response;
  }

  public function _api_response_system_statistics_current_user( $response, $action, $request_data ) {
    if( 'configuration-report' !== $action )
      return $response;

    if( !is_array( $response ) || !isset( $response[ 'user_ids' ] ) )
      return $response;

    $request_data[ 'user_ids' ] = $response[ 'user_ids' ];

    return $this->_api_response_system_statistics( $response, $action, $request_data );
  }


}