<?php

namespace WPEP\ContentType\Assessment\Integration;

use WPEP\ContentType\Skeleton\Integration\Builder as Builder_Integration;
use WPEP\ContentType\Assessment as Content_Type_Assessment;
use WPEP\Container\FormFields as WPEP_FormFields_Configuration;
use WPEP\Entity\Course as Entity_Course;

class Builder extends Builder_Integration {

  use WPEP_FormFields_Configuration;

  /**
   * @var Content_Type_Assessment
   */
  protected $_contentTypeInstance;

  public $enable_downloads           = true;
  public $enable_assessments         = false;
  public $enable_timing_settings     = false;
  public $enable_on_boarding_options = false;
  public $enable_member_magnet                = false;
  public $enable_presentation_card_settings   = true;
  public $enable_presentation_button_settings = true;
  public $enable_learning_paths               = true;
  public $enable_progress_repeat              = false;
  public $new_course_sections   = 3;
  public $new_course_lessons    = 5;

  public $section_course_type = [];

  /**
   * @param Content_Type_Assessment $content_type_instance
   */
  public function __construct( $content_type_instance ) {
    parent::__construct( $content_type_instance );
  }

  public function display() {
    echo '<div id="wpep_builder" class="wpep-builder-no-tabs">';

    $this->_contentTypeInstance->admin->display_assessment_setup_functionality( get_the_ID(), 'wpep_assessment', true );

    echo '</div>';
  }

  public function save_from_post( $post_id, $entity_id_map = [] ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_ASSESSMENT )
      return $entity_id_map;

    if( !isset( $_POST['wpep_assessment']['question'] ) )
      return $entity_id_map;

    $this->_contentTypeInstance->admin->_handle_request_delete_questions_or_answers();

    $entity_id_map = $this->_contentTypeInstance->admin->save_assessment_questions( $_POST['wpep_assessment']['question'], $post_id, $entity_id_map );

    foreach( $_POST['wpep_assessment']['meta_information'] as $assessment_meta_key => $assessment_meta_value )
      update_post_meta( $post_id, $assessment_meta_key, $assessment_meta_value );

    return $entity_id_map;
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    $field_value = get_post_meta( get_the_ID(), $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

}