<?php

namespace WPEP\ContentType\Assessment\Integration;

use WPEP\ContentType\Skeleton\Integration\LearningJourney as Learning_Journey_Integration;
use WPEP\Controller as WPEP_Controller;

class LearningJourney extends Learning_Journey_Integration {

  /**
   * The array format
   * @param int $post_id
   * @return array [ name, description, status, last_action_timestamp, link, link_title, design_attribute ]
   */
  public function get_step_map_for_current_user( $post_id ) {
    $assessment_status = wpep_user_assessment_status( $post_id, get_current_user_id() );

    $learning_journey_status = WPEP_LEARNING_JOURNEY_STATUS_PENDING;

    if( $assessment_status === WPEP_USER_ASSESSMENT_STATUS_COMPLETED )
      $learning_journey_status = WPEP_LEARNING_JOURNEY_STATUS_COMPLETED;

    $last_action_timestamp = wpep_content_type_assessment_instance()->user->data->get_value(
      WPEP_USER_ASSESSMENT_SUBMISSION_TIME,
      $post_id,
      0,
      0,
      get_current_user_id()
    );

    $response = [
      'design_attribute'       => $this->_contentTypeInstance->get_post_type(),
      'name'                   => get_the_title( $post_id ),
      'description'            => '',
      'status'                 => $learning_journey_status,
      'last_action_timestamp'  => ( $last_action_timestamp === null ? false : $last_action_timestamp ),
      'link'                   => get_permalink( $post_id ),
      'link_title'             => wpep_get_setting_translated( 'assessment-view' ),
      'icon_html'              => '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 480.021 480.021" style="enable-background:new 0 0 480.021 480.021;" xml:space="preserve"> <g> <g> <path d="M479.323,168.932c-0.803-1.798-2.242-3.237-4.04-4.04l-232-104c-2.082-0.933-4.462-0.933-6.544,0l-232,104 c-4.034,1.802-5.843,6.534-4.04,10.568c0.803,1.798,2.242,3.237,4.04,4.04l75.272,33.736v142.96c0,2.36,1.042,4.6,2.848,6.12 c90.736,76.683,223.568,76.683,314.304,0c1.806-1.52,2.848-3.76,2.848-6.12v-142.96l32-14.344v138.904l-12,17.952 c-2.622,3.943-4.014,8.576-4,13.312v9.272c0,13.255,10.745,24,24,24s24-10.745,24-24v-9.296c0.014-4.736-1.378-9.369-4-13.312 l-12-17.928V191.724l27.272-12.224C479.316,177.697,481.125,172.966,479.323,168.932z M384.01,352.436 c-83.779,68.504-204.221,68.504-288,0v-132l140.728,63.088c2.082,0.933,4.462,0.933,6.544,0l140.728-63.112V352.436z M448.01,378.308c0,4.418-3.582,8-8,8s-8-3.582-8-8v-9.272c-0.002-1.578,0.466-3.121,1.344-4.432l6.656-10.008l6.656,9.984 c0.878,1.311,1.346,2.854,1.344,4.432V378.308z M435.466,179.812L240.65,164.196c-4.418-0.353-8.287,2.942-8.64,7.36 s2.942,8.287,7.36,8.64l165.72,13.256l-165.08,73.944l-212.44-95.2l212.44-95.2l212.44,95.2L435.466,179.812z"/> </g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> <g> </g> </svg> '
    ];

    return $response;
  }

}