<?php

namespace WPEP\ContentType\Assessment;

use WPEP\Entity\Assessment as WPEP_Entity_Assessment;

class User {

  /**
   * @var User\Data
   */
  public $data;

  public function __construct() {
    $this->data = new User\Data();
  }

  public function get_status( $assessment_id, $user_id = 0 ) {
    $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

    if( $user_id == 0 )
      return WPEP_USER_ASSESSMENT_STATUS_PENDING;

    $user_status = $this->data->get(
      WPEP_USER_ASSESSMENT_STATUS, $assessment_id, 0, 0, $user_id, true
    );

    $user_status = ( $user_status === null ? WPEP_USER_ASSESSMENT_STATUS_PENDING : $user_status->value );

    return $user_status;
  }

  public function get_answers_text_by_assessment_id_and_user_id( $assessment_id, $user_id ) {
    $sql = 'SELECT ud_a.`question_id`,
                   IFNULL( qa.`title`, ud_a.`value` ) as `value`
              FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA . '` ud_a
         LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER . '` qa ON qa.id = ud_a.`question_answer_id`
             WHERE ud_a.`key` = "answer"
               AND ud_a.`post_id` = ' . intval( $assessment_id ) . '
               AND ud_a.`user_id` = ' . intval( $user_id );

    $results = wpep_wpdb()->get_results( $sql );

    if( empty( $results ) )
      return [];

    return wpep_utility_map_to_array_assoc( $results, 'question_id', 'value', true );
  }

  public function check_submission_answers( $assessment_id, $user_id = false, $assessment_type = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $assessment_type === false )
      $assessment_type = wpep_get_assessment_type( $assessment_id );

    if( $assessment_type === WPEP_ASSESSMENT_TYPE_ASSIGNMENT )
      wpep_content_type_assessment_instance()->user->data->set(WPEP_USER_ASSESSMENT_STATUS, WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW, $assessment_id, 0, 0, $user_id );
    else if( $assessment_type === WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING  )
      $this->refresh_submission_grading( $assessment_id, $user_id, $assessment_type );
    else
      $this->_check_submission_answers_standard( $assessment_id, $user_id );
  }

  private function _check_submission_answers_standard( $assessment_id, $user_id ) {
    $answer_list = wpep_utilities_map_object(
      $this->data->get( 'answer', $assessment_id, false, false, $user_id, false ),
      'question_id',
      true
    );

    $grading_status    = get_post_meta( $assessment_id, 'wpep_grading_option', true );

    $internal_question_list = wpep_utilities_map_object( WPEP_Entity_Assessment::instance()->get_questions_by_assessment_id( $assessment_id, false, false ), 'id' );
    $internal_answer_list   = WPEP_Entity_Assessment::instance()->get_answers_by_assessment_id( $assessment_id );
    $question_type_data     = wpep_get_assessment_field_types();

    foreach( $answer_list as $question_id => $answer_user_data ) {
      if( !isset( $internal_question_list[$question_id] ) )
        continue;

      $current_question_type_data = $question_type_data[ $internal_question_list[$question_id]->type ];

      $is_correct = ( in_array( $grading_status, [ 1,2 ] ) ? -1 : 1 );

      if( $current_question_type_data['multiple_answers'] && in_array( $grading_status, [ 2,3 ] ) ) {
        $question_correct_answers = wpep_utilities_filter_object_list_by_params( $internal_answer_list, [
          'question_id' => $question_id,
          'is_correct'  => '1'
        ], 'id' );

        if( count( $question_correct_answers ) != count( $answer_user_data ) ) {
          $is_correct = 0;
        } else {
          foreach( $answer_user_data as $current_user_answer ) {
            if( !isset( $question_correct_answers[ $current_user_answer->question_answer_id ] ) ) {
              $is_correct = 0;
              break;
            }
          }
        }

        foreach( $answer_user_data as $answer_user_data_single ) {
          if( !isset( $internal_answer_list[$answer_user_data_single->question_answer_id] ) )
            continue;

          if( $is_correct != 0 && 1 == 1 )
            $is_correct = $internal_answer_list[$answer_user_data_single->question_answer_id]->is_correct;
        }

        wpep_content_type_assessment_instance()->user->data->set(
          'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id
        );
      } else if( $current_question_type_data['has_answers'] && in_array( $grading_status, [ 2,3 ] ) ) {
        $answer_user_data = array_shift( $answer_user_data );

        if( in_array( $grading_status, [ 2,3 ] ) && isset( $answer_user_data->question_answer_id ) )
          $is_correct = ( isset( $internal_answer_list[ $answer_user_data->question_answer_id ] )
            ? $internal_answer_list[ $answer_user_data->question_answer_id ]->is_correct
            : 0
          );

        wpep_content_type_assessment_instance()->user->data->set(
          'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id
        );
      } else {
        wpep_content_type_assessment_instance()->user->data->set(
          'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id
        );
      }

    }

    $this->refresh_submission_grading( $assessment_id, $user_id );
  }

  public function refresh_submission_grading( $assessment_id, $user_id, $assessment_type = null ) {
    if( $assessment_type === null )
      $assessment_type = wpep_get_assessment_type( $assessment_id );

    do_action( "wpep_user_assessment_before_grading", $assessment_id, $user_id, $assessment_type );

    $grading_score     = null;
    $status            = null;

    if( $assessment_type === WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING ) {
      $grading_score = $this->_get_score_simple_scoring( $assessment_id, $user_id );
      $status = WPEP_USER_ASSESSMENT_STATUS_COMPLETED;

    } elseif( $assessment_type === WPEP_ASSESSMENT_TYPE_ASSIGNMENT ) {
      $grading_score = $this->_get_grading_score_standard( $assessment_id, $user_id );

      if( $grading_score == 100 )
        $status = WPEP_USER_ASSESSMENT_STATUS_COMPLETED;
      else
        $status = WPEP_USER_ASSESSMENT_STATUS_FAILED;

    } elseif( get_post_meta( $assessment_id, 'wpep_grading_option', true ) === WPEP_ASSESSMENT_GRADING_DISABLED ) {
      $status = WPEP_USER_ASSESSMENT_STATUS_COMPLETED;
    } else {
      $grading_score = $this->_get_grading_score_standard( $assessment_id, $user_id );

      if( $grading_score !== null ) {
        if( $grading_score >= intval( get_post_meta( $assessment_id, 'wpep_grading_requirement', true ) ) )
          $status = WPEP_USER_ASSESSMENT_STATUS_COMPLETED;
        else
          $status = WPEP_USER_ASSESSMENT_STATUS_FAILED;
      } else {
        $status = WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW;
      }
    }

    if( $status !== null ) {
      wpep_content_type_assessment_instance()->user->data->set(
        WPEP_USER_ASSESSMENT_STATUS, $status, $assessment_id, 0, 0, $user_id
      );
    }

    if( $grading_score !== null ) {
      wpep_content_type_assessment_instance()->user->data->set(
        WPEP_USER_ASSESSMENT_SCORE, $grading_score, $assessment_id, 0, 0, $user_id
      );
    }

    do_action( "wpep_user_assessment_after_grading", $assessment_id, $user_id, $assessment_type );
  }

  private function _get_score_simple_scoring( $assessment_id, $user_id ) {
    $user_answer_list = wpep_utility_map_to_array_assoc(
      $this->data->get( 'answer', $assessment_id, false, false, $user_id, false ),
      'question_id',
      'value'
    );

    $internal_question_list = wpep_utilities_map_object( WPEP_Entity_Assessment::instance()->get_questions_by_assessment_id( $assessment_id, false, false ), 'id' );
    $question_type_data     = wpep_get_assessment_field_types();

    $user_score   = 0;
    $weight_count = 0;
    $max_question_score = $this->_get_simple_scoring_points_cap( $question_type_data, $internal_question_list );

    foreach( $user_answer_list as $answer_question_id => $answer ) {
      $answer = intval( $answer );

      $current_question = $internal_question_list[ $answer_question_id ];
      $current_weight   = intval( $current_question->weight );

      if( $question_type_data[ $current_question->type ]['max_score'] != $max_question_score )
        $answer *= $max_question_score / $question_type_data[ $current_question->type ]['max_score'];

      $weight_count += $current_weight;
      $user_score += $answer * $current_weight;
    }

    return number_format( $user_score / $weight_count, 2 );
  }

  private function _get_simple_scoring_points_cap( $question_type_data, $internal_question_list ) {
    $question_type_count = [];

    foreach( $internal_question_list as $internal_question ) {
      $question_type_count[ $internal_question->type ] = ( isset( $question_type_count[ $internal_question->type ] ) ? $question_type_count[ $internal_question->type ] : 0 ) + 1;
    }

    uasort( $question_type_count, function( $a, $b ) {
      return ( $a == $b ? 0 : ( $a > $b ? -1 : 1 ) );
    } );

    $major_question_type = key( $question_type_count );

    return isset( $question_type_data[ $major_question_type ] ) && isset( $question_type_data[ $major_question_type ]['max_score'] ) ? $question_type_data[ $major_question_type ]['max_score'] : 10;
  }

  private function _get_grading_score_standard( $assessment_id, $user_id ) {
    $internal_question_list = wpep_utilities_map_object( WPEP_Entity_Assessment::instance()->get_questions_by_assessment_id( $assessment_id, false, false ), 'id' );

    $answers_correct   = 0;
    $answers_graded    = 0;
    $answers_score     = 0;

    $question_answer_status_list = wpep_utility_map_to_array_assoc(
      $this->data->get( 'is_correct', $assessment_id, false, false, $user_id, false ),
      'question_id',
      'value'
    );

    foreach( $question_answer_status_list as $question_id => $is_correct ) {
      $is_correct = intval( $is_correct );

      if( $is_correct === -1 )
        return null;

      if( $is_correct ) {
        $answers_score += $internal_question_list[$question_id]->weight;

        $answers_correct++;
      }

      $answers_graded++;
    }

    $answers_score_max = 0;

    foreach( $internal_question_list as $internal_question )
      $answers_score_max += $internal_question->weight;

    $grading_score = ( $answers_score == 0 ? 0 : ( $answers_score / $answers_score_max ) * 100 );

    return $grading_score;
  }

}