<?php

namespace WPEP\ContentType\Container;

use WPEP\ACL\Post as WPEP_ACL_Post;

trait Admin {

  public function _setup_administration() {
    if( isset( $_GET[ 'wpep-duplicate-post' ] ) ) {
      if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
        return;

      $post = get_post( intval( $_GET[ 'wpep-duplicate-post' ] ) );

      if( $post === NULL )
        return;

      if( !isset( $this->contentTypes[ $post->post_type ] ) )
        return;

      $post_export = $this->contentTypes[ $post->post_type ]->data_export_single_post_array( $post, false );

      $new_post_id = $this->contentTypes[ $post->post_type ]->data_import_single_post_array( $post_export, [] );

      $this->contentTypes[ $post->post_type ]->data_import_single_post_connection( $post_export, [], $new_post_id );

      if( $new_post_id === 0 )
        return;

      wpep_redirect( wpep_post_admin_edit_link( $new_post_id ) );
      exit;
    }

    add_action( 'post_row_actions', [ $this, '_action_row_actions' ], 20, 2 );
    add_action( 'save_post', [ $this, '_action_save_post' ] );
    add_filter( 'wpep_meta_boxes', [ $this, '_filter_meta_boxes' ] );

    foreach( $this->standardContentTypes as $contentType ) {
      add_filter( 'bulk_actions-edit-' . $contentType->get_post_type(), [ $this, '_filter_bulk_actions' ] );
      add_filter( 'handle_bulk_actions-edit-' . $contentType->get_post_type(), [ $this, '_filter_handle_bulk_actions' ], 10, 3 );
    }
  }

  public function _action_row_actions($actions, $post) {
    if( !isset( $this->standardContentTypes[$post->post_type ] ) )
      return $actions;

    if( $this->standardContentTypes[ $post->post_type ]->has_duplicate() && WPEP_ACL_Post::export( $post ) ) {
      $actions['wpep_duplicate'] = '<a href="' . add_query_arg( 'wpep-duplicate-post', $post->ID, admin_url() ) . '">' . __( "Duplicate", "wpep" ) . '</a>';
    }

    if( WPEP_ACL_Post::analytics( $post->ID, 'view' ) ) {
      $actions[ 'wpep_statistics' ] = '<a data-ec-component-trigger="open" data-ec-component-trigger-args="' . esc_attr(
          json_encode( [
            'component' => 'interval-navigation-panel',
            'action'    => 'wpep/system/statistics/post-id-configuration-report',
            'post_id'   => $post->ID
          ] )
        ). '" href="#">' . sprintf( __( "%s Analytics", "wpep" ), WPEP_PLUGIN_NAME ) . '</a>';
    }

    return $actions;
  }

  public function _action_save_post( $post_id ) {
    global $post_type;

    if ( !(is_admin() && ( isset( $this->standardContentTypes[ $post_type ] ) ) ) )
      return;

    if( !isset( $this->standardContentTypes[ get_post_type( $post_id ) ] ) )
      return;

    if( intval( get_post_meta( $post_id, 'wpep_order', true ) ) == false )
      update_post_meta( $post_id, 'wpep_order', 0 );
  }

  public function _filter_meta_boxes( $meta_boxes ) {
    if( empty( $this->standardContentTypes ) )
      return $meta_boxes;

    global $post_type;

    if( !isset( $this->standardContentTypes[ $post_type ] ) )
      return $meta_boxes;

    $content_type_meta_fields = $this->standardContentTypes[ $post_type ]->standard_get_post_meta_fields();

    if( empty( $content_type_meta_fields ) )
      return $meta_boxes;

    $content_type_meta_boxes['wpep-' . $this->standardContentTypes[ $post_type ]->get_post_type() . '-settings'] = [
      'id'         => 'wpep_' . $this->standardContentTypes[ $post_type ]->get_post_type() . '_settings',
      'title'      => __( "General Options", "wpep" ),
      'post_types' => [ $this->standardContentTypes[ $post_type ]->get_post_type() ],
      'context'    => $this->standardContentTypes[ $post_type ]->standard_get_post_meta_fields_context(),
      'priority'   => 'low',
      'fields'     => $content_type_meta_fields
    ];

    $content_type_meta_boxes = apply_filters( 'wpep_administration_' . $this->standardContentTypes[ $post_type ]->get_post_type() . '_metaboxes', $content_type_meta_boxes );

    $meta_boxes += $content_type_meta_boxes;

    return $meta_boxes;
  }

  public function _filter_bulk_actions( $bulk_actions ) {
    $bulk_actions[ 'wpep_bulk_export' ] = __( "Export as ZIP", "wpep" );

    return $bulk_actions;
  }

  public function _filter_handle_bulk_actions( $redirect_to, $do_action, $post_ids ) {
    if ( $do_action != 'wpep_bulk_export' )
      return $redirect_to;

    $export_information = [];

    foreach ( $post_ids as $key => $post_id ) {
      $post = get_post( $post_id );

      if( !WPEP_ACL_Post::export( $post ) )
        continue;

      if( !isset( $export_information[ $post->post_type ] ) )
        $export_information[ $post->post_type ] = [];

      $current_export_information = $this->contentTypes[ $post->post_type ]->data_export_single_post_array( $post, false );

      $export_information[ $post->post_type ][ $post->ID ] = $current_export_information;

    }

    if( empty( $export_information ) )
      return $redirect_to;

    $export_file = wpep_data_system()->data_export_array_to_zip( $export_information, 'bulk-export' );

    header('Content-disposition: attachment; filename=' . basename( $export_file ) );
    header('Content-type: application/zip');

    readfile( $export_file );

    unlink( $export_file );
    exit;

    return $export_file;
  }

}