<?php

namespace WPEP\ContentType\Container;

trait Library {

  public function library_post_type_list() {
    $response = [];

    foreach( $this->libraryContentTypes as $libraryContentType )
      $response[ $libraryContentType->get_post_type() ] = $libraryContentType->get_plural_name();

    return $response;
  }

  public function library_post_types() {
    $response = [];

    foreach( $this->libraryContentTypes as $libraryContentType )
      $response[] = $libraryContentType->get_post_type();

    return $response;
  }

  public function library_get_search_details( $current_details, $post ) {
    if( !isset( $this->libraryContentTypes[ $post->post_type] ) )
      return $current_details;

    return $this->libraryContentTypes[ $post->post_type ]->library_get_search_details( $current_details, $post );
  }

  public function library_output_grid_content_html( $course_categories, $options = [] ) {
    $paginate_per_page     = isset( $options['entries_per_page'] ) ? $options['entries_per_page'] : 0;
    $paginate_current_page = isset( $options['current_page'] ) ? $options['current_page'] : 1;
    $entries_to_skip       = ( $paginate_current_page - 1 ) * $paginate_per_page;
    $entries_to_display    = $paginate_per_page;
    $entries_total         = 0;

    foreach( $this->libraryContentTypes as $libraryContentType ) {
      if( $paginate_per_page === 0 ) {
        $libraryContentType->library_output_grid_content_html( $course_categories );

        continue;
      }

      if( $libraryContentType->library_allow_content() == false )
        continue;

      $current_posts = $libraryContentType->library_get_grid_posts( $course_categories, ( $entries_to_display <=1 ? 1 : $entries_to_display ), $entries_to_skip, ( isset( $options['search'] ) ? $options['search'] : '' ) );

      $entries_total += $current_posts->found_posts;

      if( $entries_to_skip > 0 )
        $entries_to_skip = ( $entries_to_skip > $current_posts->found_posts ? $entries_to_skip - $current_posts->found_posts : 0 );

      if( $entries_to_display <= 0 || empty( $current_posts->posts ) )
        continue;

      $entries_to_display = ( $current_posts->post_count < $entries_to_display ? $entries_to_display - $current_posts->post_count : 0 );

      $libraryContentType->library_output_grid_content_html( [], $current_posts->posts );
    }

    if( $paginate_per_page === 0 )
      return;

    $base_url = wpep_get_current_url_index_page();

    if( isset( $options['current_category'] ) ) {
      $base_url = add_query_arg( 'wpep-library-base-category', $options['current_category'], $base_url );
    } elseif( !empty( $course_categories ) && $options['require_all_categories'] ) {
      $base_url = add_query_arg( 'wpep-library-categories', implode( "-", $course_categories ), $base_url );
    }

    wpep_template_load( 'primary/pagination.php', [
      'paginate_total_pages'  => ceil( $entries_total / $paginate_per_page ),
      'paginate_current_page' => $paginate_current_page,
      'paginate_per_page'     => $paginate_per_page,
      'paginate_total_entries'=> $entries_total,
      'base_url'              => $base_url
    ] );
  }

}