<?php

namespace WPEP\ContentType\Course\Admin;

class Settings {

  public function __construct() {
    add_filter( 'wpep_administration_settings_map', [ $this, '_admin_settings_map' ], 20 );
  }

  public function _admin_settings_map( $settings_map ) {
    $settings_map_course_page = [
      'name'      => __( "Course Page", "wpep" ),
      'alias'     => 'course_page',
      'sections'  => [
        'general'         => $this->_get_general_settings(),
        'execution_plan'  => $this->_get_execution_plan_settings(),
        'standard'        => $this->_get_standard_settings(),
        'video'           => $this->_get_video_settings(),
        'focus'           => $this->_get_focus_settings(),
        'one_off'         => $this->_get_one_off_settings(),
        'challenge'       => $this->_get_challenge_settings(),
        'scorm'           => $this->_get_scorm_settings()
      ]
    ];

    $settings_map = [
      'general'     => $settings_map[ 'general' ],
      'index_page'  => $settings_map[ 'index_page' ],
      'course_page' => $settings_map_course_page,
    ] + array_diff_key( $settings_map, [ 'general' => [], 'index_page' => [] ] );

    if( isset( $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ] ) ) {
      $content_types_map = isset( $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ][ 'content_types' ] ) ? $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ][ 'content_types' ] : false;

      if( $content_types_map !== false )
        unset( $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ][ 'content_types' ] );

      $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ] = [
          'course_options' => $this->_get_content_type_course_options()
        ] + $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ];

      if( $content_types_map !== false )
        $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ] = [
            'content_types' => $content_types_map
          ] + $settings_map[ 'general' ][ 'sections' ][ 'content' ][ 'field_map' ];
    }

    if( isset( $settings_map[ 'general' ][ 'sections' ][ 'advanced' ][ 'field_map' ] ) ) {

      $settings_map[ 'general' ][ 'sections' ][ 'advanced' ][ 'field_map' ][ 'administration' ][ 'fields' ] += [
        'admin-course-preset-sections' => [
          'name'        => __( 'New Course Sections', "wpep" ),
          'alias'       => 'admin-course-preset-sections',
          'type'        => 'select',
          'options'     => [ 1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5 ],
          'description' => __( 'Choose the number of Sections that are automatically Pre-Loaded for you when adding a new Course.', "wpep" )
        ],
        'admin-course-preset-lessons' => [
          'name'        => __( 'New Course Lessons', "wpep" ),
          'alias'       => 'admin-course-preset-lessons',
          'type'        => 'select',
          'options'     => [ 1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10 ],
          'description' => __( 'Choose the number of Lessons that are automatically Pre-Loaded into Sections when adding a new Course.', "wpep" )
        ],
        'admin-course-save-alert' => [
          'name'        => __('Course Save Alert', "wpep"),
          'alias'       => 'admin-course-save-alert',
          'type'        => 'checkbox',
          'description' => __( "Enable keep your content save alert while editing a course.", "wpep" )
        ],
        'lesson-wp-editor-preload' => [
          'name'        => __( 'Instant Lesson Editor', "wpep" ),
          'alias'       => 'lesson-wp-editor-preload',
          'type'        => 'checkbox',
          'description' => __( 'Disable this if your device behaves poorly while editing lessons.', "wpep" )
        ],
        'lesson-wp-editor-enabled-new' => [
          'name'        => __( 'Open New Lessons Editor', "wpep" ),
          'alias'       => 'lesson-wp-editor-enabled-new',
          'type'        => 'checkbox',
          'description' => __( 'Check this to automate your wysiwyg text editor to open by default when adding a new lesson.', "wpep" )
        ]
      ];

      if( wpep_get_setting_flag( 'enable-course-discussions-option' ) )
        $settings_map[ 'general' ][ 'sections' ][ 'advanced' ][ 'field_map' ][ 'frontend' ][ 'fields' ][ 'disable-course-discussion-access-verification' ] = [
          'name'        => __( 'Disable Discussions Access Verification', "wpep" ),
          'alias'       => 'disable-course-discussion-access-verification',
          'type'        => 'checkbox',
          'description' => __( "Not recommended except if you're encountering issues with Course Discussions on your setup, any user may post comments to the course, even if they do not have access.", "wpep" )
        ];
    }

    return $settings_map;
  }

  private function _get_content_type_course_options() {
    return [
      'name'    => __( "Course Options", "wpep" ),
      'alias'   => 'course_options',
      'fields'  => [
        'enable-course-assessments' => [
          'name'        => __( 'Assessments', "wpep"),
          'alias'       => 'enable-course-assessments',
          'type'        => 'checkbox'
        ],
        'enable-certifications-support' => [
          'name'        => __( 'Certifications', "wpep"),
          'alias'       => 'enable-certifications-support',
          'type'        => 'checkbox'
        ],
        'enable-course-bookmarks' => [
          'name'        => __( 'Bookmarks', "wpep"),
          'alias'       => 'enable-course-bookmarks',
          'type'        => 'checkbox'
        ],
        'enable-course-deliverables' => [
          'name'        => __( 'Deliverables', "wpep"),
          'alias'       => 'enable-course-deliverables',
          'type'        => 'checkbox'
        ],
        'enable-course-discussions-option' => [
          'name'        => __( 'Discussions', "wpep"),
          'alias'       => 'enable-course-discussions-option',
          'type'        => 'checkbox'
        ],
        'enable-course-timing-option' => [
          'name'        => __( 'Drip Options', "wpep"),
          'alias'       => 'enable-course-timing-option',
          'type'        => 'checkbox'
        ],
        'enable-progress-repeat-support' => [
          'name'        => __( 'Progress Repeat', "wpep"),
          'alias'       => 'enable-progress-repeat-support',
          'type'        => 'checkbox'
        ],
        'enable-sid-support' => [
          'name'        => __('SID Identification', "wpep"),
          'alias'       => 'enable-sid-support',
          'type'        => 'checkbox'
        ],
        'enable-course-on-boarding-options' => [
          'name'        => __('Onboarding Options', "wpep"),
          'alias'       => 'enable-course-on-boarding-options',
          'type'        => 'checkbox'
        ],
        'enable-course-time-spent-option' => [
          'name'        => __('Time-spent', "wpep" ),
          'alias'       => 'enable-course-time-spent-option',
          'type'        => 'checkbox'
        ],
      ]
    ];
  }

  private function _get_general_settings() {
    return [
      'name'      => __( 'General', "wpep" ),
      'alias'     => 'general',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'enable-lesson-incomplete-option' => [
              'name'        => __('Enable Lesson Incomplete', "wpep"),
              'alias'       => 'enable-lesson-incomplete-option',
              'type'        => 'checkbox',
              'description' => __( 'Allow Lessons within Courses to be Marked as incomplete, after they have been completed, Execution Plans allow this by default.', "wpep" )
            ],
            'enable-course-timing-option' => [
              'name'        => __('Enable Timing Options', "wpep"),
              'alias'       => 'enable-course-timing-option',
              'type'        => 'checkbox',
              'description' => __( 'Enable Content Dripfeeding & Course Timing Options.', "wpep" )
            ],
            'enable-course-tracking-code-option' => [
              'name'        =>  __('Course Tracking Code', "wpep"),
              'alias'       => 'enable-course-tracking-code-option',
              'type'        => 'checkbox',
              'description' => __( 'Allow Adding Course Completion Tracking Codes.', "wpep" )
            ],
            'course-resize-wistia-embeds' => [
              'name'        =>  __('Resize Wistia Videos', "wpep"),
              'alias'       => 'course-resize-wistia-embeds',
              'type'        => 'checkbox',
              'description' =>  __( "Depending on how you are embeding Wistia Videos, you'll want to check this option.", "wpep" )
            ],
            'course-use-the-content-filter' => [
              'name'        =>  __('The Content Filter', "wpep"),
              'alias'       => 'course-use-the-content-filter',
              'type'        => 'checkbox',
              'description' =>  __( 'Disabling this option is situational, and not recommended.', "wpep" )
            ],
            'course-pre-authentication-tracking' => [
              'name'        =>  __('Pre Authentication Tracking', "wpep"),
              'alias'       => 'course-pre-authentication-tracking',
              'type'        => 'checkbox',
              'description' => __( 'Track User Course Progress trough Cookies if they are not logged in.', "wpep" )
            ],
            'opt-course-progressbar' => [
              'name'        =>  __('Progress Text', "wpep"),
              'alias'       => 'opt-course-progressbar',
              'type'        => 'text_translated',
              'description' => sprintf( __( "To personalize the text below the %s progress bar enter it here", "wpep" ), WPEP_PLUGIN_NAME )
            ],
            'opt-course-percentage-bar' => [
              'name'        =>  __('Progress Percentage Bar', "wpep"),
              'alias'       => 'opt-course-percentage-bar',
              'type'        => 'checkbox',
              'description' => __('Display a course progress percentage bar, you may personalize this for each course.', "wpep")
            ],
            'opt-course-percentage-bar-number' => [
              'name'        =>  __('Progress Percentage Bar Number', "wpep"),
              'alias'       => 'opt-course-percentage-bar-number',
              'type'        => 'checkbox',
              'description' => __('When the above option is enabled, will also display the % of the progress.', "wpep")
            ],
            'course-coming-soon-student-notification' => [
              'name'        =>  __( "Coming Soon Access Restricted", "wpep" ),
              'alias'       => 'course-coming-soon-student-notification',
              'type'        => 'editor',
              'description' => '<p>' . __( "While a Course is Coming Soon, students may not do the Course earlier by guessing the URL, this is the error message.", "wpep" ) . '</p>' .
                               '<p>' . __( "This does not apply for shortcode embeds.", "wpep" ) . '</p>'
            ],
          ]
        ],
        'deliverables' => [
          'name'      => __( "Deliverables Settings", "wpep" ),
          'has_title' => true,
          'alias'     => 'deliverables',
          'fields'    => [
            'course-downloads-template' => [
              'name'        =>  __( 'Template', "wpep" ),
              'alias'       => 'course-downloads-template',
              'type'        => 'select',
              'options'     => [
                WPEP_COURSE_DOWNLOADS_TEMPLATE_STANDARD => __( "Standard", "wpep" ),
                WPEP_COURSE_DOWNLOADS_TEMPLATE_PLAIN    => __( "Plain", "wpep" ),
                WPEP_COURSE_DOWNLOADS_TEMPLATE_ICON     => __( "Icon", "wpep" )
              ],
              'attributes'  => [
                'data-wpep-conditional-display' => 'data-wpep-cs-downloads-template'
              ]
            ],
            'course-downloads-list-title' => [
              'name'        =>  __( 'Title', "wpep" ),
              'alias'       => 'course-downloads-list-title',
              'type'        => 'text_translated'
            ],
            'course-downloads-list-download' => [
              'name'        =>  __( 'Button Text', "wpep" ),
              'alias'       => 'course-downloads-list-download',
              'type'        => 'text_translated',
              'row_data_attributes' => [
                'wpep-cs-downloads-template'  => WPEP_COURSE_DOWNLOADS_TEMPLATE_STANDARD
              ]
            ],
            'course-downloads-position'  => [
              'name'    =>  __( 'Position', "wpep" ),
              'alias'   => 'course-downloads-position',
              'type'    => 'select',
              'options' => [
                'bottom'  => __( "Bottom", "wpep" ),
                'top'     => __( "Top", "wpep" )
              ],
              'row_data_attributes' => []
            ]
          ]
        ],
        'prerequisites' => [
          'name'      => __( "Prerequisites", "wpep" ),
          'has_title' => true,
          'alias'     => 'prerequisites',
          'fields'    => [
            'prerequisite-course-button-lock' => [
              'name'        =>  __( 'Lock Course', "wpep"),
              'alias'       => 'prerequisite-course-button-lock',
              'type'        => 'checkbox',
              'description' => __('Enable this option if you want to restrict students from accessing courses they have not completed the pre-requisites requirement, from the Index Page.', "wpep")
            ],
            'prerequisite-course-button-lock-text' => [
              'name'        =>  __( 'Locked Course Button', "wpep"),
              'alias'       => 'prerequisite-course-button-lock-text',
              'type'        => 'text_translated',
              'description' => __('The text displayed on the Button on the Index Page when the Course is Locked from the above option.', "wpep")
            ],
            'prerequisite-suggested-notification' => [
              'name'        =>  __('Suggested Notification', "wpep"),
              'alias'       => 'prerequisite-suggested-notification',
              'type'        => 'text_translated',
              'description' => __('This Notification is displayed at the top of a given Course if the Student has not yet gone thru items set as prerequisites . %s will be replaced with the title of the course(s) the student has not completed yet.', "wpep")
            ],
            'prerequisite-required-notification' => [
              'name'        =>  __('Required Notification', "wpep"),
              'alias'       => 'prerequisite-required-notification',
              'type'        => 'text_translated',
              'description' => __('This Notification is displayed at the top of a given Course if the Student has not yet gone thru prerequisite courses, %s will be replaced with the course the student wanted to access.', "wpep")
            ],
            'prerequisite-next-courses-title' => [
              'name'        =>  __('Suggested Courses Title', "wpep"),
              'alias'       => 'prerequisite-next-courses-title',
              'type'        => 'text_translated',
              'description' => __('Based on the prerequisites at the end of a given Course the student will get a suggested next content notification.', "wpep" )
            ],
          ]
        ],
        'progress_repeat' => [
          'name'      => __( "Progress Repeat", "wpep" ),
          'has_title' => true,
          'alias'     => 'progress_repeat',
          'fields'    => [
            'enable-progress-repeat-support' => [
              'name'        =>  __( 'Status', "wpep"),
              'alias'       => 'enable-progress-repeat-support',
              'type'        => 'checkbox'
            ],
            'progress-repeat-headline' => [
              'name'        =>  __( 'Title', "wpep"),
              'alias'       => 'progress-repeat-headline',
              'type'        => 'text_translated'
            ],
            'progress-repeat-button' => [
              'name'        =>  __('Button Text', "wpep"),
              'alias'       => 'progress-repeat-button',
              'type'        => 'text_translated'
            ],
          ]
        ],

        'course_download' => [
          'name'      => __( "Course Download", "wpep" ),
          'has_title' => true,
          'alias'     => 'course_download',
          'fields'    => [
            'course-download-button' => [
              'name'        =>  __( 'Button Text', "wpep"),
              'alias'       => 'course-download-button',
              'type'        => 'text_translated'
            ],
          ]
        ],
      ]
    ];
  }

  private function _get_execution_plan_settings() {
    return [
      'name'      => __( 'Execution Plan', "wpep" ),
      'alias'     => 'execution_plan',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'show-lesson-hover-color' => [
              'name'        => __( 'Lesson Hover Color', "wpep" ),
              'alias'       => 'show-lesson-hover-color',
              'type'        => 'checkbox',
              'description' => __( 'Show a different color when the mouse click is on the Lesson.', "wpep" )
            ],
            'lesson-learning-path-placeholder' => [
              'name'        =>  __('Learning Path Placeholder', "wpep"),
              'alias'       => 'lesson-learning-path-placeholder',
              'type'        => 'text_translated',
              'description' => __('If you are building courses that have learning paths, this placeholder should reflect a call to action to select one, for example : Choose Your Learning Path', "wpep")
            ]
          ]
        ],
        'lesson_quick_continue' => [
          'name'      => __( 'Lesson Quick Continue', "wpep" ),
          'has_title' => true,
          'alias'     => 'lesson_quick_continue',
          'fields'    => [
            'show-lesson-continue-button' => [
              'name'        => __( 'Enable Quick Continue', "wpep"),
              'alias'       => 'show-lesson-continue-button',
              'type'        => 'checkbox',
              'description' => __('Display a button at the end of each lesson that allows students to easily continue to the next lesson.', "wpep")
            ],
            'lesson-quick-continue-button-text' => [
              'name'        =>  __( 'Button Text', "wpep"),
              'alias'       => 'lesson-quick-continue-button-text',
              'type'        => 'text_translated'
            ],
            'lesson-quick-continue-final-button-text' => [
              'name'        =>  __( 'Final Button Text', "wpep"),
              'alias'       => 'lesson-quick-continue-final-button-text',
              'type'        => 'text_translated'
            ],
            'lesson-continue-button-position' => [
              'name'    =>  __( 'Button Alignment', "wpep" ),
              'alias'   => 'lesson-continue-button-position',
              'type'    => 'select',
              'options' => [
                "default" => __( "Default", "wpep" ),
                'left'    => __( 'Left', "wpep" ),
                'center'  => __( 'Center', "wpep" ),
                'right'   => __( 'Right', "wpep" )
              ],
              'description' => __( 'Customize the position of the Lesson Quick Continue button.', "wpep" )
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_standard_settings() {
    return [
      'name'      => __( 'Standard', "wpep" ),
      'alias'     => 'standard',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => true,
          'alias'     => 'general',
          'fields'    => [
            'course-standard-allow-index-page' => [
              'name'        => __('Overview Page', "wpep"),
              'alias'       => 'course-standard-allow-index-page',
              'type'        => 'checkbox'
            ],
            'course-standard-allow-index-page-widgets' => [
              'name'        => __('Overview Page Widgets', "wpep"),
              'alias'       => 'course-standard-allow-index-page-widgets',
              'type'        => 'checkbox'
            ],
            'course-standard-redirect-on-completion' => [
              'name'        => __('Redirect on Completion', "wpep"),
              'alias'       => 'course-standard-redirect-on-completion',
              'type'        => 'checkbox',
              'description' => __( "Redirect the Student to the next Lesson on Completion.", "wpep" )
            ]
          ]
        ],
        'text_options' => [
          'name'      => __( 'Text Options', "wpep" ),
          'has_title' => true,
          'alias'     => 'text_options',
          'fields'    => [
            'course-standard-section' => [
              'name'        => __( 'Section', "wpep" ),
              'alias'       => 'course-standard-section',
              'type'        => 'text_translated'
            ],
            'course-standard-section-list' => [
              'name'        => __( 'Section List Title', "wpep" ),
              'alias'       => 'course-standard-section-list',
              'type'        => 'text_translated'
            ],
            'course-standard-lesson' => [
              'name'        => __( 'Lesson', "wpep" ),
              'alias'       => 'course-standard-lesson',
              'type'        => 'text_translated'
            ],
            'course-standard-course-progress' => [
              'name'        => __( 'Course Progress', "wpep" ),
              'alias'       => 'course-standard-course-progress',
              'type'        => 'text_translated'
            ],
            'course-standard-course-navigation' => [
              'name'        => __( 'Course Navigation', "wpep" ),
              'alias'       => 'course-standard-course-navigation',
              'type'        => 'text_translated'
            ],
            'course-standard-start-lesson' => [
              'name'        => __( 'Start Lesson', "wpep" ),
              'alias'       => 'course-standard-start-lesson',
              'type'        => 'text_translated'
            ],
            'course-standard-previous-lesson' => [
              'name'        => __( 'Previous Lesson', "wpep" ),
              'alias'       => 'course-standard-previous-lesson',
              'type'        => 'text_translated'
            ],
            'course-standard-next-lesson' => [
              'name'        => __( 'Next Lesson', "wpep" ),
              'alias'       => 'course-standard-next-lesson',
              'type'        => 'text_translated'
            ],
            'course-standard-complete-lesson' => [
              'name'        => __( 'Complete Lesson', "wpep" ),
              'alias'       => 'course-standard-complete-lesson',
              'type'        => 'text_translated'
            ],
            'course-standard-lesson-completed' => [
              'name'        => __( 'Message Lesson Completed', "wpep"),
              'alias'       => 'course-standard-lesson-completed',
              'type'        => 'text_translated'
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_video_settings() {
    return [
      'name'      => __( 'Video', "wpep" ),
      'alias'     => 'video',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => true,
          'alias'     => 'general',
          'fields'    => [
            'course-video-allow-index-page' => [
              'name'        => __('Overview Page', "wpep"),
              'alias'       => 'course-video-allow-index-page',
              'type'        => 'checkbox'
            ],
            'course-video-redirect-on-completion' => [
              'name'        => __('Redirect on Completion', "wpep"),
              'alias'       => 'course-video-redirect-on-completion',
              'type'        => 'checkbox',
              'description' => __( "Redirect the Student to the next Video on Completion.", "wpep" )
            ]
          ]
        ],
        'text_options' => [
          'name'      => __( 'Text Options', "wpep" ),
          'has_title' => true,
          'alias'     => 'text_options',
          'fields'    => [
            'course-video-course-status' => [
              'name'        => __('Course Status', "wpep"),
              'alias'       => 'course-video-course-status',
              'type'        => 'text_translated'
            ],
            'course-video-course-status-pending' => [
              'name'        => __( 'Course Status Pending', "wpep" ),
              'alias'       => 'course-video-course-status-pending',
              'type'        => 'text_translated'
            ],
            'course-video-course-status-progress' => [
              'name'        => __( 'Course Status Progressing', "wpep" ),
              'alias'       => 'course-video-course-status-progress',
              'type'        => 'text_translated'
            ],
            'course-video-course-status-completed' => [
              'name'        => __( 'Course Status Completed', "wpep" ),
              'alias'       => 'course-video-course-status-completed',
              'type'        => 'text_translated'
            ],
            'course-video-course-progress' => [
              'name'        => __( 'Course Progress', "wpep" ),
              'alias'       =>'course-video-course-progress',
              'type'        => 'text_translated'
            ],
            'course-video-course-table-sections' => [
              'name'        => __( 'Table Sections', "wpep" ),
              'alias'       => 'course-video-course-table-sections',
              'type'        => 'text_translated'
            ],
            'course-video-course-table-status' => [
              'name'        => __( 'Table Status', "wpep" ),
              'alias'       => 'course-video-course-table-status',
              'type'        => 'text_translated'
            ],
            'course-video-start-section' => [
              'name'        => __('Table Start Section', "wpep"),
              'alias'       => 'course-video-start-section',
              'type'        => 'text_translated'
            ],
            'course-video-lesson-description' => [
              'name'        => __( 'Available Downloads', "wpep" ),
              'alias'       => 'course-video-lesson-description',
              'type'        => 'text_translated'
            ],
            'course-video-lesson-description-icon' => [
              'name'    =>  __('Available Downloads Icon', "wpep"),
              'alias'   => 'course-video-lesson-description-icon',
              'type'    => 'select',
              'class'   => 'wpep-select-field',
              'options' => [
                wpep_get_setting( 'course-video-lesson-description-icon' )
                => sprintf( __( "Currently Selected : %s", "wpep" ), wpep_alias_to_label( wpep_get_setting( 'course-video-lesson-description-icon' ) ) )
              ],
              'attributes' => [
                'data-ec-library'      => "select2" ,
                'data-ec-library-mode' => "icon"
              ]
            ],
            'course-video-lesson-back' => [
              'name'        => __('Back to Overview Page', "wpep"),
              'alias'       => 'course-video-lesson-back',
              'type'        => 'text_translated'
            ],
            'course-video-lesson-completed' => [
              'name'        => __('Mark video as watched', "wpep"),
              'alias'       => 'course-video-lesson-completed',
              'type'        => 'text_translated'
            ],
            'course-video-lesson-complete' => [
              'name'        => __('Video has been watched', "wpep"),
              'alias'       => 'course-video-lesson-complete',
              'type'        => 'text_translated'
            ],
            'course-video-previous-lesson' => [
              'name'        => __('Previous Video', "wpep"),
              'alias'       => 'course-video-previous-lesson',
              'type'        => 'text_translated'
            ],
            'course-video-next-lesson' => [
              'name'        => __('Next Video', "wpep"),
              'alias'       => 'course-video-next-lesson',
              'type'        => 'text_translated'
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_focus_settings() {
    return [
      'name'      => __( 'Focus Course', "wpep" ),
      'alias'     => 'focus',
      'field_map' => [
        'overview_options' => [
          'name'      => __( 'Curriculum / Overview', "wpep" ),
          'has_title' => true,
          'alias'     => 'overview_options',
          'fields'    => [
            'course-focus-overview-start' => [
              'name'        => __( 'Start Lesson', "wpep"),
              'alias'       => 'course-focus-overview-start',
              'type'        => 'text_translated'
            ],
            'course-focus-overview-view' => [
              'name'        => __( 'View Lesson', "wpep"),
              'alias'       => 'course-focus-overview-view',
              'type'        => 'text_translated'
            ],
            'course-focus-overview-assessment' => [
              'name'        => __( 'View Assessment', "wpep"),
              'alias'       => 'course-focus-overview-assessment',
              'type'        => 'text_translated'
            ],
          ]
        ],
        'lesson_options' => [
          'name'      => __( 'Lesson Page', "wpep" ),
          'has_title' => true,
          'alias'     => 'lesson_options',
          'fields'    => [
            'course-focus-previous-lesson' => [
              'name'        => __('Previous Lesson', "wpep"),
              'alias'       => 'course-focus-previous-lesson',
              'type'        => 'text_translated'
            ],
            'course-focus-next-lesson' => [
              'name'        => __('Next Lesson', "wpep"),
              'alias'       => 'course-focus-next-lesson',
              'type'        => 'text_translated'
            ],
            'course-focus-complete-and-next-lesson' => [
              'name'        => __( 'Complete and Continue', "wpep"),
              'alias'       => 'course-focus-complete-and-next-lesson',
              'type'        => 'text_translated'
            ],
            'course-focus-complete-lesson' => [
              'name'        => __( 'Complete', "wpep"),
              'alias'       => 'course-focus-complete-lesson',
              'type'        => 'text_translated'
            ],
            'course-focus-discussion-title' => [
              'name'        => __( 'Discussion Title', "wpep"),
              'alias'       => 'course-focus-discussion-title',
              'type'        => 'text_translated'
            ],
            'course-focus-percentage-text' => [
              'name'        => __( 'Percentage Text', "wpep"),
              'alias'       => 'course-focus-percentage-text',
              'type'        => 'text_translated'
            ],
          ]
        ]
      ]
    ];
  }

  private function _get_one_off_settings() {
    return [
      'name'      => __( 'One-Off Course', "wpep" ),
      'alias'     => 'one_off',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'course-basic-complete-course' => [
              'name'        =>  __('Complete Course', "wpep"),
              'alias'       => 'course-basic-complete-course',
              'type'        => 'text_translated'
            ],
            'course-basic-course-completed' => [
              'name'        =>  __('Course Completed', "wpep"),
              'alias'       => 'course-basic-course-completed',
              'type'        => 'text_translated'
            ],
            'course-basic-enable-video-description' => [
              'name'    =>  __( 'Video Description', "wpep" ),
              'alias'   => 'course-basic-enable-video-description',
              'type'    => 'select',
              'options' => wpep_flag_setting_field_options(),
              'description' => __( "Enable an extra editor field on the One-Off Course Builder, that enables you to add content under the video.", "wpep" )
            ],
            'course-basic-jump-link-title' => [
              'name'        =>  __('Jump Links Title', "wpep"),
              'alias'       => 'course-basic-jump-link-title',
              'type'        => 'text_translated'
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_challenge_settings() {
    return [
      'name'      => __( 'Challenge Course', "wpep" ),
      'alias'     => 'challenge',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'course-challenge-section-slug' => [
              'name'        =>  __('URL Slug', "wpep"),
              'alias'       => 'course-challenge-section-slug',
              'type'        => 'text_translated',
              'description' => __( "No spaces allowed", "wpep" )
            ],
            'course-challenge-introduction-text' => [
              'name'        =>  __( 'Introduction Text', "wpep" ),
              'alias'       => 'course-challenge-introduction-text',
              'type'        => 'editor'
            ]
          ]
        ],
        'social_share' => [
          'name'      => __( "Social Share", "wpep" ),
          'has_title' => true,
          'alias'     => 'social_share',
          'fields'    => [
            'course-challenge-social-share-status' => [
              'name'    =>  __('Status', "wpep"),
              'alias'   => 'course-challenge-social-share-status',
              'type'    => 'select',
              'options' => wpep_flag_setting_field_options()
            ],
            'course-challenge-social-share-cta' => [
              'name'    =>  __( 'CTA', "wpep" ),
              'alias'   => 'course-challenge-social-share-cta',
              'type'    => 'editor'
            ],
            'course-challenge-social-share-networks' => [
              'name'    =>  __( 'Networks', "wpep" ),
              'alias'   => 'course-challenge-social-share-networks',
              'type'    => 'multiple_select',
              'options' => wpep_utility_map_to_array_assoc( wpep_utilities_social_share_map(), 'alias', 'alias' )
            ]
          ]
        ]
      ]
    ];
  }

  private function _get_scorm_settings() {
    return [
      'name'      => __( 'Scorm Course', "wpep" ),
      'alias'     => 'scorm',
      'field_map' => [
        'general' => [
          'name'      => __( "General Options", "wpep" ),
          'has_title' => false,
          'alias'     => 'general',
          'fields'    => [
            'course-scorm-button-open' => [
              'name'        =>  __('Start Training', "wpep"),
              'alias'       => 'course-scorm-button-open',
              'type'        => 'text_translated',
              'description' => __( "No spaces allowed", "wpep" )
            ],
            'course-scorm-notification-completed' => [
              'name'        =>  __( 'Course Completed', "wpep" ),
              'alias'       => 'course-scorm-notification-completed',
              'type'        => 'text_translated'
            ]
          ]
        ]
      ]
    ];
  }

}