<?php

namespace WPEP\ContentType\Course\Integration;

use WPEP\ContentType\Skeleton\Integration\Builder as Builder_Integration;
use WPEP\ContentType\Course as Content_Type_Course;
use WPEP\Container\FormFields as WPEP_FormFields_Configuration;

class Builder extends Builder_Integration {

  use WPEP_FormFields_Configuration;

  /**
   * @var Content_Type_Course
   */
  protected $_contentTypeInstance;

  /**
   * @var Builder\General
   */
  public $builderGeneral;
  /**
   * @var Builder\Section
   */
  public $builderSection;
  /**
   * @var Builder\Lesson
   */
  public $builderLesson;

  public $enable_downloads           = true;
  public $enable_assessments         = false;
  public $enable_timing_settings     = false;
  public $enable_on_boarding_options = false;
  public $enable_member_magnet                = false;
  public $enable_presentation_card_settings   = true;
  public $enable_presentation_button_settings = true;
  public $enable_learning_paths               = true;
  public $enable_progress_repeat              = false;
  public $new_course_sections   = 3;
  public $new_course_lessons    = 5;

  public $section_course_type = [];

  /**
   * @param Content_Type_Course $content_type_instance
   */
  public function __construct( $content_type_instance ) {
    parent::__construct( $content_type_instance );
  }

  public function init() {
    $this->set_field_prefix( 'wpep_meta_value' );

    $this->section_course_type    = [
      'assessment_management'  => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . ' ' . WPEP_COURSE_TYPE_FOCUS . ' ' . WPEP_COURSE_TYPE_STANDARD . ' ' . WPEP_COURSE_TYPE_VIDEO . ' ' . WPEP_COURSE_TYPE_ONE_OFF . ' ' . WPEP_COURSE_TYPE_SCORM,
      'course_management'      => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . ' ' . WPEP_COURSE_TYPE_FOCUS . ' ' . WPEP_COURSE_TYPE_STANDARD . ' ' . WPEP_COURSE_TYPE_VIDEO . ' ' . WPEP_COURSE_TYPE_CHALLENGE,
      'course_document'    => WPEP_COURSE_TYPE_ONE_OFF,
      'course_video'       => WPEP_COURSE_TYPE_ONE_OFF,
      'course_downloads'   => WPEP_COURSE_TYPE_ONE_OFF,
      'learning_path'      => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP,
      'section_checkbox'   => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
      'lesson_content'     => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . ' ' . WPEP_COURSE_TYPE_FOCUS . ' ' . WPEP_COURSE_TYPE_STANDARD . ' ' . WPEP_COURSE_TYPE_VIDEO,
      'lesson_downloads'   => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . ' ' . WPEP_COURSE_TYPE_FOCUS . ' ' . WPEP_COURSE_TYPE_STANDARD . ' ' . WPEP_COURSE_TYPE_VIDEO,
      'lesson_video'       => WPEP_COURSE_TYPE_VIDEO,
      'lesson_icon'        => WPEP_COURSE_TYPE_FOCUS
    ];

    $this->enable_downloads       = wpep_get_setting_flag( 'enable-course-deliverables' );
    $this->enable_timing_settings = wpep_get_setting_flag( 'enable-course-timing-option' );
    $this->enable_on_boarding_options = wpep_get_setting_flag( 'enable-course-on-boarding-options' );

    $this->enable_assessments     = wpep_assessments_enabled();
    $this->enable_progress_repeat = wpep_get_setting_flag( 'enable-progress-repeat-support' );

    $this->enable_member_magnet   = wpep_get_setting_flag( 'enable-post-member-magnet-option' );

    $this->new_course_sections    = wpep_get_setting( 'admin-course-preset-sections' );
    $this->new_course_lessons     = wpep_get_setting( 'admin-course-preset-lessons' );

    $this->section_course_type    = apply_filters( "wpep_administration_course_builder_section_functionality", $this->section_course_type );

    $this->builderGeneral = new Builder\General( $this );
    $this->builderSection = new Builder\Section( $this );
    $this->builderLesson  = new Builder\Lesson( $this );

    do_action( 'wpep_administration_course_builder_settings_setup', $this );
  }

  public function display() {
    $current_primary_tab = apply_filters( "wpep_administration_builder_current_tab", intval( get_post_meta( get_the_ID(), 'wpep_current_primary_tab', true ) ), get_the_ID(), WPEP_POST_TYPE_COURSE );

    echo '<div id="wpep_builder">';
    echo    '<input type="hidden" id="wpep_current_primary_tab" name="wpep_current_primary_tab" value="' . $current_primary_tab . '"/>';

    $this->display_from_tab_list( $this->get_tab_list() );

    echo '</div>';
  }

  public function get_tab_list() {
    $response = [
      'present' => [
        'name'  => __( "Present", "wpep" ),
        'slug'  => 'course-presentation',
        'type'  => 'standard',
        'content' => [ $this, '_tab_content_presentation' ]
      ],
      'management' => [
        'name'    => __( "Build", "wpep" ),
        'slug'    => 'course-management',
        'type'    => 'course_management',
        'navigation_attributes' => [
          'data-wpep-course-type-information-block' => $this->section_course_type[ 'course_management' ],
          'data-hidden-callback'                    => "#wpep-tab-links > .wpep-tab-course-presentation",
          'data-visible-callback'                   => "#wpep-tab-links > .wpep-tab-course-management"
        ],
        'content' => [ $this, '_tab_content_management' ]
      ]
    ];

    if( $this->enable_on_boarding_options )
      $response[ 'on_boarding' ] = [
        'name'    => __( "Onboarding", "wpep" ),
        'slug'    => 'course-on-boarding',
        'type'    => 'standard',
        'content' => [ $this, '_tab_content_on_boarding' ]
      ];

    foreach( $this->_contentTypeInstance->get_dependent_content_types() as $dependent_content_type )
      $response = $dependent_content_type->attach_course_builder_tabs( $response, $this );

    return $response;
  }

  public function save_from_post( $course_post_id, $entity_id_map = [] ) {
    if( get_post_type( $course_post_id ) != WPEP_POST_TYPE_COURSE )
      return $entity_id_map;

    if( !isset( $entity_id_map[ 'section' ] ) )
      $entity_id_map[ 'section' ] = [];

    if( !isset( $entity_id_map[ 'lesson' ] ) )
      $entity_id_map[ 'lesson' ] = [];

    $searchable_post_content = '';

    if (isset($_POST['wpep_course_description'])) {
      $course_description = stripcslashes ( $_POST['wpep_course_description'] );

      if( $course_description !== '' )
        update_post_meta( $course_post_id, 'wpep_course_description', $course_description );
      else
        delete_post_meta( $course_post_id, 'wpep_course_description' );

      $searchable_post_content .= $course_description . "\r\n";
    } else {
      $searchable_post_content .= get_post_meta( $course_post_id, 'wpep_course_description', true ) . "\r\n";
    }

    if( isset( $_POST[ 'wpep_meta_value' ][ '_wpep_course_downloads' ] ) ) {
      $course_downloads = $_POST[ 'wpep_meta_value' ][ '_wpep_course_downloads' ];

      if( !is_array( $course_downloads ) ) {
        $course_downloads = [];
      } else {
        foreach( $course_downloads as $download_key => $download ) {
          unset( $course_downloads[ $download_key ] );
          $course_downloads[ urldecode( $download_key ) ] = $download;
        }
      }

      update_post_meta( $course_post_id, '_wpep_course_downloads', $course_downloads );
    }

    if( wpep_course_type_from_id( $course_post_id ) != WPEP_COURSE_TYPE_ONE_OFF ) {
      $entity_id_map = $this->_handle_general_save_functionality( $course_post_id, $entity_id_map, $searchable_post_content );
    } else {
      $jump_link_information = ( isset( $_POST[ 'wpep_meta_value' ][ '_wpep_course_video_jump_links' ] )
        ? $_POST[ 'wpep_meta_value' ][ '_wpep_course_video_jump_links' ]
        : get_post_meta( $course_post_id, '_wpep_course_video_jump_links', true )
      );

      if( !empty( $jump_link_information ) && is_array( $jump_link_information ) ) {
        foreach( $jump_link_information as $jump_link_timestamp => $jump_link_title )
          $searchable_post_content .= $jump_link_title . "\r\n";
      }
    }

    foreach( $this->_contentTypeInstance->get_dependent_content_types() as $dependent_content_type )
      $entity_id_map = $dependent_content_type->on_course_save( $course_post_id, $entity_id_map );

    if( isset( $_POST['wpep_current_primary_tab'] ) )
      update_post_meta( $course_post_id, 'wpep_current_primary_tab', $_POST['wpep_current_primary_tab'] );

    $searchable_post_content = trim( $searchable_post_content );

    wpep_wpdb()->update( wpep_wpdb()->posts, [
      'post_content' => $searchable_post_content
    ], [ 'ID' => $course_post_id ] );

    return $entity_id_map;
  }

  public function _handle_general_save_functionality( $course_post_id, $entity_id_map = [], &$searchable_post_content = '' ) {
    if( isset( $_POST['wpep_deleted_lesson'] ) ) {

      foreach( $_POST['wpep_deleted_lesson'] as $lesson_id ) {
        if( !is_numeric( $lesson_id ) )
          continue;

        wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, [ 'id' => $lesson_id ] );
      }

    }

    if( isset( $_POST['wpep_deleted_section'] ) ) {

      foreach( $_POST['wpep_deleted_section'] as $section_id ) {
        if( !is_numeric( $section_id ) )
          continue;

        wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, [ 'id' => $section_id ] );
        wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, [ 'section_id' => $section_id ] );
      }

    }

    if( isset( $_POST['wpep_course'] ) ) {
      $has_learning_path = 0;
      $lesson_position   = 1;
      $section_position  = 1;
      foreach( $_POST['wpep_course'] as $section_id => $section_information ) {
        if( !isset( $section_information['title'] ) )
          continue;

        if( !is_numeric( $section_id ) && isset( $entity_id_map['section'][$section_id] ) )
          $section_id = $entity_id_map['section'][$section_id];

        $section_options = isset( $section_information['option'] ) && is_array( $section_information['option'] ) ? $section_information['option'] : [];

        if( isset( $section_options[ 'question_answers' ] ) ) {
          foreach ($section_options['question_answers'] as $question_answer_index => $question_answer_value)
            if (trim($question_answer_value) == '')
              unset($section_options['question_answers'][$question_answer_index]);

          if( count( $section_options[ 'question_answers' ] ) >= 2 )
            $has_learning_path = 1;
        }

        $current_information = [
          'post_id' => $course_post_id,
          'title'   => stripcslashes( $section_information['title'] !== '' ? $section_information['title'] : __( "New Section", "wpep" ) ),
          'order'   => ( isset( $section_information['order'] ) ? intval( $section_information['order'] ) : $section_position ),
          'options' => json_encode( $section_options )
        ];

        foreach( [ 'journey_access'     => WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT,
                   'journey_expiration' => WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE,
                   'journey_execution'  => WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE,
                 ] as $field_prefix => $fallback ) {
          if( isset( $section_information[ $field_prefix . '_type' ] ) )
            $current_information[ $field_prefix . '_type' ] = $section_information[ $field_prefix . '_type' ];

          if( isset( $section_information[ $field_prefix . '_segment' ] ) )
            $current_information[ $field_prefix . '_segment' ] = $section_information[ $field_prefix . '_segment' ];

          if( isset( $current_information[ $field_prefix . '_type' ] ) )
            $current_information = wpep_learning_journey_information_pre_save( $current_information, $field_prefix, $fallback );
        }

        if( isset( $section_information['content'] ) )
          $current_information['content'] = stripcslashes( $section_information['content'] );

        if( is_numeric( $section_id ) ) {

          wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, $current_information, [ 'id' => $section_id ] );

          $entity_id_map['section'][$section_id] = $section_id;

        } else {

          wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, $current_information );

          $entity_id_map['section'][$section_id] = wpep_wpdb()->insert_id;

          $section_id = wpep_wpdb()->insert_id;
        }

        $searchable_post_content .= "\r\n" . $current_information['title'];

        if( isset( $current_information['content'] ) )
          $searchable_post_content .= "\r\n" . $current_information['content'];

        $section_position++;

        if( !isset( $section_information['lesson'] ) )
          continue;

        foreach( $section_information['lesson'] as $lesson_id => $lesson_information ) {
          if( !isset( $lesson_information['title'] ) )
            continue;

          if( !is_numeric( $lesson_id ) && isset( $entity_id_map['lesson'][$lesson_id] ) )
            $lesson_id = $entity_id_map['lesson'][$lesson_id];

          $lesson_options = ( isset( $lesson_information['option'] ) && is_array( $lesson_information['option'] ) ? $lesson_information['option'] : [] );

          if( isset( $lesson_options[ 'downloads_list' ] ) && !empty( $lesson_options[ 'downloads_list' ] ) ) {
            foreach( $lesson_options[ 'downloads_list' ] as $lesson_option_key => $lesson_option_value ) {
              unset( $lesson_options[ 'downloads_list' ][ $lesson_option_key ] );
              $lesson_options[ 'downloads_list' ][ urldecode( $lesson_option_key ) ] = $lesson_option_value;
            }
          }

          $current_information = [
            'post_id'     => $course_post_id,
            'section_id'  => $section_id,
            'title'       => stripcslashes( $lesson_information['title'] !== '' ? $lesson_information['title'] : __( "New Lesson", "wpep" ) ),
            'order'       => $lesson_position,
            'options'     => json_encode( $lesson_options )
          ];

          if( isset( $lesson_information['content'] ) )
            $current_information['content'] = stripcslashes($lesson_information['content']);

          foreach( [ 'journey_access'     => WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT,
                     'journey_expiration' => WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE,
                     'journey_execution'  => WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE,
                   ] as $field_prefix => $fallback ) {
            if( isset( $section_information[ $field_prefix . '_type' ] ) )
              $current_information[ $field_prefix . '_type' ] = $lesson_information[ $field_prefix . '_type' ];

            if( isset( $section_information[ $field_prefix . '_segment' ] ) )
              $current_information[ $field_prefix . '_segment' ] = $lesson_information[ $field_prefix . '_segment' ];

            if( isset( $current_information[ $field_prefix . '_type' ] ) )
              $current_information = wpep_learning_journey_information_pre_save( $current_information, $field_prefix, $fallback );
          }

          if( is_numeric( $lesson_id ) ) {

            wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, $current_information, [ 'id' => $lesson_id ] );

            $entity_id_map['lesson'][$lesson_id] = $lesson_id;

          } else {

            wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, $current_information );

            $entity_id_map['lesson'][$lesson_id] = wpep_wpdb()->insert_id;

          }

          $searchable_post_content .= "\r\n" . $current_information['title'];

          if( isset( $current_information['content'] ) )
            $searchable_post_content .= "\r\n" . $current_information['content'];

          $lesson_position++;
        }

        $searchable_post_content .= "\r\n";
      }

      $course_type = isset( $_POST['wpep_meta_value'][ WPEP_COURSE_TYPE_POST_META ] ) ? $_POST['wpep_meta_value'][ WPEP_COURSE_TYPE_POST_META ] :  wpep_course_type_from_id( $course_post_id );

      if( !in_array( $course_type, [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP ] ) )
        $has_learning_path = 0;

      update_post_meta( $course_post_id, 'wpep_learning_path_enabled', $has_learning_path );

    }

    return $entity_id_map;
  }

  public function _tab_content_presentation() {
    $this->builderGeneral->presentation();
  }

  public function _tab_content_management() {
    $this->builderSection->display_management();
  }

  public function _tab_content_on_boarding() {
    $this->builderGeneral->on_boarding();
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    $field_value = get_post_meta( get_the_ID(), $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

}