<?php

namespace WPEP\ContentType\Course\Integration\Builder;

use WPEP\ContentType\Course\Integration\Builder as Course_Builder;

class General {

  /**
   * @var Course_Builder
   */
  public $builderInstance;

  /**
   * Lesson constructor.
   * @param Course_Builder $courseBuilderInstance
   */
  public function __construct( $courseBuilderInstance ) {
    $this->builderInstance = $courseBuilderInstance;
  }

  public function presentation() {
    do_action( "wpep_administration_course_builder_presentation_before", $this->builderInstance );

    echo  '<h2 data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_SCORM . '">' . __( "Scorm Settings", "wpep" ) . '</h2>';
    echo  '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_SCORM . '" class="wpep-form-group wpep-course-description-section">';

    $scorm_course_directory = wpep_course_scorm_directory_path( get_the_ID() );
    $scorm_course_title     = get_post_meta( get_the_ID(), 'wpep_scorm_title', true );

    if( $scorm_course_directory === '' ) {
      echo '<div class="wpep-notification wpep-notification-info">' . __( "Scorm Package Status : Pending, upload a package and update the course...", "wpep_scorm" ) . '</div>';
    } elseif( $scorm_course_directory == false ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . __( "Scorm Package Status : Invalid", "wpep_scorm" ) . '</div>';
    } else {
      echo '<div class="wpep-notification wpep-notification-success">' . __( "Scorm Package Status : Success", "wpep_scorm") . '</div>';
    }

    echo '<div class="wpep-form-group">';
    echo '<div class="wpep-form-group-row">';
    echo    '<label>' . __( "Scorm Package File", "wpep" ) . '</label>';

    $this->builderInstance->display_file_field(
      'wpep_scorm_file',
      false,
      null,
      'regular-text',
      ( $scorm_course_title === '' ? __( "Click to Upload File", "wpep_scorm" ) : $scorm_course_title )
    );

    echo      '</div>';
    echo    '</div>';

    echo  '</div>';

    echo    '<h2>' . __( "Description", "wpep" ) . '</h2>';
    echo    '<div class="wpep-form-group wpep-course-description-section">';
    wpep_editor(
      get_post_meta( get_the_ID(), 'wpep_course_description', true ),
      'wpep_course_description'
    );
    echo    '</div>';

    echo    '<h2 data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['course_document'] . '">' . __( "Document Embed", "wpep" ) . '</h2>';
    echo    '<div class="wpep-form-group textarea wpep-course-video-section" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['course_document'] . '">';
    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Upload Document", "wpep" ) . '</label>';
    $this->builderInstance->display_file_field( 'wpep_course_document_embed' );
    echo        '</div>';
    echo    '</div>';

    echo    '<h2 data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['course_video'] . '">' . __( "Video Embed", "wpep" ) . '</h2>';
    echo    '<div class="wpep-form-group textarea wpep-course-video-section" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['course_video'] . '">';
    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Video URL", "wpep" ) . '</label>';
    $this->builderInstance->display_text_field( 'wpep_course_video' );
    echo        '</div>';
    if( wpep_get_setting_flag( 'course-basic-enable-video-description' ) ) {
      echo '<div class="wpep-form-group-row">';
      echo '<label>' . __("Video Description", "wpep") . '</label>';
      $this->builderInstance->display_editor_field('wpep_course_video_description');
      echo '</div>';
    }

    echo        '<div class="wpep-form-group-row">';
    $this->builderInstance->display_attributes_field_jump_links(
      '_wpep_course_video_jump_links',
      false
    );
    echo        '</div>';
    echo    '</div>';

    if( $this->builderInstance->enable_downloads ) {
      echo    '<h2 data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['course_downloads'] . '">' . __( "Available Deliverables", "wpep" ) . '</h2>';
      echo    '<div class="wpep-form-group wpep-course-downloads-section" data-wpep-course-type-information-block="' . $this->builderInstance->section_course_type['course_downloads'] . '">';
      echo        '<div class="wpep-form-group-row">';
      $this->builderInstance->display_attributes_field_downloads('_wpep_course_downloads', false );
      echo        '</div>';
      echo    '</div>';
    }

    echo    '<div class="wpep-form-group wpep-course-presentation-card">';
    echo       '<h2>' . __( "Content Tab - Presentation Card", "wpep" ) . '</h2>';
    echo       '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Description", "wpep" ) . '</label>';
    $this->builderInstance->display_textarea_field(
      'wpep_course_excerpt',
      false,
      '50px',
      wpep_course_excerpt( get_the_ID(), true )
    );
    echo          '<p class="description">' . __( "Leaving this empty will automatically generate one based on the description", "wpep" ) . '</p>';
    echo        '</div>';
    echo       '<div class="wpep-form-group-row wpep-course-ribbon-text-container">';
    echo          '<label>' . __( "Ribbon Text", "wpep" ) . '</label>';
    $this->builderInstance->display_text_field('wpep_course_ribbon_text' );
    echo          '<p class="description">' . __( "Display an Ribbon on the Course Image.", "wpep" ) . '</p>';
    echo        '</div>';
    echo       '<div class="wpep-form-group-row wpep-show-on-index-container">';
    echo          '<label>' . __( "Show or Hide this Course", "wpep" ) . '</label>';
    $this->builderInstance->display_select_field(
      'wpep_show_on_index',
      [
        1 => __('Show', "wpep"),
        0 => __('Hide', "wpep")
      ],
      false,
      null,
      'wpep-select-field',
      1
    );
    echo          '<p class="description">' . __( "Show or Hide this Course from your Course Index Page.", "wpep" ) . '</p>';
    echo       '</div>';
    echo       '<div class="wpep-form-group-row wpep-coming-soon-container">';
    echo          '<label>' . __( "Coming Soon", "wpep" ) . '</label>';
    $this->builderInstance->display_select_field(
      'wpep_coming_soon',
      wpep_flag_setting_field_options(),
      false
    );
    echo          '<p class="description">' . __( "The Start Course button on the Index Page and Widgets will change to Coming Soon and the link will be removed.", "wpep" ) . '</p>';
    echo        '</div>';
    echo       '<div class="wpep-form-group-row wpep-identity-color-container">';
    echo          '<label>' . __( "Top Border Color", "wpep" ) . '</label>';
    $this->builderInstance->display_color_field(
      'identity_color'
    );
    echo          '<p class="description">' . __( 'Optionally select a color for this course, it will be used as the border color on the index page.', "wpep" ) . '</p>';
    echo        '</div>';

    if( $this->builderInstance->enable_presentation_button_settings ) {
      echo       '<h3>' . __( "Button Settings", "wpep" ) . '</h3>';
      echo       '<div class="wpep-form-group-row wpep-post-button-text-container">';
      echo          '<label>' . __( "Text", "wpep" ) . '</label>';
      $this->builderInstance->display_text_field(
        'wpep_post_button_text',
        false,
        null,
        false,
        wpep_get_setting_translated( 'opt-course-start-text' )
      );
      echo          '<p class="description">' . __( 'The text on the Start Course Button, leaving this empty will default to the one from settings.', "wpep" ) . '</p>';
      echo       '</div>';
      echo       '<div class="wpep-form-group-row wpep-post-button-color-container">';
      echo          '<label>' . __( "Color", "wpep" ) . '</label>';
      $this->builderInstance->display_color_field(
        'wpep_post_button_color'
      );
      echo          '<p class="description">' . __( 'A different color on the Start Course Button, leaving this empty will default to the one from settings.', "wpep" ) . '</p>';
      echo        '</div>';
      echo       '<div class="wpep-form-group-row wpep-post-button-text-color-container">';
      echo          '<label>' . __( "Text Color", "wpep" ) . '</label>';
      $this->builderInstance->display_color_field(
        'wpep_post_button_text_color'
      );
      echo          '<p class="description">' . __( 'A different text color on the Start Course Button, leaving this empty will default to the one from settings.', "wpep" ) . '</p>';
      echo        '</div>';
      echo       '<div class="wpep-form-group-row wpep-post-alternative-link-container">';
      echo          '<label>' . __( "Custom URL", "wpep" ) . '</label>';
      $this->builderInstance->display_text_field(
        'wpep_post_alternative_link',
        false,
        null,
        false,
        __( "Leave this empty to ignore this feature.", "wpep" )
      );
      echo          '<p class="description">' . __( "URL users will be redirected to when clicking the Start Button on the Course Tab. Links must start with http:// or https://.", "wpep" ) . '</p>';
      echo        '</div>';
    }

    echo    '</div>';

    do_action( "wpep_administration_course_builder_presentation_after", $this->builderInstance );
  }

  public function on_boarding() {
    do_action( "wpep_administration_course_builder_on_boarding_before", $this->builderInstance );

    echo    '<h2>' . __( "Course Started - Notification", "wpep" ) . '</h2>';

    $this->builderInstance->display_table_start();

    $this->builderInstance->display_table_select_field(
      __( "Status", "wpep" ),
      'wpep_course_on_boarding_status',
      [
        ''                   => __( "Disabled", "wpep" ),
        'email-notification' => __( "Email Notification", "wpep" )
      ],
      __( "This Notification will be sent to students that start the course, students that already started the course will not receive this notification.", "wpep" ),
      false
    );

    $this->builderInstance->display_table_text_field(
      __( "Title", "wpep" ),
      'wpep_course_on_boarding_title'
    );

    $this->builderInstance->display_table_editor_field(
      __( "Content", "wpep" ),
      'wpep_course_on_boarding_content',
      wpep_controller()->userEngagement->get_available_shortcode_list( false ),
      300
    );

    $this->builderInstance->display_table_end();

    do_action( "wpep_administration_course_builder_on_boarding_after", $this->builderInstance );
  }

}