<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class LearnPress extends Skeleton {

  public $import_entity_sub_type  = 'lp_course';
  public $taxonomy_term_map = [
    'course_category' => WPEP_TAXONOMY_CATEGORY
  ];

  public function get_name() {
    return 'LearnPress';
  }

  public function get_alias() {
    return 'learn_press';
  }

  public function get_default_preferences() {
    return [
      'default_course_type'   => WPEP_COURSE_TYPE_EXECUTION_PLAN
    ];
  }

  public function get_preferences_field_map() {
    return [
      'default_course_type'   => [
        'name'        => __( "Course Type", "wpep" ),
        'type'        => 'select',
        'options'     => wpep_course_layout_types( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP, WPEP_COURSE_TYPE_STANDARD ] ),
        'description' => sprintf( __( "When a the %s course can be split into multiple lessons and sections, this is the course type that will be used.", "wpep" ), $this->get_name() ) . '<br/>' .
                         sprintf( __( "If the system cannot determine multiple lessons or sections in %s, it will be migrated as an One-Off Course.", "wpep" ), $this->get_name() )
      ]
    ];
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    $lp_course_sections = wpep_wpdb()->get_results(
      'SELECT * FROM ' . wpep_wpdb()->prefix . 'learnpress_sections 
                      WHERE section_course_id = ' . absint( $import_entity_id ) . ' 
                   ORDER BY section_order ASC'
    );

    if( empty( $lp_course_sections ) )
      return [];

    foreach( $lp_course_sections as $lp_course_section )
      $response[ 'sections' ][ $lp_course_section->section_id ] = [
        'title'   => $lp_course_section->section_name,
        'content' => $lp_course_section->section_description
      ];

    $lp_course_section_lesson_items = wpep_wpdb()->get_results(
      'SELECT * FROM ' . wpep_wpdb()->prefix . 'learnpress_section_items 
                      WHERE section_id IN ("' . implode( '","', array_keys( $response[ 'sections' ] ) ) . '") 
                        AND item_type = "lp_lesson"
                   ORDER BY item_order ASC'
    );

    if( empty( $lp_course_section_lesson_items ) )
      return [];

    foreach( $lp_course_section_lesson_items as $lp_course_section_lesson ) {
      $lesson_post = get_post( $lp_course_section_lesson->item_id );

      $response[ 'lessons' ][ $lp_course_section_lesson->item_id ] = [
        'import_section_id' => $lp_course_section_lesson->section_id,
        'title'             => $lesson_post->post_title,
        'content'           => $lesson_post->post_content
      ];
    }


    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wp_delete_post( $import_entity_id );
  }

}