<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class Sensei extends Skeleton {

  public $import_entity_type              = "post";
  public $import_entity_sub_type          = 'course';
  public $import_entity_sub_type_meta_key = '_lesson_order';

  public $taxonomy_term_map = [
    'course-category' => WPEP_TAXONOMY_CATEGORY
  ];

  public function get_name() {
    return 'Sensei';
  }

  public function get_alias() {
    return 'sensei';
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $current_lessons    = get_posts( [
      'post_status' => wpep_post_monitored_post_status_list(),
      'numberposts' => -1,
      'post_type'   => 'lesson',
      'meta_key'    => '_lesson_course',
      'meta_value'  => $import_entity_id
    ] );

    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    if( empty( $current_lessons ) )
      return [];

    $current_lessons = wpep_utilities_map_object( $current_lessons, 'ID' );

    $lesson_order_array = explode( ',', get_post_meta( $import_entity_id, '_lesson_order', true ) );

    foreach( $lesson_order_array as $lesson_id ) {
      if( !isset( $current_lessons[ $lesson_id ] ) )
        continue;

      $current_module = wp_get_post_terms( $import_entity_id, 'module' );

      if( $current_module === null || is_wp_error( $current_module ) || !is_array( $current_module ) )
        $current_module = false;
      else
        $current_module = array_shift( $current_module );

      $response['lessons'][ $lesson_id ] = [
        'title'   => $current_lessons[ $lesson_id ]->post_title,
        'content' => $current_lessons[ $lesson_id ]->post_content
      ];

      if( $current_module === false )
        continue;

      $response['lessons'][ $lesson_id ][ 'import_section_id' ] = $current_module->term_id;

      if( !isset( $response['sections'][ $current_module->term_id ] ) )
        $response['sections'][ $current_module->term_id ] = [
          'title'   => $current_module->name,
          'content' => ''
        ];
    }

    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wp_delete_post( $import_entity_id );
  }

}