<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

abstract class Skeleton extends \WPEP_Data_Third_Party_Import_Integration {

  public function get_description() {
    return '<p>' . sprintf( __( "Import your %s Courses into %s", "wpep" ), $this->get_name(), WPEP_PLUGIN_NAME ) . '</p>';
  }

  public function get_default_preferences() {
    return [
      'default_section_title' => __( "Start Here", "wpep" ),
      'default_course_type'   => WPEP_COURSE_TYPE_EXECUTION_PLAN
    ];
  }

  public function get_preferences_field_map() {
    return [
      'default_section_title' => [
        'name'        => __( "Section Title", "wpep" ),
        'type'        => 'text',
        'description' => __( "If you have Courses that use only lessons, then a default section will be created automatically", "wpep" )
      ],
      'default_course_type'   => [
        'name'        => __( "Course Type", "wpep" ),
        'type'        => 'select',
        'options'     => wpep_course_layout_types( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP, WPEP_COURSE_TYPE_STANDARD, WPEP_COURSE_TYPE_FOCUS ] ),
        'description' => sprintf( __( "When a the %s course can be split into multiple lessons and sections, this is the course type that will be used.", "wpep" ), $this->get_name() ) . '<br/>' .
          sprintf( __( "If the system cannot determine multiple lessons or sections in %s, it will be migrated as an One-Off Course.", "wpep" ), $this->get_name() )
      ]
    ];
  }

  public function implementation_import_confirmation_content() {
    $response = '';

    if( isset( $this->import_entity_type ) && $this->import_entity_type === 'post' ) {
      $import_post_type_object = get_post_type_object( $this->import_entity_sub_type );
      $new_post_type_object    = get_post_type_object( WPEP_POST_TYPE_COURSE );

      if( isset( $import_post_type_object->rewrite['slug'] ) && isset( $new_post_type_object->rewrite['slug'] ) ) {
        if( $import_post_type_object->rewrite['slug'] === $new_post_type_object->rewrite['slug'] ) {
          $response .= '<div class="wpep-notification wpep-notification-info">';
          $response .=    sprintf( __( "The %s Course Slug is the same as the %s Course Slug, this is ok if you plan to remove %s after the import has been completed.", "wpep" ), WPEP_PLUGIN_NAME, $this->get_name(), $this->get_name() );
          $response .= '</div>';
        }
      }
    }

    $response .= '<ul class="wpep-process-information-list">';
    $response .=    '<li>' . sprintf( __( "Import Source : %s", "wpep" ), '<strong>' . $this->get_name() . '</strong>' ) . '</li>';
    $response .=    '<li>' . sprintf( __( "Courses to Import : %s", "wpep" ), '<strong>' . count( $this->implementation_get_import_entity_ids() ) . '</strong>' ) . '</li>';
    $response .= '</ul>';

    return $response;
  }

  /**
   * @param $importer_map
   * @param $importer_que
   * @return string
   */
  public function implementation_import_progressing_notification_text( $importer_map, $importer_que ) {
    return sprintf(
      __( "Imported %s/%s Courses from %s", "wpep" ),
      count( $importer_map ),
      ( count( $importer_map ) + count( $importer_que ) ),
      $this->get_name()
    );
  }

  public function implementation_import_success_preview_table_data( $importer_map ) {
    $table_data = [];

    foreach( $importer_map as $import_id => $new_post_id ) {
      $table_data[] = [
        'course_title' => get_the_title( $new_post_id ),
        'course_type'  => wpep_course_type_from_id( $new_post_id ),
        'new_link'     => get_permalink( $new_post_id ),
        'old_link'     => ( $this->import_entity_type === 'post' ? get_permalink( $import_id ) : get_term_link( $import_id, $this->import_entity_sub_type ) )
      ];
    }

    return [
      'head'  => [
        'course_title' => __( "Course Title", "wpep" ),
        'course_type'  => __( "Course Type", "wpep" ),
        'new_link'     => __( "New Link", "wpep" ),
        'old_link'     => __( "Old Link", "wpep" )
      ],
      'body'  => $table_data
    ];
  }

  public function implementation_import_base_entity( $import_entity_id ) {
    if( !is_numeric( $import_entity_id ) )
      return false;

    if( isset( $this->_entity_import_map[ $import_entity_id ] ) )
      return $this->_entity_import_map[ $import_entity_id ];

    if( $this->import_entity_type === 'post' )
      $wpep_post_id = $this->_import_base_entity_type_post( $import_entity_id );
    else
      $wpep_post_id = $this->_import_base_entity_type_taxonomy( $import_entity_id );

    if( $wpep_post_id === false )
      return false;

    update_post_meta( $wpep_post_id, 'wpep_order', 0 );

    $this->import_taxonomies( $import_entity_id, $wpep_post_id );

    return $wpep_post_id;
  }

  public function _import_base_entity_type_post( $import_entity_id ) {
    $import_post = get_post( $import_entity_id );

    $inserted_post = get_object_vars( $import_post );
    unset( $inserted_post['ID'] );
    unset( $inserted_post['guid'] );

    $inserted_post['ping_status']    = 'closed';
    $inserted_post['comment_status'] = 'closed';
    $inserted_post['post_type']      = WPEP_POST_TYPE_COURSE;

    $wpep_post_id = wp_insert_post( $inserted_post );

    if( is_wp_error( $wpep_post_id ) )
      return false;

    $thumbnail_id  = get_post_thumbnail_id( $import_post->ID );

    $thumbnail     = ( !empty( $thumbnail_id ) ? get_post( $thumbnail_id, 'full' ) : '' );

    if( isset( $thumbnail->guid ) )
      update_post_meta( $wpep_post_id, 'wpep_thumb_image', $thumbnail->guid );

    return $wpep_post_id;
  }

  public function _import_base_entity_type_taxonomy( $import_entity_id ) {
    $import_taxonomy = get_term( $import_entity_id, $this->import_entity_sub_type );

    if( $import_taxonomy === null || is_wp_error( $import_taxonomy ) )
      return false;

    $wpep_post_id = wp_insert_post( [
      'post_type'      => WPEP_POST_TYPE_COURSE,
      'post_title'     => $import_taxonomy->name,
      'post_content'   => $import_taxonomy->description,
      'comment_status' => 'closed',
      'ping_status'    => 'closed'
    ] );

    if( is_wp_error( $wpep_post_id ) )
      return false;

    return $wpep_post_id;
  }

  public function import_taxonomies( $import_entity_id, $wpep_post_id ) {
    if( empty( $this->taxonomy_term_map ) )
      return;

    foreach( $this->taxonomy_term_map as $import_taxonomy => $new_taxonomy )
      $this->import_taxonomy( $import_entity_id, $wpep_post_id, $import_taxonomy, $new_taxonomy );
  }

  public function import_taxonomy( $import_entity_id, $wpep_post_id, $import_taxonomy, $new_taxonomy = WPEP_TAXONOMY_CATEGORY ) {
    $term_ids = [];

    $current_categories = wp_get_post_terms( $import_entity_id, $import_taxonomy );

    if( is_wp_error( $current_categories ) )
      return;

    foreach( $current_categories as $current_category ) {
      if( $import_taxonomy == $new_taxonomy ) {
        $term_ids[] = $current_category->term_id;
        continue;
      }

      if( isset( $this->_terms_import_map[ $current_category->term_id ] ) ) {
        $term_ids[] = $this->_terms_import_map[ $current_category->term_id ];
        continue;
      }

      $term_ids[] = $this->_import_taxonomy_with_parents( $current_category, $new_taxonomy );
    }

    if( !empty( $term_ids ) )
      wp_set_post_terms( $wpep_post_id, $term_ids, $new_taxonomy );
  }

  /**
   * @param \WP_Term $migrate_term
   * @param string $new_taxonomy
   * @return array|\WP_Error
   */
  private function _import_taxonomy_with_parents( $migrate_term, $new_taxonomy = WPEP_TAXONOMY_CATEGORY ) {
    $parent_id = ( $migrate_term->parent == 0
      ? 0
      : ( isset( $this->_terms_import_map[$migrate_term->parent] )
          ? $this->_terms_import_map[$migrate_term->parent]
          : $this->_import_taxonomy_with_parents( get_term( $migrate_term->parent, $migrate_term->taxonomy ), $new_taxonomy )
        )
    );

    $inserted_term_id = wp_insert_term( $migrate_term->name, WPEP_TAXONOMY_CATEGORY, [
      'parent'  => $parent_id,
      'slug'    => $migrate_term->slug
    ] );

    if( is_wp_error( $inserted_term_id ) ) {
      if( isset( $inserted_term_id->error_data['term_exists'] ) ) {
        $inserted_term_id = $inserted_term_id->error_data['term_exists'];
      }
    } else {
      $inserted_term_id = $inserted_term_id['term_id'];
    }

    $this->_terms_import_map[ $migrate_term->term_id ] = $inserted_term_id;

    return $inserted_term_id;
  }

  public function implementation_get_import_entity_ids() {
    if( $this->import_entity_type === 'taxonomy' ) {
      $response = get_terms( [
        'taxonomy'    => $this->import_entity_sub_type,
        'hide_empty'  => false,
        'fields'      => 'ids'
      ] );

      if( is_wp_error( $response ) || $response === NULL )
        return [];

      return $response;
    }

    if( $this->import_entity_type === 'post' ) {
      $args = [
        'numberposts' => -1,
        'post_type'   => $this->import_entity_sub_type,
        'fields'      => 'ids',
        'post_status' => wpep_post_monitored_post_status_list(),
      ];

      if( $this->import_entity_sub_type_meta_key !== false ) {
        $args[ 'meta_key' ]     = $this->import_entity_sub_type_meta_key;
        $args[ 'meta_compare']  = 'EXISTS';
      }

      return get_posts( $args );
    }

    return [];
  }

  public function implementation_import_single( $import_entity_id, $wpep_post_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, [ 'post_id' => $wpep_post_id ] );
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, [ 'post_id' => $wpep_post_id ] );

    $single_content_map = $this->implementation_import_single_content_map( $import_entity_id );

    if( empty( $single_content_map ) || empty( $single_content_map['lessons'] ) ) {
      if( !isset( $single_content_map[ 'content' ] ) ) {
        if( $this->import_entity_type !== "post" )
          return;

        $single_content_map[ 'content' ] = get_post( $import_entity_id )->post_content;
      }

      update_post_meta( $wpep_post_id, WPEP_COURSE_TYPE_POST_META, WPEP_COURSE_TYPE_ONE_OFF );
      update_post_meta( $wpep_post_id, 'wpep_course_description', $single_content_map[ 'content' ] );

      if( isset( $single_content_map[ 'downloads' ] ) && !empty( $single_content_map[ 'downloads' ] ) )
        update_post_meta( $wpep_post_id, '_wpep_course_downloads', $single_content_map[ 'downloads' ] );

      wpep_wpdb()->update( wpep_wpdb()->posts, [
        'post_content' => $single_content_map[ 'content' ]
      ], [ 'ID' => $wpep_post_id ] );

      return;
    }

    if( !isset( $single_content_map['lessons'] ) )
      return;

    if( count( $single_content_map['lessons'] ) === 1 ) {
      $current_lesson = array_shift( $single_content_map['lessons'] );

      update_post_meta( $wpep_post_id, WPEP_COURSE_TYPE_POST_META, WPEP_COURSE_TYPE_ONE_OFF );
      update_post_meta( $wpep_post_id, 'wpep_course_description', $current_lesson[ 'content' ] );

      if( isset( $current_lesson[ 'downloads' ] ) && is_array( $current_lesson[ 'downloads' ] ) )
        update_post_meta( $wpep_post_id, '_wpep_course_downloads', $current_lesson[ 'downloads' ] );

      wpep_wpdb()->update( wpep_wpdb()->posts, [
        'post_content' => $current_lesson[ 'content' ]
      ], [ 'ID' => $wpep_post_id ] );

      return;
    }

    update_post_meta( $wpep_post_id, WPEP_COURSE_TYPE_POST_META, ( isset( $this->preferences_import_map[ 'default_course_type' ] ) ? $this->preferences_import_map[ 'default_course_type' ] : WPEP_COURSE_TYPE_EXECUTION_PLAN ) );

    $new_post_post_content = '';

    $section_id_mapping = [];
    $section_order      = 1;
    $lesson_order       = 1;

    foreach( $single_content_map['lessons'] as $current_lesson ) {
      if( !isset( $current_lesson[ 'import_section_id' ] ) )
        $current_lesson[ 'import_section_id' ] = 0;

      if( !isset( $section_id_mapping[ $current_lesson[ 'import_section_id' ] ] ) ) {
        $current_section_title   = ( isset( $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'title' ] )
          ? $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'title' ]
          : ( isset( $this->preferences_import_map[ 'default_section_title' ] ) ? $this->preferences_import_map[ 'default_section_title' ] : __( "Start Here", "wpep" ) )
        );
        $current_section_content = ( isset( $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'content' ] )
          ? $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'content' ]
          : ''
        );

        $section_options = [];

        if( isset( $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'downloads' ] ) && is_array( $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'downloads' ] ) )
          $lesson_options['downloads_list'] = $single_content_map['sections'][ $current_lesson[ 'import_section_id' ] ][ 'downloads' ];

        wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, [
          'post_id' => $wpep_post_id,
          'title'   => $current_section_title,
          'content' => $current_section_content,
          'order'   => $section_order,
          'options' => json_encode( $section_options )
        ] );

        $section_id_mapping[ $current_lesson[ 'import_section_id' ] ] = wpep_wpdb()->insert_id;

        $new_post_post_content .= $current_section_title . "\r\n";
        $new_post_post_content .= $current_section_content . "\r\n";

        $section_order++;
      }


      $lesson_options = [];

      if( isset( $current_lesson[ 'downloads' ] ) && is_array( $current_lesson[ 'downloads' ] ) )
        $lesson_options['downloads_list'] = $current_lesson[ 'downloads' ];

      wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, [
        'post_id'    => $wpep_post_id,
        'section_id' => $section_id_mapping[ $current_lesson[ 'import_section_id' ] ],
        'title'      => $current_lesson['title'],
        'content'    => $current_lesson['content'],
        'order'      => $lesson_order,
        'options'    => json_encode( $lesson_options )
      ] );

      $new_post_post_content .= $current_lesson['title'] . "\r\n";
      $new_post_post_content .= $current_lesson['content'] . "\r\n";

      $lesson_order++;
    }

    wpep_wpdb()->update( wpep_wpdb()->posts, [
      'post_content' => $new_post_post_content
    ], [ 'ID' => $wpep_post_id ] );
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wp_delete_post( $import_entity_id );
  }

}