<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class ThriveApprentice extends Skeleton {

  public $import_entity_type     = 'taxonomy';
  public $import_entity_sub_type = 'tva_courses';

  public function get_name() {
    return 'Thrive Apprentice';
  }

  public function get_alias() {
    return 'thrive_apprentice';
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_get_import_entity_ids() {
    return get_terms( [
      'taxonomy'     => $this->import_entity_sub_type,
      'hide_empty'   => false,
      'fields'       => 'ids',
      'meta_key'     => 'tva_status',
      'meta_compare' => '!=',
      'meta_value'   => 'private'
    ] );
  }

  public function _import_base_entity_type_taxonomy( $import_entity_id ) {
    $wpep_post_id = parent::_import_base_entity_type_taxonomy( $import_entity_id );
    $thumbnail    = get_term_meta( $import_entity_id, 'tva_cover_image', true );

    if( $thumbnail !== '' )
      update_post_meta( $wpep_post_id, 'wpep_thumb_image', $thumbnail );

    return $wpep_post_id;
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    $tva_lessons    = get_posts( [
      'numberposts' => -1,
      'post_type'   => 'tva_lesson',
      'meta_key'    => 'tva_lesson_order',
      'orderby'     => 'meta_value_num',
      'order'       => 'ASC',
      'tax_query'   => [
        $this->import_entity_sub_type => [
          'taxonomy' => $this->import_entity_sub_type,
          'field'    => 'term_id',
          'terms'    => $import_entity_id
        ]
      ]
    ] );

    if( empty( $tva_lessons ) )
      return [];

    $tva_chapters   = get_posts( [
      'numberposts' => -1,
      'post_type'   => 'tva_chapter',
      'meta_key'    => 'tva_chapter_order',
      'orderby'     => 'meta_value_num',
      'order'       => 'ASC',
      'tax_query'   => [
        $this->import_entity_sub_type => [
          'taxonomy' => $this->import_entity_sub_type,
          'field'    => 'term_id',
          'terms'    => $import_entity_id
        ]
      ]
    ] );

    foreach( $tva_chapters as $tva_chapter )
      $response[ 'sections' ][ $tva_chapter->ID ] = [
        'title'   => $tva_chapter->post_title,
        'content' => ( $tva_chapter->post_content === '' ? $tva_chapter->post_excerpt : $tva_chapter->post_content )
      ];

    foreach( $tva_lessons as $tva_lesson )
      $response[ 'lessons' ][ $tva_lesson->ID ] = [
        'import_section_id' => $tva_lesson->post_parent,
        'title'             => $tva_lesson->post_title,
        'content'           => ( $tva_lesson->post_content === '' ? $tva_lesson->post_excerpt : $tva_lesson->post_content )
      ];

    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wp_delete_term( $import_entity_id, $this->import_entity_sub_type );
  }

}