<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class WPCourseWare extends Skeleton {

  public $import_entity_type              = "post";
  public $import_entity_sub_type          = 'wpcw_course';

  public $taxonomy_term_map = [
    'course_category' => WPEP_TAXONOMY_CATEGORY
  ];

  public function get_name() {
    return 'WPCourseWare';
  }

  public function get_alias() {
    return 'wpcourseware';
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    $wpcw_course = wpep_wpdb()->get_row( 'SELECT * FROM ' . wpep_wpdb()->prefix . 'wpcw_courses WHERE course_post_id = ' . intval( $import_entity_id ) );

    $wpcw_modules   = wpep_wpdb()->get_results(
      'SELECT * FROM ' . wpep_wpdb()->prefix . 'wpcw_modules WHERE parent_course_id = ' . $wpcw_course->course_id . ' ORDER BY module_order ASC'
    );

    if( empty( $wpcw_modules) )
      return [];

    $wpce_module_units = wpep_wpdb()->get_results(
      'SELECT * FROM ' . wpep_wpdb()->prefix . 'wpcw_units_meta WHERE parent_course_id = ' . $wpcw_course->course_id . ' ORDER BY unit_order ASC'
    );

    $wpce_module_units = wpep_utilities_map_object( $wpce_module_units, 'parent_module_id', true );

    foreach( $wpcw_modules as $wpcw_module ) {
      if( !isset( $wpce_module_units[ $wpcw_module->module_id ] ) )
        continue;

      $response[ 'sections' ][ $wpcw_module->module_id ] = [
        'title'   => $wpcw_module->module_title,
        'content' => $wpcw_module->module_desc
      ];

      foreach( $wpce_module_units[ $wpcw_module->module_id ] as $wpce_module_unit ) {
        $module_unit_post = get_post( $wpce_module_unit->unit_id );

        $response[ 'lessons' ][ $module_unit_post->ID ] = [
          'import_section_id' => $wpcw_module->module_id,
          'title'             => $module_unit_post->post_title,
          'content'           => $module_unit_post->post_content
        ];
      }
    }

    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wp_delete_post( $import_entity_id );
  }

}