<?php

namespace WPEP\ContentType\Course\ThirdPartyImport;

class WPLMS extends Skeleton {

  public $import_entity_sub_type  = 'course';
  public $import_entity_sub_type_meta_key = 'vibe_course_curriculum';

  public $taxonomy_term_map = [
    'course-cat' => WPEP_TAXONOMY_CATEGORY
  ];

  public function get_name() {
    return 'WPLMS';
  }

  public function get_alias() {
    return 'wplms';
  }

  public function get_content_types_list() {
    return [ WPEP_POST_TYPE_COURSE ];
  }

  public function implementation_import_single_content_map( $import_entity_id ) {
    $course_curriculum = get_post_meta( $import_entity_id, 'vibe_course_curriculum', true );

    if( empty( $course_curriculum ) )
      return [];

    $response = [
      'lessons'   => [],
      'sections'  => [],
    ];

    $current_section_id = 0;

    foreach( $course_curriculum as $current_curriculum_item ) {
      if( is_numeric( $current_curriculum_item ) ) {
        $post = get_post( $current_curriculum_item );

        if( $post !== null ) {
          $response[ 'lessons' ][ $post->ID ] = [
            'import_section_id' => $current_section_id,
            'title'             => $post->post_title,
            'content'           => $post->post_content
          ];

          continue;
        }
      }

      $current_section_id++;

      $response[ 'sections' ][ $current_section_id ] = [
        'title'   => $current_curriculum_item
      ];
    }


    return $response;
  }

  public function implementation_import_single_remove( $import_entity_id ) {
    wp_delete_post( $import_entity_id );
  }

}