<?php

namespace WPEP\ContentType\Course;

class User {

  /**
   * @var User\Data
   */
  public $data;

  /**
   * @var User\Flag
   */
  public $flag;

  /**
   * @var User\Lesson
   */
  public $lesson;

  /**
   * @var User\Section
   */
  public $section;

  public function __construct() {
    $this->data    = new User\Data();
    $this->flag    = new User\Flag();
    $this->lesson  = new User\Lesson();
    $this->section = new User\Section();
  }

  public function get_recommended_ids( $user_id = false, $limit = 4 ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $courses_started   = wpep_utility_map_to_array_assoc( wpep_content_type_course_instance()->user->data->get( WPEP_USER_COURSE_STARTED, false, false, false, $user_id, false ), 'course_id', 'value' );
    $courses_completed = wpep_utility_map_to_array_assoc( wpep_content_type_course_instance()->user->data->get( WPEP_USER_COURSE_COMPLETED_AT, false, false, false, $user_id, false ), 'course_id', 'value' );
    $response = array_diff_key( $courses_started, $courses_completed);

    if( empty( $response ) )
      return apply_filters( 'wpep_user_activity_recommended_course_ids', [], $user_id, $limit, $courses_started, $courses_completed );

    arsort( $response, SORT_NUMERIC );

    $response = array_keys( $response );

    return apply_filters( 'wpep_user_activity_recommended_course_ids', array_slice( $response, 0, $limit ), $user_id, $limit, $courses_started, $courses_completed );
  }

  public function get_progress_percent( $course_id, $user_id = false, $option_section_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $course_progress = $this->data->get( WPEP_USER_COURSE_PROGRESS, $course_id, false, false, $user_id );

    if( $course_progress !== null && isset( $course_progress->value ) && $option_section_id == false ) {
      $course_progress = floatval( $course_progress->value );

    } else if( get_post_meta( $course_id, 'wpep_learning_path_enabled', true ) ) {
      $course_progress = floatval( 0 );
    } else {
      $course_progress = $this->get_progress_percent_raw( $course_id, $user_id, $option_section_id );
    }


    return apply_filters( 'wpep_user_activity_course_progress_percentage', $course_progress, $course_id, $user_id, $course_progress );
  }

  public function get_progress_percent_raw( $course_id, $user_id = false, $option_section_id = false ) {
    $where_query = 'l.post_id = ' . $course_id;

    if( is_numeric( $option_section_id ) && $option_section_id != false )
      $where_query = 'l.section_id = ' . $option_section_id;
    else if( is_array( $option_section_id ) )
      $where_query = 'l.section_id IN ( ' . implode( ',', $option_section_id ) . ' )';

    $sql = 'SELECT ( SUM(IF(lc.id IS NULL, 0, 1)) / COUNT(l.id) * 100 ) as completed_percent, 
                   COUNT(l.id) as lesson_count
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE ' . $where_query . '
              ORDER BY `order` ASC';

    return round( floatval( wpep_wpdb()->get_var( $sql ) ), 2 );
  }

  public function reset_progress( $course_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $sql = 'DELETE lc FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE l.post_id = ' . intval( $course_id );

    wpep_wpdb()->query( $sql );

    foreach( [ WPEP_USER_COURSE_COMPLETED_AT,
               WPEP_USER_COURSE_COMPLETED,
               WPEP_USER_COURSE_PROGRESS,
               WPEP_USER_COURSE_STARTED ,
               WPEP_USER_COURSE_LATEST_INTERACTION
             ] as $data_key )
      wpep_content_type_course_instance()->user->data->delete( $data_key, null, $course_id, null, null, $user_id );

    wpep_content_type_course_instance()->user->data->add(WPEP_USER_COURSE_REPEAT_TRACKER, time(), $course_id, 0, 0, $user_id );

    if( empty( wpep_content_type_course_instance()->get_dependent_content_types() ) )
      return true;

    foreach( wpep_content_type_course_instance()->get_dependent_content_types() as $dependent_content_type )
      $dependent_content_type->on_course_progress_reset( $course_id, $user_id );

    return true;
  }

  public function get_started_courses_count( $user_id = false ) {
    return count( $this->get_started_course_list( $user_id ) );
  }

  public function get_completed_course_count( $user_id = false ) {
    return count( $this->get_completed_course_list( $user_id ) );
  }

  public function get_completed_lesson_count( $user_id = false, $completed_from = false, $completed_to = false ) {
    return count( $this->get_completed_lesson_list( $user_id, $completed_from, $completed_to ) );
  }

  /**
   * @param bool|int $user_id
   * @param bool|int $started_at
   * @param bool|int $ids_only
   * @return array
   */
  public function get_started_course_list( $user_id = false, $started_at = false, $ids_only = true ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( wpep_content_type_course_instance()->user->data->get(
      WPEP_USER_COURSE_STARTED, false, false, false, $user_id, false, $started_at
    ) as $user_data_course_started ) {
      if( $ids_only ) {
        $response[] = $user_data_course_started->course_id;

        continue;
      }

      $response[$user_data_course_started->course_id] = $user_data_course_started->value;
    }

    return $response;
  }

  /**
   * @param bool $user_id
   * @return array
   */
  public function get_progressing_course_list( $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( wpep_content_type_course_instance()->user->data->get(
      WPEP_USER_COURSE_COMPLETED, false, false, false, $user_id, false, 0
    ) as $user_data_course_completed ) {
      $response[] = $user_data_course_completed->course_id;
    }

    return $response;
  }

  /**
   * @param bool|int $user_id
   * @param bool|int $completed_at
   * @return array
   */
  public function get_completed_course_list( $user_id = false, $completed_at = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( wpep_content_type_course_instance()->user->data->get(
      WPEP_USER_COURSE_COMPLETED_AT, false, false, false, $user_id, false, $completed_at
    ) as $user_data_course_completed ) {
      $response[] = $user_data_course_completed->course_id;
    }

    return $response;
  }

  /**
   * @param bool $user_id
   * @param int $is_bookmarked
   * @return array
   */
  public function get_bookmarked_course_list( $user_id = false, $is_bookmarked = 1 ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( wpep_content_type_course_instance()->user->data->get(
      WPEP_USER_COURSE_IS_BOOKMARKED, false, false, false, $user_id, false, $is_bookmarked
    ) as $user_data_course_completed ) {
      $response[] = $user_data_course_completed->course_id;
    }

    return $response;
  }

  public function get_completed_lesson_list( $user_id = false, $completed_from = false, $completed_to = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $sql = 'SELECT lc.lesson_id
             FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc 
             WHERE lc.user_id = ' . intval( $user_id );

    if( $completed_from !== false )
      $sql .= ' AND lc.created_at >= "' . date( "Y-m-d H:i:s", $completed_from) . '"';

    if( $completed_to !== false )
      $sql .= ' AND lc.created_at <= "' . date( "Y-m-d H:i:s", $completed_to) . '"';

    return wpep_wpdb()->get_col( $sql );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @return string|null
   */
  public function get_delivered_section_id( $course_id, $user_id = false ) {
    return $this->_get_delivered_variable( $course_id, $user_id, 'l.section_id' );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @return string|null
   */
  public function get_delivered_lesson_id( $course_id, $user_id = false ) {
    return $this->_get_delivered_variable( $course_id, $user_id, 'l.id' );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @param string $param
   * @return string|null
   */
  private function _get_delivered_variable( $course_id, $user_id = false, $param = 'l.id' ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT ' . $param . '
                  FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
              LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
                     ON l.`id`       = lc.`lesson_id`
                    AND lc.`user_id` = ' . intval( $user_id ) . '
              LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` s
                     ON s.`id` = l.`section_id`
              WHERE lc.`id` IS NULL
                AND l.`post_id` = ' . intval( $course_id ) . ' ORDER BY l.`order` ASC';


    $resp = wpep_wpdb()->get_var($sql);

    if( is_null( $resp ) ) {
      $sql = 'SELECT ' . $param . '
                  FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
              LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
                     ON l.`id`       = lc.`lesson_id`
                    AND lc.`user_id` = ' . intval( $user_id ) . '
              LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` s
                     ON l.`section_id` = s.`id`
              WHERE l.`post_id` = ' . intval( $course_id ) . ' ORDER BY l.`order` ASC';

      $resp = wpep_wpdb()->get_var($sql);
    }

    return $resp;
  }

}