<?php

namespace WPEP\ContentType\Course\User;

class Data {

  private $_storage = [];

  /**
   * @param array|string $keys
   * @param array|int $course_ids
   * @param int $user_id
   */
  public function preload( $keys, $course_ids, $user_id ) {
    $user_data_list = $this->get( $keys, $course_ids, false, false, $user_id, false, false );

    if( !isset( $this->_storage[ $user_id ] ) )
      $this->_storage[ $user_id ] = [];

    $keys = ( !is_array( $keys ) ? [ $keys ] : $keys );
    $course_ids = ( !is_array( $course_ids ) ? [ $course_ids ] : $course_ids );

    foreach( $course_ids as $course_id ) {
      if( !isset( $this->_storage[ $user_id ][ $course_id ] ) )
        $this->_storage[ $user_id ][ $course_id ] = [];

      foreach( $keys as $key )
        $this->_storage[ $user_id ][ $course_id ][ $key ] = null;
    }

    foreach( $user_data_list as $user_data )
      $this->_storage[ $user_id ][ $user_data->course_id ][ $user_data->key ] = $user_data;
  }

  public function get_value( $key = false, $course_id = false, $section_id = false, $lesson_id = false, $user_id = false, $single_row = true ) {
    if( $single_row ) {
      $response = null;

      $user_data = $this->get( $key, $course_id, $section_id, $lesson_id, $user_id, true );

      if( $user_data !== null )
        return $user_data->value;

      return $response;
    }

    $response = [];

    foreach( $this->get( $key, $course_id, $section_id, $lesson_id, $user_id, false ) as $user_data )
      $response[$user_data->id] = $user_data->value;

    return $response;
  }

  public function get( $key = false, $course_id = false, $section_id = false, $lesson_id = false, $user_id = false, $single_row = true, $value = false ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;

      if( $course_id !== false && $section_id === false && $lesson_id === false ) {
        if( isset( $this->_storage[ $user_id ][ $course_id ] )
          && array_key_exists( $key, $this->_storage[ $user_id ][ $course_id ] ) )
          return $this->_storage[ $user_id ][ $course_id ][ $key ];
      }
    }

    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud WHERE 1 = 1 ';

    if( $user_id !== false )
      $sql .= ' AND cud.user_id = ' . intval( $user_id );

    if( $key !== false )
      $sql .= ' AND cud.key ' . (
        is_array( $key ) && !empty( $key )
          ? ' IN ("' . implode( '","', array_map( 'esc_sql', $key ) ) . '")'
          : ' = "' . esc_sql( $key ) . '"'
        );

    if( $course_id !== false )
      $sql .= ' AND cud.course_id ' . ( is_array( $course_id ) ? ' IN (' . implode( ',', array_map( 'intval', $course_id ) ) . ')' : ' = ' . intval( $course_id ) );

    if( $section_id !== false )
      $sql .= ' AND cud.section_id = ' . intval( $section_id );

    if( $lesson_id !== false )
      $sql .= ' AND cud.lesson_id  = ' . intval( $lesson_id );

    if( $value !== false ) {
      if( is_array( $value ) && isset( $value[0] ) && isset( $value[1] )  ) {
        if( $value[0] !== false )
          $sql .= ' AND cud.value  >= ' . intval( $value[0] );
        if( $value[1] !== false )
          $sql .= ' AND cud.value  <= ' . intval( $value[1] );
      } else {
        $sql .= ' AND cud.value  = ' . intval( $value );
      }
    }

    if( $single_row ) {
      $response = wpep_wpdb()->get_row( $sql );

      if( $section_id === false && $lesson_id === false ) {
        if( !isset( $this->_storage[ $user_id ] ) )
          $this->_storage[ $user_id ] = [];

        if( !isset( $this->_storage[ $user_id ][ $course_id ] ) )
          $this->_storage[ $user_id ][ $course_id ] = [];

        $this->_storage[ $user_id ][ $course_id ][ $key ] = $response;
      }

      return $response;
    }

    return wpep_wpdb()->get_results( $sql );
  }

  public function set( $key, $value, $course_id, $section_id, $lesson_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $course_id  = intval( $course_id );
    $section_id = intval( $section_id );
    $lesson_id  = intval( $lesson_id );

    $existent_entry = $this->get( $key, $course_id, $section_id, $lesson_id, $user_id, true );

    $value_updated = true;

    if( $existent_entry === null
      || $existent_entry === false )
      $this->add( $key, $value, $course_id, $section_id, $lesson_id, $user_id );
    else if( $existent_entry->value != $value ) {
      wpep_wpdb()->update(
        wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA,
        [ 'value' => $value ],
        [ 'id'    => $existent_entry->id ]
      );
    } else {
      $value_updated = false;
    }

    if( !$value_updated )
      return true;

    unset( $this->_storage[ $user_id ][ $course_id ][ $key ] );

    do_action( "wpep_user_set_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry );

    return true;
  }

  public function add( $key, $value, $course_id, $section_id = 0, $lesson_id = 0, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $course_id  = intval( $course_id );
    $section_id = intval( $section_id );
    $lesson_id  = intval( $lesson_id );

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA, [
      'user_id'    => $user_id,
      'course_id'  => $course_id,
      'section_id' => $section_id,
      'lesson_id'  => $lesson_id,
      'key'        => $key,
      'value'      => $value,
    ] );

    do_action( "wpep_user_add_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id );
  }

  public function delete( $key = null, $value = null, $course_id = null, $section_id = null, $lesson_id = null, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $delete_where = [ 'user_id' => $user_id ];

    if( $key !== null )
      $delete_where['key'] = $key;

    if( $value !== null )
      $delete_where['value'] = $value;

    if( $course_id !== null )
      $delete_where['course_id'] = intval( $course_id );

    if( $section_id !== null )
      $delete_where['section_id'] = intval( $section_id );

    if( $lesson_id !== null )
      $delete_where['lesson_id'] = intval( $lesson_id );

    wpep_wpdb()->delete(
      wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA,
      $delete_where
    );

    unset( $this->_storage[ $user_id ][ $course_id ][ $key ] );

    do_action( "wpep_user_remove_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id );
  }

}