<?php

namespace WPEP\ContentType\Course\User;

class Flag {

  public function set_status_completed( $course_id, $user_id, $update_progress = true ) {
    wpep_content_type_course_instance()->user->data->set(
      WPEP_USER_COURSE_COMPLETED,
      1,
      $course_id,
      0,
      0,
      $user_id
    );

    wpep_content_type_course_instance()->user->data->set(
      WPEP_USER_COURSE_COMPLETED_AT,
      time(),
      $course_id, 0, 0, $user_id
    );

    if( $update_progress )
      wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_PROGRESS, 100, $course_id, 0, 0, $user_id);
  }

  public function set_status_incomplete( $course_id, $user_id, $update_progress = true ) {
    wpep_content_type_course_instance()->user->data->set(
      WPEP_USER_COURSE_COMPLETED,
      0,
      $course_id,
      0,
      0,
      $user_id
    );

    wpep_content_type_course_instance()->user->data->delete(
      WPEP_USER_COURSE_COMPLETED_AT,
      null,
      $course_id, 0, 0, $user_id
    );

    if( $update_progress )
      wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_PROGRESS, 0, $course_id, 0, 0, $user_id);
  }

  public function get_last_interaction_by_flag( $key = WPEP_USER_COURSE_STARTED, $user_id = false, $plain = true ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . ' cud 
                    WHERE cud.user_id = ' . intval( $user_id ) . '
                      AND cud.key = "' . esc_sql( $key ) . '"
                 ORDER BY cud.value DESC';

    $entry = wpep_wpdb()->get_row( $sql );

    if( $entry === null )
      return '-';

    if( $plain )
      return get_the_title( $entry->course_id ) . ' ' . wpep_format_date_with_time( $entry->value );

    return '<span data-wpep-information="title"><a href="' . get_permalink( $entry->course_id ) . '">' . get_the_title( $entry->course_id ) . '</a></span> ' .
           '<span data-wpep-information="date-time">' . wpep_format_date_with_time( $entry->value ) . '</span>';
  }

  public function has_completed_course( $course_id = false, $user_id = false, $including_assessments = false ) {
    $course_id = ( $course_id == false ? get_the_ID()          : $course_id );
    $user_id   = ( $user_id   == false ? get_current_user_id() : $user_id );

    if( $course_id == 0 || $user_id == 0 )
      return false;

    $is_completed = wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_COMPLETED, $course_id, false, false, $user_id, true );

    if( ( $is_completed === null ? false : intval( $is_completed ) ) == false )
      return false;

    if( $including_assessments == false )
      return true;

    if( !wpep_assessments_enabled() )
      return true;

    $remaining_assessments = wpep_get_assessment_requirements_for_course( $course_id, $user_id );

    return empty( $remaining_assessments );
  }

  public function set_section_learning_path( $course_id, $section_id, $learning_path_index, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    wpep_content_type_course_instance()->user->data->set( 'learning_path', $learning_path_index, $course_id, $section_id, 0, $user_id );
    wpep_content_type_course_instance()->user->data->set( WPEP_USER_COURSE_LATEST_INTERACTION, time(), $course_id, 0, 0, $user_id );

    do_action( "wpep_user_set_section_learning_path", $course_id, $section_id, $learning_path_index, $user_id );
  }

  public function get_bookmark_status( $course_id, $user_id = false ) {
    return wpep_content_type_course_instance()->user->data->get_value(
      WPEP_USER_COURSE_IS_BOOKMARKED,
      $course_id,
      false,
      false,
      $user_id,
      true
    );
  }


  public function set_bookmark_status( $course_id, $is_bookmarked, $user_id = false ) {
    return wpep_content_type_course_instance()->user->data->set(
      WPEP_USER_COURSE_IS_BOOKMARKED,
      intval( $is_bookmarked ),
      $course_id,
      false,
      false,
      $user_id
    );
  }

}