<?php

namespace WPEP\ContentType\Course\User;

use WPEP\Entity\Course as Entity_Course;

class Lesson {

  private $_lesson_cache = [
    'get' => []
  ];

  /**
   * @param $lesson_id
   * @param bool $user_id
   * @param bool $fetch_next_previous
   * @return array|null|object
   */
  public function get( $lesson_id, $user_id = false, $fetch_next_previous = true ) {
    if( !is_numeric( $lesson_id ) )
      return null;

    if( $user_id === false )
      $user_id = get_current_user_id();

    if( isset( $this->_lesson_cache[ 'get' ][ $lesson_id . '_' . $user_id ] ) )
      return $this->_lesson_cache[ 'get' ][ $lesson_id . '_' . $user_id ];

    $sql = 'SELECT l.*,
                   s.post_id as course_id,
                   s.id      as section_id,
                   s.title   as section_title,
                   IF(lc.id IS NULL, 0, 1) as is_completed 
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . ' s ON s.id = l.section_id ';

    $sql .=  'WHERE l.id = ' . $lesson_id . ' LIMIT 1';

    $course_lesson = wpep_wpdb()->get_row($sql);

    if( $course_lesson === null )
      return null;

    if( $fetch_next_previous ) {
      $raw_query = ' SELECT inner_lesson.*,
                            IF( inner_lesson_completed.id IS NULL, 0, 1 ) as user_has_completed
                              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' inner_lesson
                         LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . ' inner_section
                                ON inner_lesson.section_id = inner_section.id
                         LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' inner_lesson_completed
                                ON inner_lesson_completed.lesson_id = inner_lesson.id
                               AND inner_lesson_completed.user_id   = ' . intval( $user_id ) . '
                             WHERE inner_lesson.post_id = ' . $course_lesson->post_id;

      $previous_lesson = wpep_wpdb()->get_row( $raw_query . ' AND inner_lesson.order < ' . $course_lesson->order . ' ORDER BY inner_lesson.order DESC' );
      $next_lesson = wpep_wpdb()->get_row( $raw_query . ' AND inner_lesson.order > ' . $course_lesson->order . ' ORDER BY inner_lesson.order ASC' );

      $course_lesson->previous_lesson_id         = ( $previous_lesson === null ? null : $previous_lesson->id );
      $course_lesson->previous_lesson_title      = ( $previous_lesson === null ? null : $previous_lesson->title );
      $course_lesson->previous_lesson_section_id = ( $previous_lesson === null ? null : $previous_lesson->section_id );
      $course_lesson->previous_lesson_is_completed = ( $previous_lesson === null ? null : $previous_lesson->user_has_completed );

      $course_lesson->next_lesson_id           = ( $next_lesson === null ? null : $next_lesson->id );
      $course_lesson->next_lesson_title        = ( $next_lesson === null ? null : $next_lesson->title );
      $course_lesson->next_lesson_section_id   = ( $next_lesson === null ? null : $next_lesson->section_id );
      $course_lesson->next_lesson_is_completed = ( $next_lesson === null ? null : $next_lesson->user_has_completed );
    }

    $course_lesson->is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $course_lesson->is_completed, $course_lesson->id, $user_id, $course_lesson );

    $this->_lesson_cache[ 'get' ][ $lesson_id . '_' . $user_id ] = $course_lesson;

    return $course_lesson;
  }

  /**
   * @param $section_id
   * @param bool $user_id
   * @return array|null|object
   */
  public function get_all_by_section_id( $section_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $section_id = intval( $section_id );

    $sql = 'SELECT l.*, 
                   IF(lc.id IS NULL, 0, 1) as is_completed,
                   IF(lc.created_at IS NULL, 0, lc.created_at) as completed_at
                FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l
               LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                    ON l.id       = lc.lesson_id
                   AND lc.user_id = ' . intval( $user_id ) . '
               WHERE l.section_id = ' . $section_id;

    $sql .= ' ORDER BY l.`order` ASC';

    $response = wpep_wpdb()->get_results($sql);

    foreach( $response as $response_key => $response_value )
      $response[$response_key]->options = ( $response_value->options == '' ? new \stdClass() : json_decode( $response_value->options ) );

    return $response;
  }

  public function get_completed_timestamp( $course_id, $user_id = false, $order = 'ASC' ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT lc.created_at
                  FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE l.post_id = ' . intval( $course_id ) . '      
              ORDER BY lc.created_at ' . ( $order == 'ASC' ? 'ASC' : 'DESC' );

    $date = wpep_wpdb()->get_var( $sql );

    return ( is_string( $date ) ? strtotime( $date ) : 0 );
  }

  /**
   * @param $lesson_id
   * @param bool $user_id
   * @return array|null|object
   */
  public function get_next_lesson_id( $lesson_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT l.*
              FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' l
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . ' s ON s.id = l.section_id ';

    $sql .=  'WHERE l.id = ' . $lesson_id . ' LIMIT 1';

    $course_lesson = wpep_wpdb()->get_row($sql);

    if( $course_lesson === null )
      return null;

    $raw_query = ' SELECT inner_lesson.* 
                            FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` inner_lesson
                       LEFT JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` inner_section
                              ON inner_lesson.`section_id` = inner_section.`id`
                           WHERE inner_lesson.`post_id` = ' . $course_lesson->post_id;

    $next_lesson = wpep_wpdb()->get_row( $raw_query . ' AND inner_lesson.`order` > ' . $course_lesson->order . ' ORDER BY inner_lesson.`order` ASC' );

    return ( $next_lesson === null ? 0 : $next_lesson->id );
  }

  public function set_status( $lesson_id, $is_completed = true, $user_id = false, $created_at = false, $set_progress_flags = true ) {
    $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

    if( $user_id == 0 ) {
      do_action( "wpep_user_activity_lesson_status", $lesson_id, $is_completed, $user_id );
      return;
    }

    $lesson_id    = intval( $lesson_id );
    $is_completed = intval( $is_completed );
    $course_id    = Entity_Course::instance()->get_course_id_by_lesson_id( $lesson_id );

    $existent_flag = wpep_wpdb()->get_row( 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . ' WHERE lesson_id = ' . $lesson_id . ' AND user_id = ' . $user_id );

    if( $existent_flag !== null ) {
      if( $is_completed )
        return;

      wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED, [
        'user_id'   => $user_id,
        'lesson_id' => $lesson_id
      ] );
    }

    if( $is_completed ) {
      wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED, [
        'user_id'    => $user_id,
        'lesson_id'  => $lesson_id,
        'created_at' => ( $created_at == false ? date( "Y-m-d H:i:s", time() ) : $created_at )
      ] );
    }

    if( $set_progress_flags ) {
      wpep_content_type_course_instance()->user->data->set(
        WPEP_USER_COURSE_PROGRESS,
        wpep_content_type_course_instance()->user->get_progress_percent_raw( $course_id, $user_id ),
        $course_id,
        0,
        0,
        $user_id
      );

      if( wpep_content_type_course_instance()->user->get_progress_percent( $course_id, $user_id ) == 100 ) {
        wpep_content_type_course_instance()->user->flag->set_status_completed( $course_id, $user_id, false );
      } else {
        wpep_content_type_course_instance()->user->flag->set_status_incomplete( $course_id, $user_id, false );
      }

    }

    $started_at = wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_STARTED, $course_id, false, false, $user_id, true );

    if( $started_at === null )
      wpep_content_type_course_instance()->user->data->set( WPEP_USER_COURSE_STARTED, time(), $course_id, 0, 0, $user_id );

    wpep_content_type_course_instance()->user->data->set( WPEP_USER_COURSE_LATEST_INTERACTION, time(), $course_id, 0, 0, $user_id );

    do_action( "wpep_user_activity_lesson_status", $lesson_id, $is_completed, $user_id );

    if( isset( $this->_lesson_cache[ 'get' ][ $lesson_id . '_' . $user_id ] ) )
      unset( $this->_lesson_cache[ 'get' ][ $lesson_id . '_' . $user_id ] );
  }

}