<?php

namespace WPEP\ContentType;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class ListItem extends Skeleton\Library {

  /**
   * @return ListItem\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new ListItem\LearningJourney( $this );
  }

  public function get_post_supports() {
    return [ 'title', 'thumbnail', "custom-fields" ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_LIST_ITEM;
  }

  public function get_singular_name() {
    return __( 'List Item', "wpep" );
  }

  public function get_plural_name() {
    return __( 'List Items', "wpep" );
  }

  public function get_label_all_items() {
    return __( 'All List Items', "wpep" );
  }

  public function get_index_page_settings_field_map() {
    return [
      'frontend' => [
        'name'      => __( "Frontend", "wpep" ),
        'has_title' => true,
        'alias'     => 'frontend',
        'fields'    => [
          'index-page-list-item-button-position'  => [
            'name'    =>  __( 'Button Position', "wpep" ),
            'alias'   => 'index-page-list-item-button-position',
            'type'    => 'select',
            'options' => [ 'left' => __( 'Left' ), 'center' => __( 'Center' ), 'right' => __( 'Right' ) ]
          ]
        ]
      ],
    ];
  }

  public function get_settings_defaults() {
    return [
      'index-page-list-item-button-position' => 'left',
    ];
  }

  public function standard_get_post_meta_fields() {
    return [
      [
        'name'        => __( 'Description', "wpep" ),
        'id'          => 'wpep_content_description',
        'type'        => 'editor'
      ],
      [
        'name'        => __( 'Link', "wpep" ),
        'id'          => 'wpep_content_link',
        'type'        => 'text',
        'description' => __( 'The link where people will be taken when they redeem the offer.', "wpep" )
      ],
      [
        'name'        => __( 'Link Text', "wpep" ),
        'id'          => 'wpep_content_link_title',
        'type'        => 'text',
        'description' => __( 'The text displayed on the button.', "wpep" )
      ],
      [
        'name'    => __('Button Color', "wpep"),
        'id'      => 'wpep_post_button_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different color for the button, leave this empty for default behaviour.', "wpep" )
      ],
      [
        'name'    => __('Button Text Color', "wpep"),
        'id'      => 'wpep_post_button_text_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different text color for the button, leave this empty for default behaviour.', "wpep" )
      ],
    ];
  }

  /**
   * @return mixed
   */
  public function get_cap_name() {
    return WPEP_POST_CAP_LIST_ITEM;
  }

  public function is_publicly_queryable() {
    return false;
  }

  public function library_get_search_details( $current_details, $post ) {
    $current_details['permalink'] = apply_filters( 'wpep_primary_content_item_link', wpep_controller()->actionTracking->get_event_url( WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK, $post->ID ), $post->ID );

    $thumbnail_id = get_post_thumbnail_id( $post->ID );

    if( !empty( $thumbnail_id ) )
      $current_details['thumbnail'] = get_post( $thumbnail_id, 'full' )->guid;

    return $current_details;
  }

  public function get_grid_content_options() {
    return [
      'list_items'         => [
        'name'          => __( "List Items", "wpep" ),
        'type'          => 'multiple_select',
        'options'       => [],
        'description'   => __( "If you want to Display specific List Items Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" ),
        'attributes'    => [
          'data-ec-library'      => 'select2',
          "data-ec-library-mode" => 'post-search-map|post_type:' . $this->get_post_type()
        ]
      ],
      'list_items_per_row' => [
        'name'    => __( "List Items Per Row", "wpep" ),
        'type'    => 'select',
        'options' => [
          ''  => __( "Default", "wpep" ),
          1   => __( "One per row", "wpep" ),
          0   => __( "Do not display", "wpep" )
        ]
      ],
    ];
  }

  public function process_grid_content_options( $filter_proxy_list, $attributes ) {
    $list_items_per_row       = isset($attributes['list_items_per_row']) ? intval( $attributes['list_items_per_row'] )  : '';
    $list_item_list_ids       = isset($attributes['list_items'])         ? $attributes['list_items']  : '';

    if( $list_item_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_list_item_list_args'] = function ($args) use ($list_item_list_ids) {
        $args['post__in'] = wp_parse_id_list( $list_item_list_ids );

        return $args;
      };
    }

    if( $list_items_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_list_items_per_row'] = function( $current_per_row ) use ( $list_items_per_row ) {
        return $list_items_per_row;
      };

    return $filter_proxy_list;
  }

  public function library_output_grid_content_html( $course_categories = [], $post_list = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_list_items_per_row', 1 );

    if( $items_per_row == 0 )
      return;

    if( empty( $post_list ) ) {
      $post_list = $this->library_get_grid_post_list( $course_categories );

      if( empty( $post_list ) )
        return;
    }

    echo '<div class="wpep-list-item-grid-items-container" data-wpep-content-type="' . $this->get_post_type() . '" data-items-per-row="' . $items_per_row . '">';

    foreach( $post_list as $key => $current_list_item ) {
      $thumbnail_id = get_post_thumbnail_id( $current_list_item->ID );

      $link      = wpep_controller()->actionTracking->get_event_url( WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK, $current_list_item->ID );
      $link_text = get_post_meta( $current_list_item->ID, 'wpep_content_link_title', true );

      echo wpep_controller()->template->get_template( 'primary/item-list-item.php', [
        'list_item_id'    => $current_list_item->ID,
        'title'           => get_the_title( $current_list_item ),
        'description'     => get_post_meta( $current_list_item->ID, 'wpep_content_description', true ),
        'image'           => ( !empty( $thumbnail_id ) ? get_post( $thumbnail_id, 'full' )->guid : '' ),
        'link'            => apply_filters( 'wpep_primary_content_item_link', $link, $current_list_item->ID ),
        'link_title'      => apply_filters( 'wpep_primary_content_item_link_title', $link_text, $current_list_item->ID )
      ] );
    }
    echo '<div class="wpep-clearfix"></div>';
    echo '</div>';
  }

  public function library_allow_content() {
    return apply_filters( 'wpep_primary_content_grid_list_items_per_row', 1 ) != 0;
  }

}