<?php

namespace WPEP\ContentType\Skeleton\Integration;

use WPEP\ContentType\Skeleton\Skeleton as Content_Type_Skeleton;

abstract class Builder {

  /**
   * @var Content_Type_Skeleton
   */
  protected $_contentTypeInstance;

  /**
   * LearningJourney constructor.
   * @param Content_Type_Skeleton $content_type_instance
   */
  public function __construct( $content_type_instance ) {
    $this->_contentTypeInstance = $content_type_instance;
  }

  abstract public function display();

  public function display_from_tab_list( $tab_list ) {
    echo    '<ul id="wpep-tab-links">';

    $i = 1; foreach( $tab_list as $tab_key => $tab_information ) {
      echo '<li class="wpep-tab-link wpep-tab-' . $tab_information[ 'slug' ] . ( $i === 1 ? ' active' : '' ) . '"';

      if( isset( $tab_information[ 'navigation_attributes'] ) && !empty( $tab_information[ 'navigation_attributes'] ) ) {
        foreach( $tab_information[ 'navigation_attributes'] as $attribute_key => $attribute_value )
          echo ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
      }

      echo '><h4>' . $tab_information[ 'name' ] . '</h4></li>';

      $i++;
    }

    echo    '</ul>';

    //if ( get_current_screen()->action != 'add' )
    //  echo    '<span class="wpep-post-quick-save" data-text="' . __( "Quick Save", "wpep" ) . '"></span>';

    echo    '<span class="wpep-toggle-large-builder"></span>';
    echo    '<div class="clear"></div>';

    echo    '<div id="wpep-tabs">';

    foreach( $tab_list as $tab_key => $tab_information ) {
      echo '<div class="wpep-tab-item wpep-tab-' . $tab_information[ 'slug' ] . '" data-wpep-tab="' . ( isset( $tab_information[ 'type' ] ) ? $tab_information[ 'type' ] : 'standard' ) . '">';

      if( isset( $tab_information[ 'content' ] ) ) {
        echo is_array( $tab_information[ 'content' ] ) || is_callable( $tab_information[ 'content' ] ) ? call_user_func( $tab_information[ 'content' ] ) : $tab_information[ 'content' ];
      } else {
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "This tab has not been properly configured.", "wpep" ) . '</div>';
      }

      echo '</div>';
    }

    echo    '</div>';
  }

  /**
   * @param $post_id
   * @param array $entity_id_map
   * @return mixed
   */
  abstract public function save_from_post( $post_id, $entity_id_map = [] );

}