<?php

namespace WPEP\ContentType\Skeleton;

use WPEP\Settings as WPEP_Settings;

abstract class Library extends Standard {

  private $_query_grid_posts_data = [];

  /**
   * @return array
   */
  abstract public function get_index_page_settings_field_map();

  /**
   * @param $filter_proxy_list
   * @param $attributes
   * @return array
   */
  abstract public function process_grid_content_options( $filter_proxy_list, $attributes );

  abstract public function get_grid_content_options();

  /**
   * @param array $course_categories
   * @param array $post_list
   *
   * @return mixed
   */
  public function library_output_grid_content_html( $course_categories = [], $post_list = [] ) {
    echo '<p>' . __( __CLASS__ . " method library_output_grid_content_html needs to be implemented" ) . '</p>';
  }

  public function library_allow_content() {
    return true;
  }

  public function library_get_search_details( $current_details, $post ) {
    $current_details['permalink'] = apply_filters( 'wpep_primary_content_item_link', get_permalink( $post ) , $post->ID );

    return $current_details;
  }

  /**
   * @param array $course_categories
   * @return \WP_Post[]|array
   */
  public function library_get_grid_post_list( $course_categories = [] ) {
    $post_list = get_posts( $this->library_get_grid_post_query_args( $course_categories ) );

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $post_list = wpep_post_list_ordering( $post_list );

    return $post_list;
  }

  /**
   * @param array $course_categories
   * @param bool|int $posts_per_page
   * @param int $offset
   * @param string $search
   * @return \WP_Query
   */
  public function library_get_grid_posts( $course_categories = [], $posts_per_page = false, $offset = 0, $search = '' ) {
    $query_args = $this->library_get_grid_post_query_args( $course_categories );

    if( $posts_per_page === false ) {
      $posts_query = new \WP_Query( $query_args );

      return $posts_query;
    }

    $query_args['nopaging']         = true;
    $query_args['offset']           = 0;
    $query_args['no_found_rows']    = false;

    if( $search != '' ) {
      $query_args['s'] = $search;
      unset( $query_args['orderby'] );
      unset( $query_args['order'] );
    }

    $query_args['supress_filters'] = false;

    $this->_query_grid_posts_data = [
      'offset'          => $offset,
      'posts_per_page'  => $posts_per_page
    ];

    add_filter( 'split_the_query', '__return_true', 999 );
    add_filter( 'found_posts', [ $this, '_library_get_grid_posts_filter_found_posts' ], 1, 2 );

    $posts_pagination_query = new \WP_Query( $query_args );

    remove_filter( 'split_the_query', '__return_true', 999 );
    remove_filter( 'found_posts', [ $this, '_library_get_grid_posts_filter_found_posts' ], 1 );

    $this->_query_grid_posts_data = [];

    return $posts_pagination_query;
  }

  /**
   * @param $found_posts
   * @param \WP_Query $wp_query
   * @return mixed
   */
  public function _library_get_grid_posts_filter_found_posts( $found_posts, $wp_query ) {
    $this->_query_grid_posts_data['total'] = count( $wp_query->posts );

    if( $this->_query_grid_posts_data['offset'] > $found_posts )
      $wp_query->posts = [];
    else
      $wp_query->posts = array_slice( $wp_query->posts, $this->_query_grid_posts_data['offset'], $this->_query_grid_posts_data['posts_per_page'] );

    return $found_posts;
  }

  /**
   * @param array $course_categories
   * @return array
   */
  public function library_get_grid_post_query_args( $course_categories = [] ) {
    $meta_query = [];

    if( apply_filters( 'wpep_primary_content_verify_index_page_flag', true ) ) {
      $meta_query['wpep_show_on_index'] = [
        'relation' => 'OR',
        [
          'key'     => 'wpep_show_on_index',
          'value'   => '1',
          'compare' => '=',
        ],
        [
          'key'     => 'wpep_show_on_index',
          'compare' => 'NOT EXISTS'
        ]
      ];
    }

    $args = [
      'post_type'   => $this->get_post_type(),
      'post_status' => 'publish',
      'order'       => 'ASC',
      'orderby'     => [
        'meta_value' => 'ASC',
        'date'       => 'DESC'
      ],
      'meta_key'    => 'wpep_order',
      'meta_type'   => 'NUMERIC',
      'nopaging'    => true,
      'meta_query'  => $meta_query
    ];

    if ( !empty( $course_categories ) ) {
      $args['tax_query'] = [
        WPEP_TAXONOMY_CATEGORY => [
          'taxonomy' => WPEP_TAXONOMY_CATEGORY,
          'field'    => 'term_id',
          'terms'    => $course_categories
        ]
      ];
    }

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $args['post_status'] = wpep_post_monitored_post_status_list();

    $args = apply_filters( 'wpep_primary_content_list_args', $args, $this->get_post_type() );
    $args = apply_filters( 'wpep_primary_content_' . $this->get_post_type() . '_list_args', $args );

    return $args;
  }

  public function library_register_settings() {
    WPEP_Settings::instance()->defaultOptions[ 'content-type-' . $this->get_post_type() . '-enabled' ]      = 1;

    if( !$this->is_publicly_queryable() )
      return;

    WPEP_Settings::instance()->defaultOptions[ 'content-type-' . $this->get_post_type() . '-rewrite-slug' ] = $this->get_post_type();
    WPEP_Settings::instance()->defaultOptions[ 'content-type-' . $this->get_post_type() . '-enable-archive' ] = 1;
  }

  public function is_enabled() {
    return wpep_get_setting( 'content-type-' . $this->get_post_type() . '-enabled', false );
  }

}