<?php

namespace WPEP\ContentType\Skeleton;

abstract class Standard extends Skeleton {

  abstract public function get_post_supports();

  public function is_publicly_queryable() {
    return true;
  }

  public function has_duplicate() {
    return true;
  }

  public function standard_get_post_meta_fields() {
    return [];
  }

  public function standard_get_post_meta_fields_context() {
    return 'normal';
  }

  public function get_post_type_labels() {
    return [
      'name'               => _x( $this->get_singular_name(), 'Post Type General Name', "wpep"),
      'singular_name'      => _x( $this->get_singular_name(), 'Post Type Singular Name', "wpep"),
      'menu_name'          => $this->get_label_all_items(),
      'name_admin_bar'     => $this->get_singular_name(),
      'parent_item_colon'  => sprintf( __( 'Parent %s:', "wpep"), $this->get_singular_name() ),
      'add_new'            => sprintf( __( 'New %s', "wpep" ), $this->get_singular_name() ),
      'add_new_item'       => sprintf( __( 'Add New %s', "wpep" ), $this->get_singular_name() ),
      'new_item'           => sprintf( __( 'New %s', "wpep" ), $this->get_singular_name() ),
      'edit_item'          => sprintf( __( 'Edit %s', "wpep" ), $this->get_singular_name() ),
      'update_item'        => sprintf( __( 'Update %s', "wpep" ), $this->get_singular_name() ),
      'view_item'          => sprintf( __( 'View %s', "wpep" ), $this->get_singular_name() ),
      'all_items'          => $this->get_label_all_items(),
      'search_items'       => sprintf( __( 'Search %s', "wpep" ), $this->get_plural_name() ),
      'not_found'          => sprintf( __( 'No %s found', "wpep" ), $this->get_plural_name() ),
      'not_found_in_trash' => sprintf( __( 'No %s found in Trash', "wpep" ), $this->get_plural_name() )
    ];
  }

  public function register_post_type() {
    $taxonomies = [ WPEP_TAXONOMY_CATEGORY ];

    if( wpep_get_setting_flag( 'register-default-categories' ) )
      $taxonomies[] = 'category';

    $post_supports = $this->get_post_supports();

    if( wpep_get_setting_flag( 'enable-post-author-field' ) )
      $post_supports[] = "author";

    $post_params = [
      'supports'            => $post_supports,
      'taxonomies'          => $taxonomies,
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => 2,
      'menu_icon'           => '',
      'can_export'          => true,
      'exclude_from_search' => false,
      'publicly_queryable'  => $this->is_publicly_queryable(),
      'map_meta_cap'        => true,
      'has_archive'         => ( wpep_get_setting_flag( 'content-type-' . $this->get_post_type() . '-enable-archive' )
        ? wpep_get_setting( 'content-type-' . $this->get_post_type()  . '-rewrite-slug', $this->get_post_type() )
        : false
      ),
      'singular_name'   => $this->get_singular_name(),
      'plural_name'     => $this->get_plural_name(),
      'description'     => sprintf( __( '%s information pages', "wpep" ), $this->get_singular_name() ),
      'label'           => strtolower( $this->get_singular_name() ),
      'labels'          => $this->get_post_type_labels(),
      'capability_type' => $this->get_cap_name()
    ];

    $post_params['rewrite'] = [
      'slug'  => wpep_get_setting( 'content-type-' . $this->get_post_type()  . '-rewrite-slug', $this->get_post_type() )
    ];

    register_post_type( $this->get_post_type(), $post_params);
  }

  public function get_settings_defaults() {
    return [];
  }

  public function data_asset_map() {
    $response = [];

    if( in_array( 'editor', $this->get_post_supports() ) )
      $response['post_information'] = [ 'post_content' ];

    if( in_array( 'thumbnail', $this->get_post_supports() ) )
      $response['meta'] = [ '_thumbnail_id' ];


    return $response;
  }

  /**
   * @param \WP_Post|int|bool $post
   * @return string
   */
  public function get_post_permalink( $post ) {
    return get_permalink( $post );
  }

}