<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Assets {

  private $_has_registered_cropper_support = false;

  public function setup() {
    add_action( 'wp_enqueue_scripts', [ $this, 'frontend_assets' ], 5 );
  }

  public function frontend_assets() {
    $current_theme_information = WPEP_Controller::instance()->templateStyle->get_current_theme();
    $custom_style_file_dependency = [ 'wpep-front-style' ];

    $front_css_file       = apply_filters('wpep_frontend_style_file', $current_theme_information['style_file'] );
    $front_css_expiration = apply_filters('wpep_frontend_style_tag', 'primary-' . WPEP_VERSION);

    wp_enqueue_style('wpep-front-style', $front_css_file, [ 'dashicons' ], $front_css_expiration);

    $primary_script_dependencies = [ 'jquery', 'elearncommerce' ];
    $primary_script_variables    = [
      'ajax_url'     => admin_url( 'admin-ajax.php' ),
      'is_logged_in' => '' . ( is_user_logged_in() ? 1 : 0 ),
      'misc' => [
        'date_format'   => ( !wpep_get_setting_flag( 'admin-library-datetimepicker-simple-format' ) ? get_option( 'date_format' ) : 'Y-m-d' ),
        'resize_wistia' => wpep_get_setting( 'course-resize-wistia-embeds' ),
      ],
      'emoji' => [
        'url_prefix' => apply_filters( 'emoji_url', 'https://s.w.org/images/core/emoji/12.0.0-1/72x72/' ),
        'url_suffix' => apply_filters( 'emoji_ext', '.png' )
      ],
      'index_page_filter_require_all' => intval( wpep_get_setting_flag( 'index-page-categories-filter-require-all') ),
      'assessment_status_completed'   => ( defined( 'WPEP_USER_ASSESSMENT_STATUS_COMPLETED' ) ? WPEP_USER_ASSESSMENT_STATUS_COMPLETED : 'completed' ),
      'course_pre_auth_tracking'      => '' . intval( wpep_get_setting_flag( 'course-pre-authentication-tracking' ) ),
      'libraries'                     => [
        'inputmask'     => WPEP_BASE_URL_PATH . 'assets/jquery.inputmask.min.js',
        'scorm_player'  => WPEP_BASE_URL_PATH . 'assets/wpep-scorm-player.min.js',
      ]
    ];

    if( WPEP_Controller::instance()->discussion->has_integrations() ) {
      $primary_script_variables[ 'discussions' ] = [
        'has_reactions'     => 0,
        'refresh_interval'  => intval( wpep_get_setting( 'comments-refresh-ms-interval' ) ) * 1000,
        'lang' => [
          'feature_comment'       => wpep_get_setting_translated( 'comments-button-feature-comment' ),
          'authority_badge_name'  => wpep_get_setting_translated( 'comments-authority-badge-name' ),
          'reply'                 => wpep_get_setting_translated( 'comments-button-reply' ),
          'delete'                => wpep_get_setting_translated( 'comments-button-delete' ),
          'confirm_delete'        => wpep_get_setting_translated( 'comments-button-confirm-delete' ),
          'view_profile'          => wpep_get_setting_translated( 'comments-reactions-popup-view-profile' )
        ]
      ];

      if( wpep_get_setting_flag( 'comments-reactions-status' ) ) {
        $primary_script_variables[ 'discussions' ][ 'has_reactions' ] = 1;
        $primary_script_variables[ 'discussions' ][ 'lang' ][ 'reaction_text' ] = wpep_get_setting( 'comments-reactions-button-text' );
        $primary_script_variables[ 'discussions' ][ 'misc' ][ 'default_reaction' ] = wpep_get_setting( 'comments-reactions-default-reaction' );
        $primary_script_variables[ 'discussions' ][ 'misc' ][ 'reactions' ] = wpep_get_setting( 'comments-reactions-available-emoji-list' );
      }
    }

    if( wpep_has_engagement_embed_support() ) {
      wp_enqueue_script('clipboard', WPEP_BASE_URL_PATH . 'assets/clipboard.min.js', [], '2.0.0' , true);
      $primary_script_dependencies[] = 'clipboard';

      $primary_script_variables['external_embed_helper_script'] = WPEP_BASE_URL_PATH . 'assets/front/embed-support-min.js';
    }

    wpep_register_universal_interface();

    /* enqueue Script. */
    wp_register_script( 'wpep-front-primary', WPEP_BASE_URL_PATH . 'assets/front/primary-min.js', $primary_script_dependencies, WPEP_VERSION, true);
    wp_localize_script( 'wpep-front-primary', 'wpep', $primary_script_variables );

    wp_enqueue_script( 'wpep-front-primary');

    $custom_style_file = ( wpep_get_setting_flag( 'wpep-custom-css-by-file' ) ? wpep_get_custom_style_file() : false );

    if( $custom_style_file != false ) {
      wp_enqueue_style(
        'wpep-front-custom-style',
        $custom_style_file,
        $custom_style_file_dependency,
        filemtime( wpep_get_custom_style_file( 'basedir' ) )
      );

      WPEP_Controller::instance()->templateStyle->custom_style_delivered = true;
    }
  }

  public function enqueue_cropper_support() {
    if( $this->_has_registered_cropper_support )
      return;

    wp_enqueue_script( 'jquery.cropper', WPEP_BASE_URL_PATH . 'assets/cropper.min.js', [], '1.4.3', true);

    $this->_has_registered_cropper_support = true;
  }

}