<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FilterProxy as FilterProxy;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Authentication {

  use FilterProxy;

  public function setup() {
    if( isset( $_GET[ 'wpep-express-authentication' ] ) )
      add_action( 'init', [ $this, '_express_authentication_request' ] );

    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'profile_authentication', [ $this, 'get_form' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'simple_redirect', [ $this, 'simple_redirect' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_is_logged_in', [ $this, 'user_is_logged_in' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_not_logged_in', [ $this, 'user_not_logged_in' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_name', [ $this, 'user_name' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'user_first_name', [ $this, 'user_first_name' ] );
  }

  public function _express_authentication_request() {
    if( !isset( $_GET[ 'email' ] ) || !isset( $_GET[ 'token' ] ) )
      exit( __( "Invalid Request", "wpep" ) );

    $email = urldecode( $_GET[ 'email' ] );

    if( md5( AUTH_SALT . $email ) !== $_GET[ 'token' ] )
      exit( __( "Invalid Request", "wpep" ) );


    if( $_GET[ 'wpep-express-authentication' ] === 'login' ) {
      $user = get_user_by('email', $email );

      if( false === $user )
        exit( __( "Invalid Request", "wpep" ) );

      if( !isset( $_GET[ 'key' ] ) )
        exit( __( "Invalid Request", "wpep" ) );

      if( intval( get_user_meta( $user->ID, 'wpep_express_authentication_timestamp', true ) ) + HOUR_IN_SECONDS < time() )
        exit( __( "Invalid Request", "wpep" ) );

      if( get_user_meta( $user->ID, 'wpep_express_authentication_key', true ) !== $_GET[ 'key' ] )
        exit( __( "Invalid Request", "wpep" ) );

      delete_user_meta( $user->ID, 'wpep_express_authentication_key' );
      delete_user_meta( $user->ID, 'wpep_express_authentication_timestamp' );
    } else {

      if( !isset( $_GET[ 'password' ] ) )
        exit( __( "Invalid Request", "wpep" ) );

      $user_id = wp_create_user( md5( $email ), $_GET[ 'password' ], $email );

      if( is_wp_error( $user_id ) )
        exit( $user_id->get_error_message() );

      $user = get_user_by( 'ID', $user_id );

      wp_update_user( [
        'ID'           => $user->ID,
        'first_name'   => __( "Student", "wpep" ),
        'display_name' => __( "Student", "wpep" )
      ] );
    }

    wp_set_current_user( $user->ID );
    wp_set_auth_cookie( $user->ID );
    do_action( 'wp_login', $user->user_login, $user );

    WPEP_Entity_ActionTracking::instance()->track_event( $user->ID, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_EXPRESS_REQUEST, '' );

    $redirect = ( isset( $_GET[ 'redirect' ] ) ? urldecode( $_GET[ 'redirect' ] ) : get_site_url() );

    wpep_redirect( $redirect );
    exit;
  }

  public function get_form($attributes, $content = null) {
    if( get_current_user_id() != 0 && !isset( $attributes[ 'ignore_auth' ] ) )
      return WPEP_Controller::instance()->content->userProfile->get_content( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $filter_proxy = [];

    if( isset( $attributes[ 'success_type' ] ) ) {
      $filter_proxy[ 'wpep_user_profile_authentication_form_success_response_type' ] = $attributes[ 'success_type' ];

      if( $attributes[ 'success_type' ] == "redirect" && isset( $attributes[ 'url' ] ) )
        $filter_proxy[ 'wpep_user_profile_authentication_form_success_response_redirect' ] = $attributes[ 'url' ];
    }

    if( isset( $attributes[ 'hide_registration' ] ) )
      $filter_proxy[ 'wpep_user_profile_authentication_form_has_register' ] = ( intval( $attributes[ 'hide_registration' ] ) ? false : true );

    if( isset( $attributes[ 'default_view' ] ) )
      $filter_proxy[ 'wpep_user_profile_authentication_form_view' ] = $attributes[ 'default_view' ];

    $this->_filter_proxy_setup( $filter_proxy );

    $response = WPEP_Controller::instance()->template->get_template( 'authentication.php',
      [
        'include_header'   => false,
        'include_footer'   => false,
        'lead_segment'     => ( isset( $attributes[ 'lead_segment' ] ) ? $attributes[ 'lead_segment' ] : '' )
      ]
    );

    $this->_filter_proxy_remove();

    return $response;
  }

  public function simple_redirect( $attributes, $content ) {
    if( trim( $content ) == '' )
      return '<div class="wpep-notification wpep-notification-danger">' . sprintf( __( "%s - simple redirect shortcode missing url", "wpep" ), WPEP_PLUGIN_NAME ) . '</div>';

    return '<script>document.location = "' . trim( $content ) . '";</script>';
  }

  public function user_is_logged_in( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return '';

    return do_shortcode( $content );
  }

  public function user_not_logged_in( $attributes, $content = '' ) {
    if( is_user_logged_in() )
      return '';

    return do_shortcode( $content );
  }

  public function user_name( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return $content;

    return wpep_content_user_profile_format_name( wp_get_current_user() );
  }

  public function user_first_name( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return $content;

    $user = wp_get_current_user();

    return ( isset( $user->first_name ) ? $user->first_name : '' );
  }


}