<?php

namespace WPEP\Coordinator\Content;

use WPEP\Controller as WPEP_Controller;

class EmailMarketing {

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'email_marketing_tag_add', [ $this, 'tag_add' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'email_marketing_tag_remove', [ $this, 'tag_remove' ] );
  }

  public function tag_add( $attributes, $content ) {
    $tag         = isset( $attributes[ 'tag' ] ) && trim( $attributes[ 'tag' ] ) != ''                 ? sanitize_text_field( $attributes[ 'tag' ] ) : false;
    $integration = isset( $attributes[ 'integration' ] ) && trim( $attributes[ 'integration' ] ) != '' ? sanitize_text_field( $attributes[ 'integration' ] ) : false;

    if( get_current_user_id() === 0 )
      return '';

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ) {
      if( $tag === false )
        return '<div class="wpep-notification wpep-notification-info">' . __( "Tag is missing", "wpep" ) . '</div>';

      if( isset( $_POST[ 'wpep-preview-content' ] ) ) {
        if( $integration !== false ) {
          return '<div class="wpep-notification wpep-notification-info">' . sprintf( __("Preview detected, will not add tag %s for %s.", "wpep" ), $tag, $integration ) . '</div>';
        } else {
          return '<div class="wpep-notification wpep-notification-info">' . sprintf( __("Preview detected, will not add tag %s.", "wpep" ), $tag ) . '</div>';
        }
      }
    }

    if( $tag === false )
      return '';

    if( $integration !== false ) {
      $integration_instance = WPEP_Controller::instance()->emailMarketing->get_integration( $integration );

      if( $integration_instance !== false )
        $integration_instance->add_tag( $tag, get_current_user_id() );
    } else {
      WPEP_Controller::instance()->emailMarketing->tag_user( $tag, get_current_user_id() );
    }

    return '';
  }

  public function tag_remove( $attributes, $content ) {
    $tag         = isset( $attributes[ 'tag' ] ) && trim( $attributes[ 'tag' ] ) != ''                 ? sanitize_text_field( $attributes[ 'tag' ] ) : false;
    $integration = isset( $attributes[ 'integration' ] ) && trim( $attributes[ 'integration' ] ) != '' ? sanitize_text_field( $attributes[ 'integration' ] ) : false;

    if( get_current_user_id() === 0 )
      return '';

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ) {
      if( $tag === false )
        return '<div class="wpep-notification wpep-notification-info">' . __( "Tag is missing", "wpep" ) . '</div>';

      if( isset( $_POST[ 'wpep-preview-content' ] ) ) {
        if( $integration !== false ) {
          return '<div class="wpep-notification wpep-notification-info">' . sprintf( __( "Preview detected, will not remove tag %s for %s.", "wpep" ), $tag, $integration ) . '</div>';
        } else {
          return '<div class="wpep-notification wpep-notification-info">' . sprintf( __( "Preview detected, will not remove tag %s.", "wpep" ), $tag ) . '</div>';
        }
      }
    }

    if( $tag === false )
      return '';

    if( $integration !== false ) {
      $integration_instance = WPEP_Controller::instance()->emailMarketing->get_integration( $integration );

      if( $integration_instance !== false )
        $integration_instance->remove_tag( $tag, get_current_user_id() );
    } else {
      WPEP_Controller::instance()->emailMarketing->remove_tag_user( $tag, get_current_user_id() );
    }

    return '';
  }

}