<?php

namespace WPEP\Coordinator\Content\Library;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Container\FilterProxy as FilterProxy;

class Block {

  use FilterProxy;

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'index', [ $this, 'primary' ] );
  }

  public function primary($attributes, $content = null) {
    if( !is_user_logged_in() && in_array( 'index-page-shortcode', wpep_get_auth_status_page_restrictions() ) ) {
      return WPEP_Controller::instance()->authentication->get_form( $attributes );
    }

    $current_user_id = get_current_user_id();

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $post_id         = isset($attributes['post_id'])         ? intval( $attributes['post_id'] )        : false;
    $include_header  = isset($attributes['include_header'])  ? intval( $attributes['include_header'] ) : false;
    $include_footer  = isset($attributes['include_footer'])  ? intval( $attributes['include_footer'] ) : false;
    $user_id         = isset($attributes['user_id'])         ? intval( $attributes['user_id'] ) : '';

    if( $user_id !== '' && $user_id != $current_user_id )
      wp_set_current_user( $user_id );
    else
      $user_id = '';

    $this->_filter_proxy_setup( $this->_primary_filters( $attributes ) );

    $response = WPEP_Controller::instance()->template->get_template(
      'primary.php', [
        'post_id'        => $post_id,
        'include_header' => $include_header,
        'include_footer' => $include_footer
      ]
    );

    $this->_filter_proxy_remove();

    if( $user_id !== '' )
      wp_set_current_user( $current_user_id );

    return $response;
  }

  private function _primary_filters( $attributes ) {
    $filter_proxy_list = [];

    $deprecated_posts_per_page  = isset($attributes['posts_per_page'])       ? $attributes['posts_per_page']   : '';
    $display_restriction  = isset($attributes['display_restriction'])  ? $attributes['display_restriction'] : 'none';
    $verify_show_flag     = isset($attributes['verify_show_flag'])     ? intval( $attributes['verify_show_flag'] ) : 1;
    $layout_category      = isset($attributes['layout-category'])      ? intval( $attributes['layout-category'] )  : 0;
    $category_ids         = isset($attributes['categories'])           ? $attributes['categories']  : '';
    $categories_per_row   = isset($attributes['categories_per_row'])
      ? intval( $attributes['categories_per_row'] )  : ( isset($attributes['category_navigation'] ) && intval( $attributes['category_navigation'] )  ? 1 : '' );
    $all_categories_filter= isset($attributes['all_categories_filter']) ? $attributes['all_categories_filter']  : '';
    $single_category_view = isset($attributes['single_category_view']) ? $attributes['single_category_view']  : '';
    $pagination_per_page  = isset($attributes['pagination_per_page'])  ? $attributes['pagination_per_page'] : '';
    $display_header       = isset($attributes['display_header'])       ? $attributes['display_header']  : null;
    $order                = isset($attributes['order'])                ? $attributes['order']   : '';
    $lock_by_prerequisites= isset($attributes['lock_by_prerequisites'])? $attributes['lock_by_prerequisites']   : '';
    $order_by             = isset($attributes['order_by'])             ? (
    in_array( $attributes['order_by'], array_keys( wpep_content_primary_order_by_options() ) )
      ? $attributes['order_by'] : ''
    ) : '';
    $layout               = isset($attributes['layout'])   ? $attributes['layout']   : '';
    $post_ids             = isset($attributes['post_ids']) ? $attributes['post_ids'] : '';
    $navigation_xhr       = isset($attributes['navigation_xhr']) ? $attributes['navigation_xhr'] : '';
    $link_mode            = isset($attributes['link_mode']) ? $attributes['link_mode'] : 'default';

    if( $display_restriction === 'access' ) {
      if( !wpep_controller()->contentLibrary->is_integrated() ) {
        $display_restriction = 'none';
      } else {
        $pagination_per_page = 0;
      }
    }

    if( $category_ids !== '' )
      $filter_proxy_list['wpep_primary_content_category_list_args'] = function( $args ) use ( $category_ids ) {
        $category_ids = is_array( $category_ids ) ? $category_ids : explode( ",", $category_ids );
        $category_ids = wpep_utility_array_trim( $category_ids );

        if( isset( $args['parent'] ) && $args['parent'] != 0 && in_array( $args['parent'], $category_ids ) )
          return $args;

        $args['include'] = $category_ids;
        $args['parent'] = '';

        return $args;
      };

    if( !$verify_show_flag )
      $filter_proxy_list['wpep_primary_content_verify_index_page_flag'] = false;

    if( $all_categories_filter !== '' )
      $filter_proxy_list['wpep_primary_content_all_categories_navigation_enabled'] = intval( $all_categories_filter );

    if( $categories_per_row !== '' )
      $filter_proxy_list['wpep_template_category_list_items_per_row'] = function ($current_per_row) use ($categories_per_row) {
        return ($categories_per_row == 0 ? $current_per_row : $categories_per_row);
      };

    if( $single_category_view !== '' )
      $filter_proxy_list['wpep_primary_content_grid_categories_secondary_as_primary_view'] = ( $single_category_view == true ? 1 : 0 );

    if( $display_header !== null )
      $filter_proxy_list['wpep_primary_content_display_header'] = $display_header;

    foreach( wpep_content_type()->libraryContentTypes as $contentType )
      $filter_proxy_list = $contentType->process_grid_content_options( $filter_proxy_list, $attributes );

    if( $pagination_per_page !== '' )
      $filter_proxy_list['wpep_primary_content_entries_per_page'] = intval( $pagination_per_page );

    if( $navigation_xhr !== '' ) {
      $navigation_xhr = is_array( $navigation_xhr ) ? $navigation_xhr : json_decode( htmlspecialchars_decode( $navigation_xhr ), true );

      if( is_array( $navigation_xhr ) )
        $filter_proxy_list['wpep_primary_content_navigation_xhr_args'] = $navigation_xhr;
    }
    if( $layout !== '' )
      $filter_proxy_list[ 'wpep_primary_content_grid_layout_name' ] = $layout;

    if( $layout_category !== 0 || $post_ids !== '' ) {

      if( $post_ids !== '' )
        $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list[ 'wpep_primary_content_list_args' ] = function( $args ) use ( $post_ids, $layout_category ) {
        if( $post_ids !== '' ) {
          $args['post__in'] = wp_parse_id_list( $post_ids );

          if( isset( $args['meta_query'] ) && isset( $args['meta_query']['wpep_show_on_index'] ) )
            unset( $args['meta_query']['wpep_show_on_index'] );
        }

        if( $layout_category !== 0 ) {
          $args['tax_query'][WPEP_TAXONOMY_CATEGORY . "_layout"] = [
            'taxonomy' => WPEP_TAXONOMY_CATEGORY,
            'field'    => 'term_id',
            'terms'    => $layout_category
          ];
        }

        return $args;
      };
    }

    if( $order !== '' || $order_by !== '' || $deprecated_posts_per_page !== '' || $display_restriction === 'access' ) {
      $wpep_ordering_enforced = wpep_get_setting_flag( 'index-page-enforce-ordering' );

      $filter_proxy_list['pre_get_posts'] = function( $wp_query ) use ( $order, $order_by, $display_restriction, $deprecated_posts_per_page ) {
        if( $order !== '' )
          $wp_query->set( 'order', ( $order === 'DESC' ? 'DESC' : 'ASC' ) );

        if( $order_by !== '' ) // Sanitized Earlier
          $wp_query->set( 'orderby', $order_by );

        if( $display_restriction === 'access' )
          $wp_query->set( 'suppress_filters', false );

        if( $deprecated_posts_per_page !== '' ) {
          $wp_query->set( 'posts_per_page', intval( $deprecated_posts_per_page ) );
          $wp_query->set( 'numberposts', intval( $deprecated_posts_per_page ) );
          $wp_query->set( 'nopaging', false );
        }

        return $wp_query;
      };

      if( $display_restriction === 'access' ) {
        $filter_proxy_list['posts_results'] = function( $found_posts ) {
          foreach( $found_posts as $post_key => $post ) {
            if( WPEP_Controller::instance()->contentLibrary->has_access_to_post( $post->ID ) )
              continue;

            unset( $found_posts[ $post_key ] );
          }

          $found_posts = array_values( $found_posts );

          return $found_posts;
        };
      }

      if( $wpep_ordering_enforced ) {
        WPEP_Settings::instance()->options[ 'index-page-enforce-ordering' ] = 0;

        // Actions are actually a sub-set of filters, smart architecture, we're going to use this to boost performance
        $filter_proxy_list['wpep_after_main_content'] = function() {
          WPEP_Settings::instance()->options[ 'index-page-enforce-ordering' ] = 1;
        };
      }
    }

    if( $lock_by_prerequisites !== '' ) {
      $locked_by_prerequisites = wpep_get_setting_flag( 'prerequisite-course-button-lock' );

      if( $locked_by_prerequisites != $lock_by_prerequisites ) {
        WPEP_Settings::instance()->options[ 'prerequisite-course-button-lock' ] = $lock_by_prerequisites;

        // Actions are actually a sub-set of filters, smart architecture, we're going to use this to boost performance
        $filter_proxy_list['wpep_after_main_content'] = function() use ( $locked_by_prerequisites ) {
          WPEP_Settings::instance()->options[ 'prerequisite-course-button-lock' ] = $locked_by_prerequisites;
        };
      }
    }

    if( $link_mode === 'new_tab' ) {
      add_filter( 'wpep_grid_item_button_attributes', function( $attributes, $item_id ) {
        if( isset( $attributes[ 'data-wpep-video-id' ] ) )
          return $attributes;

        $attributes[ 'target' ] = "_blank";

        return $attributes;
      }, 20, 2 );
    }

    return $filter_proxy_list;
  }

}