<?php

namespace WPEP\Coordinator;

use WPEP\Entity\Discussion as WPEP_Entity_Discussion;

class Discussion {

  /**
   * @var \WPEP_Discussion_Integration[]
   */
  protected $_integrations = [];

  /**
   * @var Discussion\Data
   */
  public $data;
  /**
   * @var Discussion\DirectURL
   */
  public $directURL;
  /**
   * @var Discussion\EmailNotifications
   */
  public $emailNotifications;
  /**
   * @var Discussion\Events
   */
  public $events;

  public function __construct() {
    $this->data       = new Discussion\Data();
    $this->directURL  = new Discussion\DirectURL();
    $this->emailNotifications = new Discussion\EmailNotifications();
    $this->events             = new Discussion\Events();
  }

  /**
   * @param \WPEP_Discussion_Integration $instance
   */
  public function register_integration( $instance ) {
    $this->_integrations[ $instance->get_namespace() ] = $instance;
  }

  public function all() {
    return $this->_integrations;
  }

  public function has_integrations() {
    return !empty( $this->_integrations );
  }

  /**
   * @param $namespace
   * @return \WPEP_Discussion_Integration|false
   */
  public function get( $namespace ) {
    return isset( $this->_integrations[ $namespace ] ) ? $this->_integrations[ $namespace ] : false;
  }

  public function setup() {
    $this->directURL->setup();
    $this->emailNotifications->setup();
    $this->events->setup();
  }

  public function user_get_discussions_information( $user_id, $args ) {
    $args = wp_parse_args( $args, [
      'count_total'   => true,
      'limit'         => WPEP_DISCUSSION_DIRECT_URL_MY_DISCUSSIONS_PER_PAGE,
      'offset'        => 0,
      'last_activity' => []
    ] );

    $discussion_ids = WPEP_Entity_Discussion::instance()->get_discussion_ids_by_user_id( $user_id, [
      'limit'       => false,
      'count_total' => false,
    ] );

    if( empty( $discussion_ids ) )
      return [
        'results' => [],
        'total'   => 0
      ];

    foreach( $this->all() as $discussion_integration )
      $args = $discussion_integration->user_get_discussions_args( $args );

    $where = [ 'id' => [ 'IN' => $discussion_ids ] ];

    if( !empty( $args[ 'last_activity'] ) )
      $where[ 'last_activity' ] = $args[ 'last_activity' ];

    if( isset( $args[ 'post_meta_where' ] ) && !empty( $args[ 'post_meta_where' ] ) ) {
      $where[ 'post_id' ] = [
        'post_meta' => $args[ 'post_meta_where' ]
      ];
    }

    return WPEP_Entity_Discussion::instance()->get_discussions( $where, [
      'count_total'           => $args[ 'count_total' ],
      'limit'                 => $args[ 'limit' ],
      'offset'                => $args[ 'offset' ],
      'bind_user_information' => $user_id
    ] );
  }

}