<?php

namespace WPEP\Coordinator\Discussion;

use WPEP\Entity\Discussion as Entity_Discussion;
use WPEP\Controller as WPEP_Controller;

class Data {

  public function get_comments( $discussion_id, $args ) {
    $args = wp_parse_args( $args, [
      'discussion_integration'     => false,
      'pagination'                 => 1,
      'pagination_current_page'    => 1,
      'pagination_per_page'        => wpep_get_setting( 'comments-per-page' ),
      'structure'                  => 'threaded',
      'search'                     => '',
      '_search_id_map'             => [],
      'can_reply_comment'          => null,
      'type'                       => '',
      'parent__in'                 => [],
      'date_query'                 => null,
      'filter_by_user'             => false,
      'include_comment_by_id'      => false,
      '_include_comment'           => false,
      '_include_comment_top_level' => false
    ] );

    if( $args[ 'discussion_integration' ] === false ) {
      $discussion = Entity_Discussion::instance()->get_discussion( [ 'id' => $discussion_id ] );

      if( $discussion === null )
        return null;

      $args[ 'discussion_integration' ] = WPEP_Controller::instance()->discussion->get( $discussion->namespace );
    }

    if( $args[ 'can_reply_comment' ] === null )
      $args[ 'can_reply_comment' ] = $args[ 'discussion_integration' ]->current_user_can_post( $discussion_id );

    if( $args['search'] !== '' || $args['filter_by_user'] !== false ) {
      $args[ '_search_id_map' ] = Entity_Discussion::instance()->search_for_discussion_comment_ids( $discussion_id, $args['search'], $args['filter_by_user'] );

      if( empty( $args[ '_search_id_map' ] ) )
        return [
          'args'        => $this->_sanitize_args_for_response( $args ),
          'results'     => [],
        ];
    }

    if( $args[ 'include_comment_by_id' ] !== false ) {
      if( $args[ 'structure' ] !== 'threaded' ) {
        $args[ 'include_comment_by_id' ]      = false;
        $args[ '_include_comment' ]           = false;
        $args[ '_include_comment_top_level' ] = false;
      } else {
        $args[ '_include_comment' ] = Entity_Discussion::instance()->get_discussion_comment(
          [
            'id' => $args[ 'include_comment_by_id' ]
          ],
          [
            'bind_user_reaction_data' => ( wpep_get_setting_flag( 'comments-reactions-status' ) ? get_current_user_id() : false )
          ]
        );

        if( $args[ '_include_comment' ] === null || $args[ '_include_comment' ]->discussion_id != $discussion_id ) {
          $args[ 'include_comment_by_id' ]      = false;
          $args[ '_include_comment' ]           = false;
          $args[ '_include_comment_top_level' ] = false;
        } elseif( $args[ '_include_comment' ]->parent_id == 0 ) {
          $args[ '_include_comment_top_level' ] = $args[ '_include_comment' ];
        } else {
          $args[ '_include_comment_top_level' ] = $args[ '_include_comment' ];

          while( false !== $args[ '_include_comment_top_level' ] && $args[ '_include_comment_top_level' ]->parent_id != 0 ) {
            $args[ '_include_comment_top_level' ] = Entity_Discussion::instance()->get_discussion_comment(
              [
                'id' => $args[ '_include_comment_top_level' ]->parent_id
              ],
              [
                'bind_user_reaction_data' => ( wpep_get_setting_flag( 'comments-reactions-status' ) ? get_current_user_id() : false )
              ]
            );

            if( $args[ '_include_comment_top_level' ] === null ) {
              $args[ 'include_comment_by_id' ]      = false;
              $args[ '_include_comment' ]           = false;
              $args[ '_include_comment_top_level' ] = false;
            }
          }
        }
      }
    }

    $discussion_comments = Entity_Discussion::instance()->get_discussion_comments(
      $this->_query_where_array( $discussion_id, $args ),
      [
        'order'                   => 'DESC',
        'order_by'                => 'last_activity',
        'search'                  => $args['search'],
        'count_total'             => true,
        'limit'                   => $args[ 'pagination_per_page' ],
        'offset'                  => $args[ 'pagination_per_page' ] * ( $args[ 'pagination_current_page' ] - 1  ),
        'bind_user_reaction_data' => ( wpep_get_setting_flag( 'comments-reactions-status' ) ? get_current_user_id() : false )
      ]
    );

    $results = [];

    foreach( $discussion_comments[ 'results' ] as $discussion_comment_key => $discussion_comment ) {
      $discussion_comment = $this->format_comment_data( $discussion_comment, $args[ 'discussion_integration' ], $args[ 'can_reply_comment' ] );

      if( $args['structure'] == 'threaded' ) {
        $replies_data = $this->get_comment_replies( $discussion_id, [ 'comment_parent_id' => $discussion_comment->id, 'pagination_per_page' => wpep_get_setting_translated( 'comment-replies-per-page' ) ] + $args );

        if( isset( $replies_data[ 'results' ] ) && !empty( $replies_data[ 'results' ] ) )
          $discussion_comment->replies = $replies_data;
      }

      $results[] = $discussion_comment;

      if( $args[ '_include_comment_top_level' ] !== false && $discussion_comment->id == $args[ '_include_comment_top_level' ]->id ) {
        $args[ '_include_comment' ] = false;
        $args[ '_include_comment_top_level' ] = false;
      }
    }

    if( $args[ '_include_comment_top_level' ] !== false ) {
      $discussion_comment = $this->format_comment_data( $args[ '_include_comment_top_level' ], $args[ 'discussion_integration' ], $args[ 'can_reply_comment' ] );

      if( $args['structure'] == 'threaded' ) {
        $replies_data = $this->get_comment_replies( $discussion_id, [ 'comment_parent_id' => $discussion_comment->id, 'pagination' => 0, 'pagination_per_page' => wpep_get_setting_translated( 'comment-replies-per-page' ) ] + $args );

        if( isset( $replies_data[ 'results' ] ) && !empty( $replies_data[ 'results' ] ) )
          $discussion_comment->replies = $replies_data;
      }

      $results = array_merge( [ $discussion_comment ], $results );
    }

    if( !$args[ 'pagination' ] )
      return [
        'args'       => $this->_sanitize_args_for_response( $args ),
        'results'    => $results
      ];

    return [
      'args'       => $this->_sanitize_args_for_response( $args ),
      'results'    => $results,
      'pagination' => [
        'current' => $args[ 'pagination_current_page' ],
        'total'   => ceil( $discussion_comments['total'] / $args[ 'pagination_per_page' ] ),
        'lang'    => [
          'previous' => __( 'Previous' ),
          'next'     => __( 'Next' )
        ]
      ]
    ];
  }

  private function _sanitize_args_for_response( $args ) {
    unset( $args[ 'discussion_integration' ] );
    unset( $args[ '_search_id_map' ] );
    unset( $args[ '_include_comment' ] );
    unset( $args[ '_include_comment_top_level' ] );

    return $args;
  }

  private function _query_where_array( $discussion_id, $args ) {
    $comments_where = [
      'discussion_id' => $discussion_id,
      'parent_id'     => ( !empty( $args['parent__in'] ) ? [
        'IN'  =>  $args['parent__in']
      ] : ( $args['structure'] == 'threaded' ? 0 : null ) )
    ];


    if( !empty( $args[ 'date_query' ] ) ) {
      $comments_where[ 'created_at' ] = $args[ 'date_query' ];
    }

    if( $args['type'] !== '' )
      $comments_where['type'] = $args['type'];

    if( isset( $args['_search_id_map'][0] ) && !empty( $args['_search_id_map'][0] ) )
      $comments_where[ 'id' ] = [ 'IN' => $args['_search_id_map'][0] ];

    return $comments_where;
  }

  public function get_comment_replies( $discussion_id, $args ) {
    $args = wp_parse_args( $args, [
      'discussion_integration'     => false,
      'pagination'                 => 1,
      'pagination_per_page'        => wpep_get_setting_translated( 'comment-replies-per-page' ),
      'comment_parent_id'          => 0,
      'structure'                  => 'threaded',
      'search'                     => '',
      '_search_id_map'             => [],
      'can_reply_comment'          => null,
      'last_activity_before'       => false,
      '_depth'                     => 2,
      '_include_comment'           => false,
      '_include_comment_top_level' => false
    ] );

    if( $args[ 'discussion_integration' ] === false ) {
      $discussion = Entity_Discussion::instance()->get_discussion( [ 'id' => $discussion_id ] );

      if( $discussion === null )
        return null;

      $args[ 'discussion_integration' ] = WPEP_Controller::instance()->discussion->get( $discussion->namespace );
    }

    if( $args[ 'can_reply_comment' ] === null )
      $args[ 'can_reply_comment' ] = $args[ 'discussion_integration' ]->current_user_can_post( $discussion_id );

    $discussion_comment_replies_where = [
      'discussion_id' => $discussion_id,
      'parent_id'     => $args[ 'comment_parent_id' ]
    ];

    if( $args['last_activity_before'] != false )
      $discussion_comment_replies_where['last_activity'] = [
        '<' => $args['last_activity_before']
      ];

    if( isset( $args['_search_id_map'][ $args[ 'comment_parent_id' ] ] ) )
      $discussion_comment_replies_where[ 'id' ] = [ 'IN' => $args['_search_id_map'][ $args[ 'comment_parent_id' ] ] ];

    $discussion_comment_replies_args = [
      'search'                  => $args['search'],
      'order'                   => 'DESC',
      'order_by'                => 'last_activity',
      'bind_user_reaction_data' => ( wpep_get_setting_flag( 'comments-reactions-status' ) ? get_current_user_id() : false ),
      'count_total'             => true
    ];

    if( $args[ 'pagination' ]
        && ( $args[ '_include_comment' ] === false
              || ( $args[ 'comment_parent_id' ] != $args[ '_include_comment' ]->id
                    && $args[ 'comment_parent_id' ] != $args[ '_include_comment' ]->parent_id
                    && $args[ 'comment_parent_id' ] != $args[ '_include_comment_top_level' ]->id
                 )
            )
      ) {
      $discussion_comment_replies_args[ 'limit' ] = $args[ 'pagination_per_page' ];
    }

    $discussion_comment_replies = Entity_Discussion::instance()->get_discussion_comments( $discussion_comment_replies_where, $discussion_comment_replies_args );

    if( empty( $discussion_comment_replies['results'] ) )
      return [
        'args' => $this->_sanitize_args_for_response( $args ),
      ];

    $results = [];

    $discussion_comment_replies['results'] = array_reverse( $discussion_comment_replies['results'] );

    foreach( $discussion_comment_replies['results'] as $discussion_comment_key => $discussion_comment ) {
      $discussion_comment = $this->format_comment_data( $discussion_comment, $args[ 'discussion_integration' ], $args[ 'can_reply_comment' ] );

      if( $args['structure'] == 'threaded'
          && $args[ 'can_reply_comment' ]
          && $args[ '_depth' ] < WPEP_DISCUSSION_COMMENTS_DEPTH_LIMIT ) {

        $replies_data = $this->get_comment_replies( $discussion_id, [
            'comment_parent_id'       => $discussion_comment->id,
            '_depth'                  => ( $args[ '_depth' ] + 1 ),
            'can_reply_comment'       => ( $args[ 'can_reply_comment' ] && $args[ '_depth' ] + 1 < WPEP_DISCUSSION_COMMENTS_DEPTH_LIMIT )
          ] + $args );

        if( isset( $replies_data[ 'results' ] ) && !empty( $replies_data[ 'results' ] ) )
          $discussion_comment->replies = $replies_data;
      }

      $results[] = $discussion_comment;
    }

    if( !$args[ 'pagination' ] )
      return [
        'args'    => $this->_sanitize_args_for_response( $args ),
        'results' => $results
      ];

    $first_displayed_comment = $discussion_comment_replies['results'][ key( $discussion_comment_replies['results'] ) ];

    if( $discussion_comment_replies['total'] > count( $discussion_comment_replies['results'] ) ) {
      return [
        'args'        => $this->_sanitize_args_for_response( $args ),
        'results'     => $results,
        'pagination'  => [
          'last_activity_before' => $first_displayed_comment->last_activity,
          'lang'                 => [
            'show_more' => sprintf(
              wpep_get_setting_translated( 'comment-replies-show-more' ),
              $discussion_comment_replies['total'] - ( count( $discussion_comment_replies['results'] ) )
            )
          ]
        ]
      ];
    } else {
      return [
        'args'        => $this->_sanitize_args_for_response( $args ),
        'results'     => $results,
        'pagination'  => [
          'last_activity_before' => null,
          'lang'                 => [
            'show_more' => sprintf(
              wpep_get_setting_translated( 'comment-replies-show-more' ),
              $discussion_comment_replies['total'] - ( count( $discussion_comment_replies['results'] ) )
            )
          ]
        ]
      ];
    }
  }

  /**
   * @param $discussion_comment
   * @param \WPEP_Discussion_Integration|false $discussionIntegration
   * @param bool $allow_reply
   * @return mixed
   */
  public function format_comment_data( $discussion_comment, $discussionIntegration, $allow_reply = true ) {
    $discussion_comment->user_profile_name       = wpep_content_user_profile_format_name( $discussion_comment->user_id );
    $discussion_comment->user_profile_picture    = wpep_user_profile_picture( $discussion_comment->user_id, 40 );

    if( wpep_public_profiles_enabled() && ( \WPEP\ACL\User::profile( $discussion_comment->user_id ) ) )
      $discussion_comment->user_profile_url = wpep_user_profile_public_link( $discussion_comment->user_id );

    $discussion_comment->user_is_manager         = (int) user_can( $discussion_comment->user_id, WPEP_CAP_MANAGE_MEMBERS );
    $discussion_comment->content                 = wpep_discussion_comment_content( $discussion_comment );
    $discussion_comment->data_reactions          = ( isset( $discussion_comment->data_reaction_list ) && $discussion_comment->data_reaction_list != '' ? explode( ',', $discussion_comment->data_reaction_list ) : [] );
    $discussion_comment->created_at_formatted    = wpep_discussion_format_date( $discussion_comment->created_at );
    $discussion_comment->last_activity_timestamp = strtotime( $discussion_comment->last_activity );

    $discussion_comment->actions_status          = [
      'reply'  => (int) $allow_reply,
      'delete' => (int) $discussionIntegration->current_user_can_delete_comment( $discussion_comment ),
      'pin'    => (int) $discussionIntegration->current_user_can_pin_comment( $discussion_comment ),
    ];

    return $discussion_comment;
  }

  public function update_comment_reaction_data( $discussion_comment_id ) {
    $query = 'UPDATE `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` discussion_comment
                 SET `data_reaction_count` = ( SELECT COUNT( dcd_count.id )
                                                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` dcd_count 
                                               WHERE dcd_count.discussion_comment_id = discussion_comment.id
                                                 AND dcd_count.key = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '" 
                                            ),
                     `data_reaction_list` = ( SELECT GROUP_CONCAT( DISTINCT dcd_list.`value` ORDER BY dcd_list.last_activity DESC SEPARATOR "," )
                                                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA . '` dcd_list 
                                               WHERE dcd_list.discussion_comment_id = discussion_comment.id
                                                 AND dcd_list.key = "' . WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION . '" 
                                            ),
                     `data_reaction_last_activity` = "' . date( "Y-m-d H:i:s", time() ) . '"
                 WHERE id = ' . intval( $discussion_comment_id );

    wpep_wpdb()->query( $query );
  }

  public function get_comment_reactions( $discussion_comment_id, $args ) {
    $args = wp_parse_args( $args, [
      'pagination'              => 1,
      'pagination_current_page' => 1,
      'pagination_per_page'     => wpep_get_setting( 'comments-reactions-popup-reactions-per-request' ),
      'reaction_code'           => '',
      'include_overview'        => 0,
    ] );

    $response = [
      'args'    => $args,
      'results' => [],
    ];

    $discussion_comment_reactions = Entity_Discussion::instance()->get_discussion_comment_reactions( $discussion_comment_id, [
      'count_total'   => 1,
      'limit'         => ( $args[ 'pagination' ] ? $args[ 'pagination_per_page' ] : 0 ),
      'offset'        => ( $args[ 'pagination' ] ? ( ( $args[ 'pagination_current_page' ] - 1 ) * $args[ 'pagination_per_page' ] ) : 0 ),
      'reaction_code' => $args[ 'reaction_code' ]
    ] );

    if( !empty( $discussion_comment_reactions[ 'results' ] ) ) {

      foreach( $discussion_comment_reactions[ 'results' ] as $result ) {
        $response[ 'results' ][] = [
          'user_id'              => $result->user_id,
          'user_profile_name'    => wpep_content_user_profile_format_name( $result->user_id ),
          'user_profile_picture' => wpep_user_profile_picture( $result->user_id, 40 ),
          'user_is_manager'      => (int) user_can( $result->user_id, WPEP_CAP_MANAGE_MEMBERS ),
          'reaction_code'        => $result->value,
          'reacted_at'           => $result->last_activity
        ];
      }

      if( wpep_public_profiles_enabled() ) {
        foreach( $response[ 'results' ] as $result_key => $result )
          if( \WPEP\ACL\User::profile( $result[ 'user_id' ] ) )
            $response[ 'results' ][ $result_key ][ 'user_profile_url' ] = wpep_user_profile_public_link( $result[ 'user_id' ] );
      }

      if( $args[ 'pagination' ] ) {
        $response[ 'pagination' ] = [
          'current' => $args[ 'pagination_current_page' ],
          'total'   => ceil( $discussion_comment_reactions['total'] / $args[ 'pagination_per_page' ] ),
          'lang'    => [
            'show_more' => wpep_get_setting_translated( 'comments-reactions-popup-show-more' )
          ]
        ];
      }
    }

    if( $args[ 'include_overview' ] ) {
      $overview_results = Entity_Discussion::instance()->get_discussion_comment_reactions_count( $discussion_comment_id );

      $response['overview_results'] = [];

      if( !empty( $overview_results ) ) {
        if( count( $overview_results ) > 1 ) {
          $response['overview_results'][] = [
            'text'  => wpep_get_setting_translated( 'comments-reactions-popup-all-reactions-text' ),
            'count' => array_sum( $overview_results )
          ];
        }

        foreach( $overview_results as $reaction_code => $reaction_count )
          $response['overview_results'][] = [
            'reaction_code' => $reaction_code,
            'count'         => $reaction_count
          ];
      }
    }

    return $response;
  }

  public function get_user_notifications( $user_id ) {
    $notifications_data = Entity_Discussion::instance()->get_notifications_data( $user_id );

    if( empty( $notifications_data ) )
      return [];

    $discussion_map = Entity_Discussion::instance()->get_discussions( [ 'id' => [ 'IN' => wpep_utility_map_to_array_assoc( $notifications_data, 'notification_discussion_id', 'notification_discussion_id' ) ] ] );
    $user_map       = get_users( [ 'paged' => -1, 'include' => wpep_utility_map_to_array_assoc( $notifications_data, 'notification_user_id', 'notification_user_id' ) ] );

    if( empty( $user_map ) || empty( $discussion_map ) )
      return [];

    $user_map = wpep_utilities_map_object( $user_map, 'ID' );
    $discussion_map = wpep_utilities_map_object( $discussion_map, 'id' );

    $response = [];

    foreach( $notifications_data as $notification_data ) {
      if( !isset( $user_map[ $notification_data->notification_user_id ] )
          || !isset( $discussion_map[ $notification_data->notification_discussion_id ] ) )
        continue;

      $current_notification = $notification_data;
      $notification_created_at = strtotime( $notification_data->notification_created_at );

      $current_notification->notification_discussion_name = wpep_controller()->discussion->get( $discussion_map[ $notification_data->notification_discussion_id ]->namespace )->get_discussion_name( $discussion_map[ $notification_data->notification_discussion_id ] );
      $current_notification->notification_user_profile_name = wpep_content_user_profile_format_name( $user_map[ $notification_data->notification_user_id ] );
      $current_notification->notification_user_profile_picture = wpep_user_profile_picture( $notification_data->notification_user_id, 40 );
      $current_notification->notification_url = wpep_discussion_direct_url_link( $discussion_map[ $notification_data->notification_discussion_id ]->identifier, $notification_data->notification_comment_id );
      $current_notification->notification_is_seen = ( $notification_created_at < $notification_data->notification_last_user_view_timestamp ? 1 : 0 );
      $current_notification->notification_icon_emoji = $current_notification->notification_icon_emoji_code;
      $current_notification->notification_created_at_formatted = wpep_format_date_with_time( $notification_created_at );

      if( $notification_data->notification_type === WPEP_DISCUSSION_NOTIFICATION_TYPE_COMMENT )
        $current_notification->notification_content = sprintf( wpep_get_setting_translated( 'comments-direct-url-menu-item-notification-comment' ), '<strong>' . $current_notification->notification_user_profile_name . '</strong>', '<strong>' . $current_notification->notification_discussion_name . '</strong>' );
      elseif( $notification_data->notification_type === WPEP_DISCUSSION_NOTIFICATION_TYPE_COMMENT_REPLY )
        $current_notification->notification_content = sprintf( wpep_get_setting_translated( 'comments-direct-url-menu-item-notification-comment-reply' ), '<strong>' . $current_notification->notification_user_profile_name . '</strong>', '<strong>' . $current_notification->notification_discussion_name . '</strong>' );
      elseif( $notification_data->notification_type === WPEP_DISCUSSION_NOTIFICATION_TYPE_COMMENT_REPLY_LEVEL )
        $current_notification->notification_content = sprintf( wpep_get_setting_translated( 'comments-direct-url-menu-item-notification-comment-reply-on-reply' ), '<strong>' . $current_notification->notification_user_profile_name . '</strong>', '<strong>' . $current_notification->notification_discussion_name . '</strong>' );
      elseif( $notification_data->notification_type === WPEP_DISCUSSION_NOTIFICATION_TYPE_REACTION )
        $current_notification->notification_content = sprintf( wpep_get_setting_translated( 'comments-direct-url-menu-item-notification-comment-reaction' ), '<strong>' . $current_notification->notification_user_profile_name . '</strong>', '<strong>' . $current_notification->notification_discussion_name . '</strong>' );
      elseif( $notification_data->notification_type === WPEP_DISCUSSION_NOTIFICATION_TYPE_MENTION )
        $current_notification->notification_content = sprintf( wpep_get_setting_translated( 'comments-direct-url-menu-item-notification-comment-mention' ), '<strong>' . $current_notification->notification_user_profile_name . '</strong>', '<strong>' . $current_notification->notification_discussion_name . '</strong>' );

      $response[] = $current_notification;
    }

    return $response;
  }

}