<?php

namespace WPEP\Coordinator;

class EmailMarketing {

  /**
   * @var \WPEP_Integration_Email_Marketing[]
   */
  protected $_integrations = [];

  /**
   * @param \WPEP_Integration_Email_Marketing $email_marketing_integration
   */
  public function register( $email_marketing_integration ) {
    $this->_integrations[ $email_marketing_integration->get_alias() ] = $email_marketing_integration;
  }

  /**
   * @param string $integration_alias
   * @return \WPEP_Integration_Email_Marketing|false
   */
  public function get_integration( $integration_alias ){
    return isset( $this->_integrations[ $integration_alias ] ) ? $this->_integrations[ $integration_alias ] : false ;
  }

  /**
   * @return \WPEP_Integration_Email_Marketing[]
   */
  public function get_integrations(){
    return $this->_integrations;
  }

  /**
   * @return bool
   */
  public function has_integrations(){
    return !empty( $this->_integrations );
  }

  public function has_text_based_tag_integrations() {
    return !empty( $this->get_text_based_tag_integrations() );
  }

  /**
   * @return \WPEP_Integration_Email_Marketing[]
   */
  public function get_text_based_tag_integrations() {
    if( empty( $this->_integrations ) )
      return [];

    $response = [];

    foreach( $this->_integrations as $email_marketing_integration ) {
      if( !$email_marketing_integration->has_text_based_tags )
        continue;

      $response[ $email_marketing_integration->get_alias() ] = $email_marketing_integration;
    }

    return $response;
  }

  /**
   * @param $tag
   * @param $user_id
   */
  public function tag_user( $tag, $user_id ) {
    if( empty( $tag ) )
      return;

    foreach( $this->get_text_based_tag_integrations() as $integration )
      $integration->add_tag( $tag, $user_id );
  }

  /**
   * @param $tag
   * @param $user_id
   */
  public function remove_tag_user( $tag, $user_id ) {
    if( empty( $tag ) )
      return;

    foreach( $this->get_text_based_tag_integrations() as $integration )
      $integration->remove_tag( $tag, $user_id );
  }

}