<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Container\Migration as WPEP_Migration_Container;

class Migration {

  use WPEP_Migration_Container;

  public function setup() {
    $current_version_available = '2.3.1';
    $migration_list = [
      '1.3.3'  => 'version-1.3.3.php',
      '1.4.0'  => 'version-1.4.0.php',
      '1.7.1'  => 'version-1.7.1.php',
      '1.8.0'  => 'version-1.8.0.php',
      '1.8.1'  => 'version-1.8.1.php',
      '1.8.2'  => 'version-1.8.2.php',
      '1.8.4'  => 'version-1.8.4.php',
      '1.8.5'  => 'version-1.8.5.php',
      '1.8.6'  => 'version-1.8.6.php',
      '1.8.7'  => 'version-1.8.7.php',
      '1.8.8'  => 'version-1.8.8.php',
      '1.9.0'  => 'version-1.9.0.php',
      '1.9.1'  => 'version-1.9.1.php',
      '1.9.2'  => 'version-1.9.2.php',
      '1.9.3'  => 'version-1.9.3.php',
      '1.9.4'  => 'version-1.9.4.php',
      '1.9.5'  => 'version-1.9.5.php',
      '2.0.0'  => 'version-2.0.0.php',
      '2.0.1'  => 'version-2.0.1.php',
      '2.0.2'  => 'version-2.0.2.php',
      '2.0.3'  => 'version-2.0.3.php',
      '2.0.4'  => 'version-2.0.4.php',
      '2.0.5'  => 'version-2.0.5.php',
      '2.1.0'  => 'version-2.1.0.php',
      '2.1.1'  => 'version-2.1.1.php',
      '2.1.2'  => 'version-2.1.2.php',
      '2.1.3'  => 'version-2.1.3.php',
      '2.1.4'  => 'version-2.1.4.php',
      '2.1.5'  => 'version-2.1.5.php',
      '2.1.6'  => 'version-2.1.6.php',
      '2.1.7'  => 'version-2.1.7.php',
      '2.2.0'  => 'version-2.2.0.php',
      '2.3.0'  => 'version-2.3.0.php',
      '2.3.1'  => 'version-2.3.1.php'
    ];

    $this->trait_setup( [
      'has_admin'                    => true,
      'namespace'                    => 'wpep',
      'plugin_file_path'             => WPEP_BASE_FILE_PATH,
      'option_name'                  => 'wpep_migration_version',
      'option_progress_name'         => 'wpep_migration_progress',
      'current_version_available'    => $current_version_available,
      'plugin_migration_folder_path' => WPEP_BASE_PATH . '/lib/migrations/',
      'migration_list'               => $migration_list,
    ] );
  }

  public function get_table_structure() {
    $charset_collate = wpep_wpdb()->get_charset_collate();

    $action_tracking_table_name     = wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING;
    $section_table_name             = wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION;
    $lesson_table_name              = wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON;
    $lesson_completed_table_name    = wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED;
    $course_user_data_table_name    = wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA;
    $assessment_question_table_name = wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION;
    $assessment_question_answer_table_name = wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER;
    $assessment_user_data     = wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA;
    $email_table              = wpep_wpdb()->prefix . WPEP_DB_TABLE_EMAIL_NOTIFICATION;
    $email_tracking_table     = wpep_wpdb()->prefix . WPEP_DB_TABLE_EMAIL_TRACKING;
    $discussion_table         = wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION;
    $discussion_data_table    = wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA;
    $discussion_comment_table = wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT;
    $discussion_comment_data_table = wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA;
    $system_cache_table_name  = wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_CACHE;
    $system_log_table_name    = wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG;

    $response = [];

    $response[ $action_tracking_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $action_tracking_table_name . '` (
                                              `id`       INT NOT NULL AUTO_INCREMENT,
                                              `post_id`  INT NOT NULL DEFAULT 0,
                                              `user_id`  INT NOT NULL DEFAULT 0,
                                              `type`     VARCHAR(500) NOT NULL DEFAULT "",
                                              `segment`  VARCHAR(500) NOT NULL DEFAULT "",
                                              `created_at` DATETIME DEFAULT NULL,
                                              PRIMARY KEY (id),
                                              INDEX `' . $action_tracking_table_name . '_post_id` (`post_id`),
                                              INDEX `' . $action_tracking_table_name . '_user_id` (`user_id`),
                                              INDEX `' . $action_tracking_table_name . '_type` (`type`)
                                              ) ' . $charset_collate;

    $response[ $section_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $section_table_name . '` (
                                              `id`       INT NOT NULL AUTO_INCREMENT,
                                              `post_id`  INT NOT NULL DEFAULT 0,
                                              `title`    VARCHAR(500) NOT NULL,
                                              `content`  LONGTEXT NOT NULL,
                                              `journey_access_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_access_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `order`  INT NOT NULL DEFAULT 0,
                                              `options` TEXT NOT NULL,
                                              PRIMARY KEY (id),
                                              INDEX `' . $section_table_name . '_post_id` (`post_id`)
                                              ) ' . $charset_collate;

    $response[ $lesson_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $lesson_table_name . '` (
                                              `id`         INT NOT NULL AUTO_INCREMENT,
                                              `post_id`    INT NOT NULL DEFAULT 0,
                                              `section_id` INT NOT NULL DEFAULT 0,
                                              `title`      VARCHAR(500) NOT NULL,
                                              `content`    LONGTEXT NOT NULL,
                                              `journey_access_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_access_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `order`    INT NOT NULL DEFAULT 0,
                                              `options`  TEXT NOT NULL,
                                              PRIMARY KEY (id),
                                              INDEX `' . $lesson_table_name . '_post_id` (`post_id`),
                                              INDEX `' . $lesson_table_name . '_section_id` (`section_id`)
                                              ) ' . $charset_collate;

    $response[ $lesson_completed_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $lesson_completed_table_name . '` (
                                                        `id`         INT NOT NULL AUTO_INCREMENT,
                                                        `lesson_id`  INT NOT NULL DEFAULT 0,
                                                        `user_id`    INT NOT NULL DEFAULT 0,
                                                        `created_at` DATETIME DEFAULT NULL,
                                                        PRIMARY KEY (id),
                                                        INDEX `' . $lesson_completed_table_name . '_lesson_id` (`lesson_id`),
                                                        INDEX `' . $lesson_completed_table_name . '_user_id` (`user_id`)
                                              ) ' . $charset_collate;

    $response[ $course_user_data_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $course_user_data_table_name . '` (
                                                        `id`         INT NOT NULL AUTO_INCREMENT,
                                                        `user_id`    INT NOT NULL DEFAULT 0,
                                                        `course_id`  INT NOT NULL DEFAULT 0,
                                                        `section_id` INT NOT NULL DEFAULT 0,
                                                        `lesson_id`  INT NOT NULL DEFAULT 0,
                                                        `key`       VARCHAR(100) NOT NULL DEFAULT "",
                                                        `value`     VARCHAR(500) NOT NULL,
                                                        PRIMARY KEY ( `id` ),
                                                        INDEX `' . $course_user_data_table_name . '_user_id` (`user_id`),
                                                        INDEX `' . $course_user_data_table_name . '_course_id` (`course_id`),
                                                        INDEX `' . $course_user_data_table_name . '_section_id` (`section_id`),
                                                        INDEX `' . $course_user_data_table_name . '_lesson_id` (`lesson_id`)
                                              ) ' . $charset_collate;

    $response[ $assessment_question_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $assessment_question_table_name . '` (
                                                           `id`      INT NOT NULL AUTO_INCREMENT,
                                                           `post_id` INT NOT NULL DEFAULT 0,
                                                           `title`   VARCHAR(500) NOT NULL DEFAULT "",
                                                           `content` LONGTEXT NOT NULL DEFAULT "",
                                                           `type`    VARCHAR(500) NOT NULL DEFAULT "",
                                                           `weight`  INT NOT NULL DEFAULT 0,
                                                           `order`   INT NOT NULL DEFAULT 0,
                                                           PRIMARY KEY (id),
                                                           INDEX `' . $assessment_question_table_name . '_post_id` (`post_id`)
                                              ) ' . $charset_collate;

    $response[ $assessment_question_answer_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $assessment_question_answer_table_name . '` (
                                                                  `id`          INT NOT NULL AUTO_INCREMENT,
                                                                  `post_id`     INT NOT NULL DEFAULT 0,
                                                                  `question_id` INT NOT NULL DEFAULT 0,
                                                                  `title`       VARCHAR(500) NOT NULL DEFAULT "",
                                                                  `content`     LONGTEXT NOT NULL DEFAULT "",
                                                                  `is_correct`  TINYINT NOT NULL DEFAULT 0,
                                                                  `order`       INT NOT NULL DEFAULT 0,
                                                                  PRIMARY KEY (id),
                                                                  INDEX `' . $assessment_question_answer_table_name . '_post_id` (`post_id`),
                                                                  INDEX `' . $assessment_question_answer_table_name . '_question_id` (`question_id`)
                                              ) ' . $charset_collate;

    $response[ $assessment_user_data ] = 'CREATE TABLE IF NOT EXISTS `' . $assessment_user_data . '` (
                                                 `id`          INT NOT NULL AUTO_INCREMENT,
                                                 `user_id`     INT NOT NULL DEFAULT 0,
                                                 `post_id`     INT NOT NULL DEFAULT 0,
                                                 `question_id` INT NOT NULL DEFAULT 0,
                                                 `question_answer_id` INT NOT NULL DEFAULT 0,
                                                 `key`         VARCHAR(100) NOT NULL DEFAULT "",
                                                 `value`       LONGTEXT NOT NULL DEFAULT "",
                                                 `created_at`  DATETIME DEFAULT NULL,
                                                 PRIMARY KEY (id),
                                                 INDEX `' . $assessment_user_data . '_user_id` (`user_id`),
                                                 INDEX `' . $assessment_user_data . '_post_id` (`post_id`),
                                                 INDEX `' . $assessment_user_data . '_question_id` (`question_id`),
                                                 INDEX `' . $assessment_user_data . '_question_answer_id` (`question_answer_id`)
                                              ) ' . $charset_collate;

    $response[ $email_table ] = 'CREATE TABLE IF NOT EXISTS `' . $email_table . '` (
                                        `id`      INT NOT NULL AUTO_INCREMENT,
                                        `post_id` INT NOT NULL DEFAULT 0,
                                        `user_id` INT NOT NULL DEFAULT 0,
                                        `title`   VARCHAR(500) NOT NULL DEFAULT "",
                                        `token`   VARCHAR(500) NOT NULL DEFAULT "",
                                        `is_opened`  INT NOT NULL DEFAULT 0,
                                        `created_at`  DATETIME DEFAULT NULL,
                                        `opened_at`   DATETIME DEFAULT NULL,
                                        PRIMARY KEY (id),
                                        INDEX `' . $email_table . '_post_id` ( `post_id` ),
                                        INDEX `' . $email_table . '_user_id` ( `user_id` ),
                                        INDEX `' . $email_table . '_is_opened` ( `is_opened` )
                                              ) ' . $charset_collate;

    $response[ $email_tracking_table ] = 'CREATE TABLE IF NOT EXISTS `' . $email_tracking_table . '` (
                                                 `id`          INT NOT NULL AUTO_INCREMENT,
                                                 `user_id`     INT NOT NULL DEFAULT 0,
                                                 `email_notification_id` INT NOT NULL DEFAULT 0,
                                                 `action`      VARCHAR(100) NOT NULL DEFAULT "",
                                                 `target`      VARCHAR(500) NOT NULL DEFAULT "",
                                                 `created_at`  DATETIME DEFAULT NULL,
                                                 PRIMARY KEY (id),
                                                 INDEX `' . $email_tracking_table . '_user_id` (`user_id`),
                                                 INDEX `' . $email_tracking_table . '_email_notification_id` (`email_notification_id`)
                                              ) ' . $charset_collate;

    $response[ $discussion_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_table . '` (
                                             `id`            INT NOT NULL AUTO_INCREMENT,
                                             `post_id`       INT NOT NULL DEFAULT 0,
                                             `identifier`    VARCHAR(500) NOT NULL DEFAULT "",
                                             `namespace`     VARCHAR(500) NOT NULL DEFAULT "",
                                             `created_at`    DATETIME DEFAULT NULL,
                                             `last_activity` DATETIME DEFAULT NULL,
                                             PRIMARY KEY (id),
                                             INDEX `' . $discussion_table . '_post_id` ( `post_id` ),
                                             INDEX `' . $discussion_table . '_identifier` ( `identifier` )
                                              ) ' . $charset_collate;

    $response[ $discussion_data_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_data_table . '` (
                                                  `id`            INT NOT NULL AUTO_INCREMENT,
                                                  `discussion_id` INT NOT NULL DEFAULT 0,
                                                  `user_id`       INT NOT NULL DEFAULT 0,
                                                  `key`           VARCHAR(100) NOT NULL DEFAULT "",
                                                  `value`         VARCHAR(500) NOT NULL,
                                                  PRIMARY KEY (id),
                                                  INDEX `' . $discussion_data_table . '_discussion_id` ( `discussion_id` ),
                                                  INDEX `' . $discussion_data_table . '_user_id` ( `user_id` )
                                              ) ' . $charset_collate;

    $response[ $discussion_comment_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_comment_table . '` (
                                                     `id`             INT NOT NULL AUTO_INCREMENT,
                                                     `parent_id`      INT NOT NULL DEFAULT 0,
                                                     `discussion_id`  INT NOT NULL DEFAULT 0,
                                                     `user_id`        INT NOT NULL DEFAULT 0,
                                                     `content`        LONGTEXT NOT NULL,
                                                     `created_at`     DATETIME DEFAULT NULL,
                                                     `last_activity`  DATETIME DEFAULT NULL,
                                                     `is_pinned`      TINYINT NOT NULL DEFAULT 0,
                                                     `type`           VARCHAR(500) NOT NULL,
                                                     `data_reaction_count` INT NOT NULL DEFAULT 0,
                                                     `data_reaction_list`  VARCHAR(1000) NOT NULL DEFAULT "",
                                                     `data_reaction_last_activity` DATETIME DEFAULT NULL,
                                                     PRIMARY KEY (id),
                                                     INDEX `' . $discussion_comment_table . '_parent_id` (`parent_id`),
                                                     INDEX `' . $discussion_comment_table . '_discussion_id` (`discussion_id`),
                                                     INDEX `' . $discussion_comment_table . '_user_id` (`user_id`)
                                              ) ' . $charset_collate;

    $response[ $discussion_comment_data_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_comment_data_table . '` (
                                            `id`            INT NOT NULL AUTO_INCREMENT,
                                            `discussion_comment_id` INT NOT NULL DEFAULT 0,
                                            `user_id`       INT NOT NULL DEFAULT 0,
                                            `key`           VARCHAR(100) NOT NULL DEFAULT "",
                                            `value`         VARCHAR(500) NOT NULL,
                                            `last_activity` DATETIME DEFAULT NULL,
                                            PRIMARY KEY (id),
                                            INDEX `' . $discussion_comment_data_table . '_discussion_comment_id` ( `discussion_comment_id` ),
                                            INDEX `' . $discussion_comment_data_table . '_user_id` ( `user_id` )
                                        ) ' . $charset_collate;

    $response[ $system_cache_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $system_cache_table_name . '` (
                                                  `id`         INT NOT NULL AUTO_INCREMENT,
                                                  `user_id`    INT NOT NULL DEFAULT 0,
                                                  `post_id`    INT NOT NULL DEFAULT 0,
                                                  `namespace`  VARCHAR(500) NOT NULL DEFAULT "",
                                                  `segment`    VARCHAR(1000) NOT NULL DEFAULT "",
                                                  `content`    LONGTEXT NOT NULL DEFAULT "",
                                                  `updated_at` DATETIME DEFAULT NULL,
                                                  `expires_at` DATETIME DEFAULT NULL,
                                                  PRIMARY KEY (id),
                                                  INDEX `' . $system_cache_table_name . '_user_id` (`user_id`),
                                                  INDEX `' . $system_cache_table_name . '_post_id` (`post_id`),
                                                  INDEX `' . $system_cache_table_name . '_namespace` (`namespace`),
                                                  INDEX `' . $system_cache_table_name . '_segment` (`segment`)
                                                  ) ' . $charset_collate;

    $response[ $system_log_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $system_log_table_name . '` (
                                                  `id`            INT NOT NULL AUTO_INCREMENT,
                                                  `namespace`     VARCHAR(500) NOT NULL DEFAULT "",
                                                  `event`         VARCHAR(500) NOT NULL DEFAULT "",
                                                  `started_at`    INT NOT NULL DEFAULT 0,
                                                  `completed_at`  INT NOT NULL DEFAULT 0,
                                                  `started_arguments` LONGTEXT NOT NULL DEFAULT "",
                                                  `completed_arguments` LONGTEXT NOT NULL DEFAULT "",
                                                  PRIMARY KEY (id),
                                                  INDEX `' . $system_log_table_name . '_namespace` (`namespace`),
                                                  INDEX `' . $system_log_table_name . '_event` (`event`),
                                                  INDEX `' . $system_log_table_name . '_started_at` (`started_at`)
                                                  ) ' . $charset_collate;

    return $response;
  }

  public function get_table_list() {
    $response = [
      wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_EMAIL_NOTIFICATION,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_EMAIL_TRACKING,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT_DATA,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_CACHE,
      wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG
    ];

    return $response;
  }

  public function get_table_data_verification_map() {
    $response = [];

    $response[ wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED ] = [
      'type'               => 'unique_data_column',
      'primary_column'     => 'id',
      'column_filter_list' => [
        'user_id'   => [],
        'lesson_id' => [],
      ]
    ];

    $response[ wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA ] = [
      'type'               => 'unique_data_column',
      'primary_column'     => 'id',
      'column_filter_list' => [
        'user_id'   => [],
        'course_id' => [],
        'key'       => [
          WPEP_USER_COURSE_STARTED,
          WPEP_USER_COURSE_COMPLETED,
          WPEP_USER_COURSE_COMPLETED_AT,
          WPEP_USER_COURSE_IS_BOOKMARKED,
          WPEP_USER_COURSE_LATEST_INTERACTION,
          WPEP_USER_COURSE_PROGRESS,
          WPEP_USER_COURSE_TIME_SPENT,
          WPEP_USER_COURSE_TIME_SPENT_FOCUSED,
          WPEP_USER_COURSE_TIME_SPENT_LAST_SYNC
        ]
      ]
    ];

    $response[ wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_DATA ] = [
      'type'               => 'unique_data_column',
      'primary_column'     => 'id',
      'column_filter_list' => [
        'user_id'       => [],
        'discussion_id' => [],
        'key'           => [
          WPEP_DISCUSSION_USER_DATA_LAST_VIEW,
          WPEP_DISCUSSION_USER_DATA_NOTIFICATION_STATUS,
          WPEP_DISCUSSION_USER_DATA_EMAIL_NOTIFICATION_STATUS,
          WPEP_DISCUSSION_USER_DATA_LAST_EMAIL_NOTIFICATION
        ]
      ]
    ];

    if( wpep_assessments_enabled() ) {
      $response[ wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_USER_DATA ] = [
        'type'               => 'unique_data_column',
        'primary_column'     => 'id',
        'column_filter_list' => [
          'user_id'       => [],
          'post_id'       => [],
          'key'           => [
            WPEP_USER_ASSESSMENT_SCORE,
            WPEP_USER_ASSESSMENT_START_TIME,
            WPEP_USER_ASSESSMENT_STATUS,
            WPEP_USER_ASSESSMENT_SUBMISSION_TIME
          ]
        ]
      ];
    }

    return $response;
  }

  public function _after_background_migration() {
    WPEP_Settings::instance()->update_settings( [
      'enable-backward-compatibility-short-codes' => '0',
      'enable-course-discussions-option'          => '0',
      'show-lesson-hover-color'                   => '1',
      'wpep-theme'                                => WPEP_Controller::instance()->templateStyle->default_theme
    ], true );
  }

  public function handle_migration_in_background() {
    if( $this->current_version != '0.0.0' )
      return false;

    return true;
  }

}