<?php

namespace WPEP\Coordinator;

class Notifications {

  private $notifications = [];
  private $author_notifications = [];

  public function setup() {
    add_action( "wpep_before_main_content", [ $this, '_wpep_before_main_content' ], 50 );
  }

  public function has_author_notifications() {
    return !empty( $this->author_notifications );
  }

  public function display_author_notifications() {
    foreach( $this->author_notifications as $author_notification_key => $author_notification ) {
      $this->display_notification( $author_notification['content'], $author_notification['type'] );

      unset( $this->author_notifications[ $author_notification_key ] );
    }
  }

  public function display_notification( $content, $type = 'default' ) {
    echo '<p class="wpep-notification wpep-notification-' . $type . '">' . $content . '</p>';
  }

  public function add_notification( $content, $type = 'success' ) {
    $this->notifications[] = [
      'content' => $content,
      'type'    => $type
    ];
  }

  public function add_author_notification( $content, $type = 'success' ) {
    $this->author_notifications[] = [
      'content' => $content,
      'type'    => $type
    ];
  }

  public function _wpep_before_main_content() {
    foreach( $this->notifications as $notification ) {
      if( !is_array( $notification ) )
        $notification = [ 'content' => $notification ];
      else if( !isset( $notification['content'] ) )
        continue;

      $notification['type'] = ( isset( $notification['type'] ) ? $notification['type'] : 'default' );

      $this->display_notification( $notification['content'], $notification['type'] );
    }

    remove_action( "wpep_before_main_content", [ $this, '_wpep_before_main_content' ], 50 );
  }


}

