<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class Template {

  public function setup() {
    add_action( 'init', [ $this, 'init' ], 50 );
  }

  public function init() {
    if( wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) && wpep_request()->is_request_type( 'frontend' ) )
      header_remove("X-Frame-Options");

    if( ( isset( $_GET[ 'wpep-preview-request-token' ] ) || isset( $_POST[ 'wpep-preview-content' ] ) )
          || (  wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) ) ) {
      remove_all_filters( 'template_redirect' );
      remove_all_filters('template_include');
      remove_all_filters('page_template');
      remove_all_filters('single_template');
      add_filter( 'status_header', [ $this, '_status_header_success' ], 100 );
    }

    add_filter( 'template_include', [ $this, 'load' ], 50 );
  }

  public function load($template_path) {
    if( wpep_has_embed_support()
        && isset( $_GET['wpep-embed'] )
        && wpep_get_setting_flag( 'user-content-embed-domain-verify' )
        && wpep_get_setting_flag( 'user-content-embed-box-enabled' ) == false ) {
      if( !isset( $_SERVER[ 'HTTP_REFERER' ] ) )
        return $this->locate_template( '_embed/error-message.php' );

      $destination_parsed_url = parse_url( $_SERVER[ 'HTTP_REFERER' ] );
      $destination_url = $destination_parsed_url['scheme'] . '://' . $destination_parsed_url['host'];
      $domain_list     = wpep_get_setting( 'user-content-embed-domain-list' );

      if( !in_array( $destination_url, $domain_list ) )
        return $this->locate_template( '_embed/error-message.php' );
    }

    $current_request_template = $this->_get_current_request_template();

    if( $current_request_template == false ) {
      $current_request_template = $this->_check_current_request_template_if_embed();

      if( $current_request_template != false )
        return $current_request_template;
    }

    if( $current_request_template == false )
      return $template_path;

    if( apply_filters( 'wpep_current_request_verify_auth_rules', true ) ) {
      if( !is_user_logged_in() && in_array( $current_request_template, wpep_get_auth_status_page_restrictions() ) ) {

        if( !is_single() || ( is_single() && !wpep_post_has_member_magnet_option( get_the_ID() ) ) )
          $current_request_template = 'authentication.php';
      }
    } else {
      if( !is_user_logged_in()
          && $current_request_template == 'content-assessment-single.php'
          && !wpep_post_has_member_magnet_option( get_the_ID() ) )
        $current_request_template = 'authentication.php';
    }

    if( wpep_is_user_agreement_required( get_current_user_id() )
        && !in_array( $current_request_template, [ 'authentication.php', 'user-profile.php' ] ) )
      $current_request_template = 'user-agreement.php';

    $this->_setup_request_template( $current_request_template );

    return $this->locate_template( $current_request_template );
  }

  private function _get_current_request_template() {
    $template_name = false;

    if ( wpep_get_setting( 'opt-course-index' ) !== '' && wpep_get_setting( 'opt-course-index' ) != 0 && get_the_ID() == wpep_get_setting('opt-course-index')  ) {
      $template_name = 'primary.php';
    } elseif (  is_single() && get_post_type() == WPEP_POST_TYPE_COURSE  ) {
      $template_name = 'content-course-single.php';
    } elseif ( is_single() && get_post_type() == WPEP_POST_TYPE_VIDEO ) {
      $template_name = 'content-video-single.php';
    } elseif ( is_single() && get_post_type() == WPEP_POST_TYPE_ASSESSMENT ) {
      $template_name = 'content-assessment-single.php';
    } elseif ( is_archive() && in_array( get_post_type(),  wpep_content_type()->library_post_types() ) ) {
      $template_name = 'primary-category.php';
    }

    if( isset( $_GET['wpep-login'] ) && $_GET['wpep-login'] == 1 && !is_user_logged_in() ) {
      $template_name = 'authentication.php';
    }

    return apply_filters( 'wpep_current_request_template', $template_name );
  }

  private function _setup_request_template( $template_name ) {
    if( $template_name == 'primary-category.php' ) {
      $current_category_object = get_queried_object();

      if( $current_category_object instanceof \WP_Term ) {
        add_filter( 'wpep_template_category_list_items_per_row', function( $current_categories_per_row, $base_categories_per_row = '' ) {
          return ( !empty( $base_categories_per_row ) ? $base_categories_per_row : $current_categories_per_row );
          }, 15, 2 );

        add_filter( 'wpep_primary_content_grid_categories', function() use ( $current_category_object ) { return [ $current_category_object ]; }, 100 );
      }
    }

    if( $template_name == 'content-video-single.php' ) {
      $current_categories = wp_get_object_terms( get_the_ID(), 'course_category', [ 'parent' => 0 ] );

      if( empty( $current_categories ) ) {
        $current_categories = [];

        foreach( wp_get_object_terms( get_the_ID(), 'course_category' ) as $child_taxonomy ) {
          if( !isset( $current_categories[ $child_taxonomy->parent ] ) )
            $current_categories[ $child_taxonomy->parent ] = get_term( $child_taxonomy->parent );
        }
      }

      add_filter( 'wpep_primary_content_display_header', function() { return false; } );
      add_filter( 'wpep_content_primary_has_categories', function() { return false; } );
      add_filter( 'wpep_primary_content_grid_categories', function() use ( $current_categories ) { return $current_categories; } );
    }
  }

  public function get_template( $template_name, $args = [], $template_path = '', $default_path = '' ) {
    ob_start();

    $this->load_template( $template_name, $args, $template_path, $default_path );

    $ret = ob_get_contents();

    ob_end_clean();

    return $ret;
  }

  public function load_template( $template_name, $args = [], $template_path = '', $default_path = '' ) {
    if ( $args && is_array( $args ) ) {
      extract( $args );
    }

    $located = $this->locate_template( $template_name, $template_path, $default_path );

    if ( ! file_exists( $located ) ) {
      _doing_it_wrong( __FUNCTION__, sprintf( '<code>%s</code> does not exist.', $located ), '1.0.0' );
      return;
    }

    // Allow 3rd party plugin filter template file from their plugin
    $located = apply_filters( 'wpep_get_template', $located, $template_name, $args, $template_path, $default_path );

    do_action( 'wpep_before_template_part', $template_name, $template_path, $located, $args );

    include( $located );

    do_action( 'wpep_after_template_part', $template_name, $template_path, $located, $args );
  }

  public function locate_template( $template_name, $template_path = '', $default_path = '' ) {
    if ( ! $template_path )
      $template_path = $this->template_path();

    if ( ! $default_path )
      $default_path = WPEP_BASE_PATH . '/templates/';

    // Look within passed path within the theme - this is priority
    $template = locate_template( [
        trailingslashit( $template_path ) . $template_name,
        $template_name
    ] );

    // Get default template
    if ( ! $template )
      $template = $default_path . $template_name;

    // Return what we found
    return apply_filters( 'wpep_locate_template', $template, $template_name, $template_path );
  }

  public function template_path() {
    return apply_filters( 'wpep_template_path', 'wpep/' );
  }

  public function get_header() {
    if( !$this->_is_embed_template() ) {
      get_header();
      return;
    }

    show_admin_bar(false);

    $this->load_template( '_embed/header.php' );
  }

  public function get_footer() {
    if( !$this->_is_embed_template() ) {
      get_footer();
      return;
    }

    $this->load_template( '_embed/footer.php' );
  }

  private function _check_current_request_template_if_embed() {
    if( wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) ) {
      if( get_post_type() == 'page' )
        return $this->locate_template( '_embed/page-post-types.php' );

      return $this->locate_template( '_embed/external-post-types.php' );
    }

    if( isset( $_GET[ 'wpep-preview-request-token' ] ) ) {
      if( isset( $_POST[ 'wpep-preview-content' ] ) ) {

        if( wp_verify_nonce( $_POST[ 'wpep-preview-content' ], 'wpep-preview-content' ) == false )
          return $this->locate_template( '_embed/custom-content-error.php' );

        if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) && isset( $_POST['wpep_settings'] ) ) {
          $wpep_settings = $_POST['wpep_settings'];

          wpep_settings_instance()->options[ 'wpep-custom-css-by-file' ] = false;

          if( isset( $wpep_settings['wpep-theme'] ) )
            wpep_controller()->templateStyle->theme = $wpep_settings['wpep-theme'];

          foreach( $wpep_settings as $setting_key => $setting_value )
            wpep_settings_instance()->options[ $setting_key ] = $setting_value;

          wpep_controller()->templateStyle->_load_element_map();
        }

        return $this->locate_template( '_embed/custom-content.php' );
      }

      return $this->locate_template( '_embed/custom-content-error.php' );
    }

    return false;
  }

  private function _is_embed_template() {
    if( wpep_has_embed_support() ) {
      if( isset( $_GET[ 'wpep-embed'] ) && $_GET[ 'wpep-embed'] == true )
        return true;
    }

    if( isset( $_GET[ 'wpep-preview-request-token' ] ) )
      return true;

    return false;
  }

  public function _status_header_success() {
    return 'HTTP/1.1 200 OK';
  }

}
