<?php

namespace WPEP\Coordinator\UserEngagement;

use WPEP\Controller as WPEP_Controller;

class Assessment {

  public function __construct() {
    if( wpep_get_setting( 'assessment-admin-email-notification-deliver-to' ) != 0 )
      add_action( "wpep_user_assessment_after_grading", [ $this, '_event_user_assessment_after_grading' ], 10, 2 );
  }

  public function send_student_email_notification( $assessment_id, $user_id ) {
    $email_data = [
      'title'   => wpep_get_setting( 'assessment-review-completed-email-notification-title' ),
      'content' => wpep_get_setting( 'assessment-review-completed-email-notification-content' )
    ];

    $email_data = $this->apply_available_shortcodes( $email_data, $assessment_id, $user_id );

    WPEP_Controller::instance()->userEngagement->send_email(
      $user_id,
      $email_data['title'],
      $email_data['content'],
      true
    );
  }

  public function _event_user_assessment_after_grading( $assessment_id, $user_id ) {
    $email_data = [
      'title'   => wpep_get_setting( 'assessment-admin-email-notification-title' ),
      'content' => wpep_get_setting( 'assessment-admin-email-notification-content' )
    ];

    $email_data = $this->apply_available_shortcodes( $email_data, $assessment_id, $user_id );

    WPEP_Controller::instance()->userEngagement->send_email(
      wpep_get_setting( 'assessment-admin-email-notification-deliver-to' ),
      $email_data['title'],
      $email_data['content'],
      true
    );
  }

  public function apply_available_shortcodes( $content, $assessment_id, $user_id ) {
    $is_array = is_array( $content );

    if( !$is_array )
      $content = [ $content ];

    $assessment_course_id = wpep_get_assessment_course_id( $assessment_id );

    $replace_data = [
      'assessment_name'             => get_the_title( $assessment_id ),
      'assessment_link'             => get_the_permalink( $assessment_id ),
      'assessment_student_name'     => wpep_content_user_profile_format_name( $user_id ),
      'assessment_student_status'   => wpep_get_setting_translated( 'assessment-status-label-' . wpep_user_assessment_status( $assessment_id, $user_id ) ),
      'assessment_student_score'    => intval( wpep_content_type_assessment_instance()->user->data->get_value( WPEP_USER_ASSESSMENT_SCORE, $assessment_id, false, false, $user_id ) ),
      'assessment_student_answers'  => wpep_template_get( 'email-template/assessment-answer-list.php', [
        'assessment_id' => $assessment_id,
        'user_id'       => $user_id
      ] ),
      'assessment_student_submission_link' => wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $assessment_id, 'user_id' => $user_id ] ),
    ];

    foreach( $content as $content_key => $content_value ) {
      if( strpos( $content[ $content_key ], '[assessment_course_link]' ) !== false )
        $content[ $content_key ] = str_replace( '[assessment_course_link]', ( $assessment_course_id === 0 ? '-' : get_the_permalink( $assessment_course_id ) ), $content[ $content_key ] );

      if( strpos( $content[ $content_key ], '[condition_assessment_has_course]' ) !== false
        && strpos( $content[ $content_key ], '[/condition_assessment_has_course]' ) !== false ) {
        if( $assessment_course_id === 0 ) {
          $position_start = strpos( $content[ $content_key ], '[condition_assessment_has_course]' );
          $position_end   = strpos( $content[ $content_key ], '[/condition_assessment_has_course]' );

          $content[ $content_key ] = substr( $content[ $content_key ], 0, $position_start ) .
            substr( $content[ $content_key ], $position_end + strlen( '[/condition_assessment_has_course]' ) );
        } else {
          $content[ $content_key ] = str_replace( [
            '[condition_assessment_has_course]',
            '[/condition_assessment_has_course]'
          ], '', $content[ $content_key ] );
        }
      }

      foreach( $replace_data as $replace_key => $replace_value )
        $content[ $content_key ] = str_replace( '[' . $replace_key . ']', $replace_value, $content[ $content_key ] );
    }

    return $is_array ? $content : array_shift( $content );
  }

  public function get_available_shortcode_list() {
    $response = '<h2>' . __( "Assessment Specific Shortcodes", "wpep" ) . '</h2>' .
                '<ul>' .
                  '<li>[assessment_name] - ' . __( "Name of the assessment that has been interacted with.", "wpep" ) . '</li>' .
                  '<li>[assessment_link] - ' . __( "Link to the assessment page.", "wpep" ) . '</li>' .
                  '<li>[assessment_course_link] - ' . __( "Link to the Course that has the assessment attached.", "wpep" ) . '</li>' .
                  '<li>[assessment_student_name] - ' . __( "Name of the Student that interacted with the assessment.", "wpep" ) . '</li>' .
                  '<li>[assessment_student_status] - ' . __( "Assessment Status of the Student that interacted with the assessment.", "wpep" ) . '</li>' .
                  '<li>[assessment_student_score] - ' . __( "Assessment Score of the Student that interacted with the assessment.", "wpep" ) . '</li>' .
                  '<li>[assessment_student_answers] - ' . __( "List of Answers the Student has given to each question.", "wpep" ) . '</li>' .
                  '<li>[assessment_student_submission_link] - ' . __( "Link to the User Assessment Submission in the Statistics in the admin.", "wpep" ) . '</li>' .
                  '<li>[condition_assessment_has_course] - ' . __( "Conditional ShortCode, display specific content only if the assessment has a course attached.", "wpep" ) . '</li>' .
                '</ul>';

    return  $response;
  }


}
