<?php

namespace WPEP\Entity;

class ActionTracking {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  private $_table_field_whitelist = [ 'id', 'post_id', 'user_id', 'type', 'segment', 'created_at' ];

  public function is_event_tracked( $where ) {
    return $this->get_first_tracked_event( $where ) !== null;
  }

  public function get_first_tracked_event( $where ) {
    $sql = 'SELECT * FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING . '` ' . wpep_sql_where( $where, $this->_table_field_whitelist ) . ' ORDER BY created_at ASC LIMIT 1';

    return wpep_wpdb()->get_row( $sql );
  }

  public function get_last_tracked_event( $where ) {
    $sql = 'SELECT * FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING . '` ' . wpep_sql_where( $where, $this->_table_field_whitelist ) . ' ORDER BY created_at DESC LIMIT 1';

    return wpep_wpdb()->get_row( $sql );
  }

  public function track_event( $post_id, $user_id, $type, $segment ) {
    $post_id = absint( $post_id );
    $user_id = absint( $user_id );

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING, [
      'post_id'    => $post_id,
      'user_id'    => $user_id,
      'type'       => $type,
      'segment'    => $segment,
      'created_at' => date( "Y-m-d H:i:s", time() )
    ] );
  }

  public function get_tracked_user_ids( $where, $args = [] ) {
    $sql = 'SELECT * FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING . '` ' . wpep_sql_where( $where, $this->_table_field_whitelist );

    $sql = ' SELECT SQL_CALC_FOUND_ROWS inner_query.user_id FROM ( ' . $sql . ' ) inner_query';

    $sql .= ' GROUP BY inner_query.user_id';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return wpep_wpdb()->get_col( $sql );

    return [
      'results' => wpep_wpdb()->get_col( $sql ),
      'total'   => absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  public function get_tracked_events( $where, $args = [] ) {
    $post_type = ( isset( $where[ 'post_type' ] ) ? $where[ 'post_type' ] : false );

    if( $post_type !== false && !wpep_content_type()->is_enabled( $post_type ) )
      return false;

    unset( $where[ 'post_type' ] );

    $sql = 'SELECT SQL_CALC_FOUND_ROWS *
              FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING . '` action_tracking';

    if( $post_type !== false )
      $sql .= ' INNER JOIN ' . wpep_wpdb()->posts . ' post_table ON post_table.ID = action_tracking.post_id';

    $sql .= ' ' . wpep_sql_where(
        $where,
        $this->_table_field_whitelist,
        false,
        'action_tracking'
      );

    if( $post_type !== false )
      $sql .= ' AND post_table.`post_type` = "' . $post_type . '"';

    $sql .= ' ORDER BY action_tracking.`created_at` DESC';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return wpep_wpdb()->get_results( $sql );

    return [
      'results' => wpep_wpdb()->get_results( $sql ),
      'total'   => absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  public function get_event_count_by_args( $args ) {
    if( isset( $args['group_by'] ) && !is_array( $args['group_by'] ) )
      $args['group_by'] = [ $args['group_by'] ];

    $has_posts_join = false;

    if( isset( $args['post_type'] ) || ( isset( $args['group_by'] ) && ( in_array( 'post_id', $args['group_by'] ) || in_array( 'post_type', $args['group_by'] ) ) ) )
      $has_posts_join = true;

    $sql = 'SELECT COUNT(action_tracking.id) as entries';

    if( isset( $args['group_by'] ) && in_array( 'type', $args['group_by'] ) )
      $sql .= ', action_tracking.`type` as type';

    if( isset( $args['group_by'] ) && in_array( 'segment', $args['group_by'] ) )
      $sql .= ', action_tracking.`segment` as segment';

    if( isset( $args['group_by'] ) && in_array( 'post_id', $args['group_by'] ) )
      $sql .= ', post_table.post_title as post_title, post_table.ID as post_id';

    if( isset( $args['group_by'] ) && in_array( 'post_type', $args['group_by'] ) )
      $sql .= ', post_table.post_type as post_type';

    $sql .= ' FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING . '` action_tracking';

    if( $has_posts_join )
        $sql .= ' INNER JOIN ' . wpep_wpdb()->posts . ' post_table ON post_table.ID = action_tracking.post_id';

    $sql .= ' WHERE 1 = 1';

    if( isset( $args['post_type'] ) )
      $sql .= ' AND post_table.`post_type` = "' . esc_sql( $args['post_type'] ) . '"';

    if( isset( $args['type' ] ) ) {
      if( is_array( $args["type"] ) && !empty( $args["type"] ) ) {
        $sql .= ' AND action_tracking.`type` IN ("' . implode( '","', array_map( 'esc_sql', $args["type"] ) ) . '")';
      } else {
        $sql .= ' AND action_tracking.`type` = "' . esc_sql( $args["type"] ) . '"';
      }
    }

    if( isset( $args[ 'segment' ] ) ) {
      if( is_array( $args["segment"] ) && !empty( $args["segment"] ) ) {
        $sql .= ' AND action_tracking.`segment` IN ("' . implode( '","', $args["segment"] ) . '")';
      } else {
        $sql .= ' AND action_tracking.`segment` = "' . esc_sql( $args["segment"] ) . '"';
      }
    } else {
      if( isset( $args['group_by'] ) && in_array( 'segment', $args['group_by'] ) )
        $sql .= ' AND action_tracking.`segment` != ""';
    }

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) )
        $sql .= ' AND action_tracking.`user_id` = ' . intval( $args[ 'user_ids' ] );
      else if( is_array( $args[ 'user_ids' ] ) )
        $sql .= ' AND action_tracking.`user_id` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
    }

    if( isset( $args[ 'post_ids' ] ) ) {
      if( is_numeric( $args[ 'post_ids' ] ) )
        $sql .= ' AND action_tracking.`post_id` = ' . intval( $args[ 'post_ids' ] );
      else if( is_array( $args[ 'post_ids' ] ) )
        $sql .= ' AND action_tracking.`post_id` IN (' . implode( ',', array_map( 'intval', $args[ 'post_ids' ] ) ) . ')';
    }

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND action_tracking.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND action_tracking.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    if( isset( $args['group_by'] ) ) {
      $sql .= ' GROUP BY';

      $i = 0; foreach( $args['group_by'] as $group_by )
        $sql .= ( $i++ === 0 ? ' ' : ',' ) . ( $group_by === 'post_type' ? 'post_table.`post_type`' : 'action_tracking.`' . $group_by . '`' );
    }

    return ( isset( $args['group_by'] ) ? wpep_wpdb()->get_results( $sql ) : absint( wpep_wpdb()->get_var( $sql ) ) );
  }

  public function delete_tracked_actions( $where ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ACTION_TRACKING, $where );
  }

}