<?php
namespace WPEP\Entity;

class Assessment {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  private $_storage = [
    'questions_by_post_id' => []
  ];

  /**
   * @param $question_id
   * @return object|null
   */
  public function get_question_by_id( $question_id ) {
    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION . ' WHERE id = ' . $question_id;

    $section = wpep_wpdb()->get_row($sql);

    return $section;
  }

  /**
   * @param bool $post_id
   * @return array
   */
  public function get_answers_by_assessment_id( $post_id = false ) {
    if( $post_id == false )
      $post_id = get_the_ID();

    $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER . ' WHERE post_id = ' . $post_id . ' ORDER BY `order` ASC';

    $questions = wpep_wpdb()->get_results($sql);

    return wpep_utilities_map_object( $questions, 'id' );
  }

  /**
   * @param bool $post_id
   * @param bool $with_answers
   * @param bool $handle_shuffle_settings
   * @return array
   */
  public function get_questions_by_assessment_id( $post_id = false, $with_answers = true, $handle_shuffle_settings = false ) {
    if( $post_id == false )
      $post_id = get_the_ID();

    if( isset( $this->_storage[ 'questions_by_post_id' ][ $post_id ] ) ) {
      $questions = $this->_storage[ 'questions_by_post_id' ][ $post_id ];
    } else {
      $sql = 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION . ' WHERE post_id = ' . intval( $post_id ) . ' ORDER BY `order` ASC';

      $questions = wpep_wpdb()->get_results($sql);

      $this->_storage[ 'questions_by_post_id' ][ $post_id ] = $questions;
    }

    if( empty( $questions ) )
      return [];

    if( $handle_shuffle_settings && get_post_meta( $post_id, 'wpep_shuffle_questions', true ) == 1 )
      shuffle( $questions );

    if( !$with_answers )
      return $questions;

    $question_answers = wpep_utilities_map_object(
      wpep_wpdb()->get_results( 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER . ' WHERE post_id = ' . $post_id . ' ORDER BY `order` ASC' ),
      'question_id',
      true
    );

    if( empty( $question_answers ) )
      return $questions;

    foreach( $questions as $k => $question ) {
      if( !isset( $question_answers[ $question->id ] ) ) {
        $questions[ $k ]->answers = [];

        continue;
      }

      $questions[$k]->answers = $question_answers[ $question->id ];

      if( $handle_shuffle_settings && get_post_meta( $post_id, 'wpep_shuffle_answers', true ) == 1 )
        shuffle( $questions[$k]->answers );
    }

    return $questions;
  }

  /**
   * @param $post_id
   * @return array|object|null
   */
  public function get_questions_with_mapped_options_by_question_id( $post_id ) {
    if( $post_id == false )
      $post_id = get_the_ID();

    $assessment_field_types = wpep_get_assessment_field_types();

    $questions = wpep_wpdb()->get_results( 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION . ' WHERE post_id = ' . $post_id . ' ORDER BY `order` ASC' );

    if( empty( $questions ) )
      return [];

    $question_answers = wpep_utilities_map_object(
      wpep_wpdb()->get_results( 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER . ' WHERE post_id = ' . $post_id . ' ORDER BY `order` ASC' ),
      'question_id',
      true
    );

    foreach( $questions as $k => $question ) {
      $questions[$k]->answers = [];

      if( !isset( $assessment_field_types[$question->type] ) )
        continue;

      if( isset( $assessment_field_types[$question->type]['predefined_answers'] ) )
        $questions[$k]->answers = $assessment_field_types[$question->type]['predefined_answers'];
      elseif( $assessment_field_types[$question->type]['has_answers'] && isset( $question_answers[ $question->id ] ) )
        $questions[$k]->answers = wpep_utility_map_to_array_assoc( $question_answers[ $question->id ], 'id', 'title' );
    }

    return $questions;
  }

  public function insert_question( $data ) {
    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION, $data );

    return wpep_wpdb()->insert_id;
  }

  public function insert_question_answer( $data ) {
    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER, $data );

    return wpep_wpdb()->insert_id;
  }

  public function update_question( $data, $where = [] ) {
    wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION, $data, $where );
  }

  public function update_question_answer( $data, $where = [] ) {
    wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER, $data, $where );
  }

  public function delete_question_by_id( $question_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION, [ 'id' => $question_id ] );
  }

  public function delete_answers_by_question_id( $question_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER, [ 'question_id' => $question_id ] );
  }

  public function delete_answer_by_id( $question_answer_id ) {
    wpep_wpdb()->delete( wpep_wpdb()->prefix . WPEP_DB_TABLE_ASSESSMENT_QUESTION_ANSWER, [ 'id' => $question_answer_id ] );
  }
  
}