<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as WPEP_Entity_Course;

class Basic extends Skeleton\Standard {

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = false;

  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );
  }

  public function init() {
    if( !is_user_logged_in() )
      return;

    parent::init();

    if( isset( $_GET['complete-course'] ) ) {
      if( !wp_verify_nonce( $_GET['complete-course'], 'complete-course' ) )
        wp_nonce_ays( 'complete-course' );

      if( get_current_user_id() != 0 )
        wpep_content_type_course_instance()->user->flag->set_status_completed( $this->course_id, get_current_user_id(), true );

    } else if( isset( $_GET['incomplete-course'] ) ) {
      if( !wp_verify_nonce( $_GET['incomplete-course'], 'incomplete-course' ) )
        wp_nonce_ays( 'incomplete-course' );

      if( get_current_user_id() != 0 )
        wpep_content_type_course_instance()->user->flag->set_status_incomplete( $this->course_id, get_current_user_id(), true );
    }

  }

  public function content() {
    $course_video             = get_post_meta( $this->course_id, 'wpep_course_video', true );
    $course_video_jump_links  = get_post_meta( $this->course_id, '_wpep_course_video_jump_links', true );
    $course_downloads_list    = get_post_meta( $this->course_id, '_wpep_course_downloads', true );
    $course_video_description = ( wpep_get_setting_flag( 'course-basic-enable-video-description' )
      ? get_post_meta( $this->course_id, 'wpep_course_video_description', true )
      : ''
    );

    if( !is_string( $course_video ) )
      $course_video = '';

    if( !is_array( $course_video_jump_links ) )
      $course_video_jump_links = [];

    if( !is_array( $course_downloads_list ) )
      $course_downloads_list = [];

    $description = get_post_meta( $this->course_id , 'wpep_course_description', true );

    $args = [
      'course_id'        => $this->course_id,
      'course_read_only' => ( is_user_logged_in() ? $this->read_only : true ),
      'course_completed' => wpep_user_course_is_completed( $this->course_id, get_current_user_id(),false ),
      'title'            => get_the_title( $this->course_id ),
      'description'      => $description,
      'course_video'     => trim( $course_video ),
      'course_video_description'=> $course_video_description,
      'course_video_jump_links' => $course_video_jump_links,
      'course_downloads_list'   => $course_downloads_list,
      'course_document_embed'   => get_post_meta( $this->course_id, 'wpep_course_document_embed', true )
    ];

    if( wpep_assessments_enabled() )
      $args['final_assessment_id'] = wpep_user_assessment_id_by_course_id( $this->course_id, 0, get_current_user_id() );

    $args = apply_filters( 'wpep_course_single_content_basic_args', $args );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      add_filter( 'wpep_single_content_basic_html', [ $this, '_content_attach_discussion_for_course' ], 1 );

    $response = apply_filters(
      'wpep_single_content_basic_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/basic-content.php', $args ),
      $args
    );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      remove_filter( 'wpep_single_content_basic_html', [ $this, '_content_attach_discussion_for_course' ], 1 );

    return $response;
  }

  public function get_alias() {
    return WPEP_COURSE_TYPE_ONE_OFF;
  }


}