<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Challenge extends Skeleton\Navigation {

  /**
   * Standard constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->section_slug = wpep_get_setting( 'course-challenge-section-slug' );

    $this->_skeleton_init( $current_course_id );

    $this->current_item = 'section';

    if( isset( $_GET[ $this->section_slug ] ) )
      $this->current_item_id = $_GET[ $this->section_slug ];

    if( !is_numeric( $this->current_item_id ) )
      $this->current_item_id = wpep_content_type_course_instance()->user->get_delivered_section_id( $current_course_id );
  }

  public function has_sidebar_content() {
    return true;
  }

  public function get_alias() {
    return "challenge";
  }

  public function init() {
    parent::init();

    add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
  }

  public function close() {
    parent::close();

    remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
  }

  public function content() {
    if( empty( $this->course_sections_map ) )
      return '';

    if( !isset( $this->course_sections_map[ $this->current_item_id ] ) )
      return '';

    $allow_social_share = get_post_meta( $this->course_id, 'wpep_disable_social_share', true );
    $allow_social_share = ( $allow_social_share === '' ? wpep_get_setting_flag( 'course-challenge-social-share-status' ) : $allow_social_share );

    $sharing_link = false;

    if( $allow_social_share ) {
      $sharing_link = get_post_meta( $this->course_id, 'wpep_social_share_link', true );

      if( $sharing_link === '' )
        $sharing_link = remove_query_arg( $this->section_slug, wpep_get_current_url() );
    }

    $args = [
      'sharing_link'          => $sharing_link,
      'course_title'          => get_the_title( $this->course_id ),
      'course_id'             => $this->course_id,
      'course_read_only'      => $this->read_only,
      'course_image'          => wpep_course_thumbnail_url( $this->course_id ),
      'section_id'            => $this->current_item_id,
      'section_title'         => $this->course_sections_map[ $this->current_item_id ]['title'],
      'section_content'       => $this->course_sections_map[ $this->current_item_id ]['content'],
      'section_lessons'       => $this->course_sections_map[ $this->current_item_id ]['lessons']
    ];

    return $this->skeleton_navigation_current_item_content_prepare( WPEP_Controller::instance()->template->get_template( 'course-single/challenge-section-content.php', $args ) );
  }

  public function display_sidebar_content( $course_id = false ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    echo '<div class="wpep-widget wpep-course-widget-primary" data-wpep-breakpoint="768">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<div class="challenge-course-thumbnail-widget">';
    echo        '<img src="' . wpep_course_thumbnail_url( $this->course_id ) . '"/>';
    echo        '<h2>' . get_the_title( $this->course_id ) . '</h2>';
    echo      '</div>';
    echo    '</div>';
    echo '</div>';

    echo '<div class="wpep-widget wpep-widget-challenge-navigation">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<ul>';

    $current_permalink = wpep_get_current_url();
    $current_permalink = remove_query_arg( $this->section_slug, $current_permalink );

    foreach( $this->course_sections_map as $course_section_id => $course_section ) {
      $is_disabled = ( isset( $course_section['options'] ) && isset( $course_section['options']['is_disabled'] ) && $course_section['options']['is_disabled'] );

      $class = 'wpep-lesson-complete-trigger-watcher';

      if( $course_section_id == $this->current_item_id )
        $class .= ' wpep-is-active';

      if( $is_disabled )
        $class .= ' wpep-section-disabled';

      echo '<li class="' . $class . '" data-wpep-is-completed="' . intval( $course_section['is_completed'] ) . '" data-wpep-section-id="' . $course_section_id . '">';
      echo    '<a href="' . ( $is_disabled ? $current_permalink : add_query_arg( $this->section_slug, $course_section_id, $current_permalink ) ) . '">' . $course_section['title'] . '</a>';
      echo    '<span class="wpep-section-status-icon"></span>';
      echo '</li>';

    }

    echo      '</ul>';
    echo    '</div>';
    echo '</div>';
  }

}