<?php

namespace WPEP\Entity\CourseType\Skeleton;

use WPEP\Controller as WPEP_Controller;

abstract class Standard {

  public $top_level_request = false;
  public $course_id;
  public $read_only = false;

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  public function init() {
    if( wpep_assessments_enabled() )
      wpep_user_assessment_preload_for_course_id( $this->course_id, get_current_user_id() );

    if( $this->top_level_request == false )
      add_filter( 'wpep_course_permalink', [ $this, 'course_permalink' ], 20, 2 );
  }

  public function _skeleton_init( $course_id ) {
    $this->course_id = $course_id;
    $this->read_only = wpep_post_read_only( $this->course_id );
  }

  public function close() {
    if( $this->top_level_request == false )
      remove_filter( 'wpep_course_permalink', [ $this, 'course_permalink' ], 20 );
  }

  abstract public function content();

  abstract public function get_alias();

  public function has_sidebar_content() {
    return false;
  }

  public function has_default_sidebar_content() {
    if( $this->top_level_request )
      return true;

    return false;
  }

  public function get_header() {
    $progress = wpep_user_course_progress_percentage( $this->course_id, get_current_user_id() );

    $args = [
      'course_id'         => $this->course_id,
      'course_thumbnail'  => get_post_meta( $this->course_id, 'wpep_icon_image', true),
      'course_title'      => get_the_title( $this->course_id ),
      'course_sub_title'  => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
      'progress_percent'  => $progress,
      'progress_label'    => number_format( $progress > 99 ? 100 : $progress, 2 ),
      'progressbar_title' => wpep_get_setting_translated( 'opt-course-progressbar' ),
      'show_progress_bar' => wpep_course_has_progress_bar( $this->course_id, $this->read_only )
    ];

    $args = apply_filters( 'wpep_course_single_content_header_args', $args );

    return apply_filters(
      'wpep_single_content_header_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/header.php', $args ),
      $args
    );
  }

  public function _content_attach_discussion_for_course( $response ) {
    if( !is_user_logged_in() )
      return $response;

    $discussion_id = $this->_content_get_discussion_id_by_args( [
      'course_id'   => $this->course_id,
      'user_id'     => get_current_user_id()
    ] );

    if( $discussion_id === 0 )
      return $response;

    if( !wpep_content_type_course_instance()->discussionIntegration->current_user_can_view( $discussion_id ) )
      return $response . '<!-- The user_id ' . get_current_user_id() . ' does not have access to view the discussion_id : ' . $discussion_id . ' for course : ' . $this->course_id . ' -->';

    $response .= $this->_content_attach_discussion_by_id( $discussion_id );

    return $response;
  }

  public function _content_get_discussion_id_by_args( $identifier_args ) {
    if( wpep_content_type_course_instance()->discussionIntegration === null )
      return 0;

    return wpep_content_type_course_instance()->discussionIntegration->discussion_id_by_identifier_args( $identifier_args, $this->course_id );
  }

  public function _content_attach_discussion_by_id( $discussion_id ) {
    return '<div class="wpep-course-discussion-container">' . wpep_content_type_course_instance()->discussionIntegration->get_comments_template( $discussion_id ) . '</div>';
  }

  public function _return_true() {
    return true;
  }

  public function _return_false() {
    return false;
  }

  public function course_permalink( $course_permalink, $course_id ) {
    if( $this->top_level_request == true )
      return $course_permalink;

    if( $this->course_id != $course_id )
      return $course_permalink;

    return wpep_get_current_url();
  }

  public function has_creator_toolbox() {
    return current_user_can( WPEP_CAP_EDIT_OTHERS_COURSE ) || ( current_user_can( WPEP_CAP_EDIT_COURSE ) && get_post( $this->course_id )->post_author == get_current_user_id() );
  }

}