<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Video extends Skeleton\Navigation {

  public $_template_args = [
    'downloads_list' => [
      'title' => ''
    ]
  ];

  /**
   * Video constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->course_has_overview_page             = wpep_get_setting_flag( 'course-video-allow-index-page' );
    $this->course_has_completion_redirect = wpep_get_setting_flag( 'course-video-redirect-on-completion' );

    $course_has_overview_page = get_post_meta( $this->course_id, 'wpep_video_course_overview_page', true );

    if( $course_has_overview_page != 'default' )
      $this->course_has_overview_page = $course_has_overview_page;
  }

  public function has_sidebar_content() {
    return $this->current_item === null;
  }

  public function has_default_sidebar_content() {
    if( $this->top_level_request )
      return false;

    return $this->current_item === null;
  }

  public function get_alias() {
    return "video";
  }

  public function init() {
    parent::init();

    if( $this->has_sidebar_content() )
      add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
  }

  public function close() {
    parent::close();

    if( $this->has_sidebar_content() )
      remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
  }

  public function content() {
    if( $this->current_item == $this->lesson_slug ) {
      $course_lesson = $this->skeleton_navigation_current_lesson();

      if( $course_lesson === null )
        return '';

      $args = [
        'lesson_list'           => $this->course_sections_map[ $course_lesson->section_id ]['lessons'],
        'course_read_only'      => $this->read_only,
        'course_id'             => $this->course_id,
        'course_name'           => get_the_title( $this->course_id ),
        'lesson_id'             => $course_lesson->id,
        'lesson_title'          => $course_lesson->title,
        'lesson_content'        => $course_lesson->content,
        'lesson_options'        => $course_lesson->options,
        'lesson_is_completed'   => $course_lesson->is_completed,
        'next_lesson_id'        => $course_lesson->next_lesson_id,
        'next_lesson_title'     => $course_lesson->next_lesson_title,
        'previous_lesson_id'    => $course_lesson->previous_lesson_id,
        'previous_lesson_title' => $course_lesson->previous_lesson_title,
        'has_overview_page'     => $this->course_has_overview_page
      ];

      return $this->skeleton_navigation_current_item_content_prepare( WPEP_Controller::instance()->template->get_template( 'course-single/video-lesson-content.php', $args ) );
    }

    $args = [
      'course_id'         => $this->course_id,
      'course_read_only'  => $this->read_only,
      'course_sections'   => $this->course_sections_map,
      'course_title'      => get_the_title( $this->course_id ),
      'course_content'    => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
      'course_status'     => $this->_get_course_status()
    ];

    $args = apply_filters( 'wpep_course_single_content_video_args', $args );

    return apply_filters(
      'wpep_single_content_video_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/video-content.php', $args ),
      $args
    );
  }

  private function _get_course_status() {
    $completed_counter = 0;

    foreach( $this->course_sections_map as $course_section )
      if( $course_section['is_completed'] )
        $completed_counter++;

    if( $completed_counter == count( $this->course_sections_map ) )
      return 'completed';

    if( $completed_counter != 0 )
      return 'progress';

    foreach( $this->course_sections_map as $course_section )
      foreach( $course_section['lessons'] as $course_section_lesson )
        if( $course_section_lesson['is_completed'] == 1 )
          return 'progress';

    return 'pending';
  }

  public function display_sidebar_content( $course_id = false ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    $thumbnail = wpep_course_thumbnail_url( $this->course_id );

    echo '<div class="wpep-widget wpep-course-widget-primary">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<div class="video-course-thumbnail-widget">';
    echo        '<img src="' . $thumbnail . '"/>';
    echo        '<h2>' . get_the_title( $this->course_id ) . '</h2>';
    echo      '</div>';
    echo    '</div>';
    echo '</div>';

    if( wpep_course_has_progress_bar( $this->course_id ) ) {
      echo '<div class="wpep-widget">';
      echo    '<div class="wpep-widget-wrapper">';
      echo      '<div class="video-course-progress-widget">';
      echo        '<h2>' . wpep_get_setting_translated( 'course-video-course-progress' ) . '</h2>';
      echo         wpep_progress_bar(
        wpep_user_course_progress_percentage($this->course_id, get_current_user_id() ),
        wpep_course_has_progress_bar_percentage_number( $this->course_id )
      );
      echo      '</div>';
      echo    '</div>';
      echo '</div>';
    }

  }

}