<?php

namespace WPEP\Entity;

class SystemCache {

  protected static $_instance = [];

  /**
   * @param $namespace
   * @param $event
   * @param $started_at
   * @return SystemCache
   */
  public static function instance( $args ) {
    $args_unique_string = wpep_implode_associative_array( $args, '_', '_' );

    if( !isset( self::$_instance[ $args_unique_string ] ) )
      self::$_instance[ $args_unique_string ] = new self( $args );

    return self::$_instance[ $args_unique_string ];
  }

  public static function get_namespace_list() {
    $sql = 'SELECT DISTINCT namespace FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG;

    return wpep_wpdb()->get_col( $sql );
  }

  public static function delete_old_entries() {
    $old_entries_started_at_marker = strtotime( 'today 00:00:00' ) - WEEK_IN_SECONDS;

    return wpep_wpdb()->query('DELETE FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_LOG . '` WHERE started_at < ' . $old_entries_started_at_marker );
  }

  public static function truncate_table() {
    wpep_wpdb()->query( 'TRUNCATE `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_CACHE . '`' );
  }

  protected $_table_system_log_field_whitelist = [ 'id', 'user_id', 'post_id', 'namespace', 'segment', 'content', 'updated_at', 'expires_at' ];

  public $id = 0;
  public $user_id = 0;
  public $post_id = 0;
  public $namespace;
  public $segment;
  public $content;
  public $updated_at;
  public $expires_at;

  public function __construct( $args ) {
    foreach( $args as $arg_key => $arg_value ) {
      if( !in_array( $arg_key, $this->_table_system_log_field_whitelist ) )
        continue;

      $this->$arg_key = $arg_value;
    }

    $this->_sync_from_db();
  }

  public function is_configuration_required() {
    if( $this->id === 0 )
      return true;

    if( $this->expires_at === null )
      return false;

    return strtotime( $this->expires_at ) < time();
  }

  public function save() {
    $this->updated_at = date( "Y-m-d H:i:s", time() );

    if( $this->id !== 0 ) {
      wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_CACHE, [
        'user_id'             => intval( $this->user_id ),
        'post_id'             => intval( $this->post_id ),
        'namespace'           => sanitize_text_field( $this->namespace ),
        'segment'             => sanitize_text_field( $this->segment ),
        'content'             => maybe_serialize( $this->content ),
        'updated_at'          => $this->updated_at,
        'expires_at'          => $this->expires_at
      ], [
        'id'  => $this->id
      ] );

      return;
    }

    wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_CACHE, [
      'user_id'             => intval( $this->user_id ),
      'post_id'             => intval( $this->post_id ),
      'namespace'           => sanitize_text_field( $this->namespace ),
      'segment'             => sanitize_text_field( $this->segment ),
      'content'             => maybe_serialize( $this->content ),
      'updated_at'          => $this->updated_at,
      'expires_at'          => $this->expires_at
    ] );

    $this->id = wpep_wpdb()->insert_id;
  }

  private function _sync_from_db() {
    $entry_from_db = wpep_wpdb()->get_row( 'SELECT * FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_SYSTEM_CACHE . ' ' . wpep_sql_where( [
        'user_id'             => intval( $this->user_id ),
        'post_id'             => intval( $this->post_id ),
        'namespace'           => sanitize_text_field( $this->namespace ),
        'segment'             => sanitize_text_field( $this->segment ),
      ] ) );

    if( $entry_from_db === null )
      return;

    $this->id = $entry_from_db->id;
    $this->content    = maybe_unserialize( $entry_from_db->content );
    $this->updated_at = $entry_from_db->updated_at;
    $this->expires_at = $entry_from_db->expires_at;

  }

}
