<?php

/**
 * @return int
 */
function wpep_get_auth_status_level() {
  $auth_status_level = wpep_get_setting_flag( 'user-auth-status-level' );

  $auth_status_level = apply_filters( 'wpep_auth_status_level', $auth_status_level );

  return $auth_status_level;
}

/**
 * @param null $index
 * @return array|string
 */
function wpep_get_auth_status_level_options( $index = null ) {
  $options = [
    0 => __( sprintf( 'Users are not required to login in order to access %s Content', WPEP_PLUGIN_NAME ), "wpep" ),
    1 => __( 'Users are required to login in order to view Courses and Videos', "wpep" ),
    2 => __( 'Users are required to login in order to view the Index Page, Category Pages, Courses, Videos', "wpep" ),
    3 => __( 'Users are required to login in order to view the Index Page, Category Pages, Courses, Course ShortCode Embeds, Index Page Embeds, Videos', "wpep" )
  ];

  return ( $index !== NULL ? $options[ $index ] : $options );
}

/**
 * @return array
 */
function wpep_get_auth_status_page_restrictions() {
  $status_level      = intval( wpep_get_auth_status_level() );
  $page_restrictions = [];

  if( $status_level >= 1 )
    $page_restrictions = array_merge( $page_restrictions, [ 'content-course-single.php', 'content-video-single.php', 'content-assessment-single.php' ] );

  if( $status_level >= 2 )
    $page_restrictions = array_merge( $page_restrictions, [ 'primary.php', 'primary-category.php' ] );

  if( $status_level >= 3 )
    $page_restrictions = array_merge( $page_restrictions, [ 'course-shortcode', 'index-page-shortcode', 'course-grid-shortcode' ] );

  return $page_restrictions;
}

/**
 * @param $user_login
 * @return bool|string|WP_Error
 */
function wpep_retrieve_password( $user_login ) {
  $errors = new WP_Error();

  if ( strpos( $user_login, '@' ) ) {
    $user_data = get_user_by( 'email', trim( wp_unslash( $user_login ) ) );
    if ( empty( $user_data ) )
      $errors->add( 'invalid_email', wpep_get_setting_translated( 'user-auth-notification-recovery-not-registered' ) );
  } else {
    $login = trim( $user_login );
    $user_data = get_user_by('login', $login);
  }

  do_action( 'lostpassword_post', $errors );

  if ( $errors->get_error_code() )
    return $errors;

  if ( !$user_data ) {
    $errors->add( 'invalidcombo', wpep_get_setting_translated( 'user-auth-notification-recovery-not-registered' ) );
    return $errors;
  }

  $user_login = $user_data->user_login;
  $user_email = $user_data->user_email;
  $key = get_password_reset_key( $user_data );

  if ( is_wp_error( $key ) ) {
    return $key;
  }

  $title   = wpep_controller()->userEngagement->misc->email_password_reset_title( '' );
  $message = wpep_controller()->userEngagement->misc->email_password_reset_content_no_template( $key, $user_login );

  WPEP\Controller::instance()->email->send( $user_email, wp_specialchars_decode( $title ), $message, true );

  return true;
}