<?php

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

/**
 * @param $post_id
 * @return int|bool
 */
function wpep_content_primary_sidebar_active( $post_id = null ) {
  $response = is_active_sidebar('wpep-sidebar');

  $post_id = ( $post_id == null ? ( get_post_type() === WPEP_POST_TYPE_COURSE ? get_the_ID() : 0 ) : $post_id );

  $response = apply_filters( 'wpep_single_content_sidebar_active', $response, $post_id );

  return $response;
}

/**
 * @return int|bool
 */
function wpep_content_primary_layout_name() {
  return WPEP_Controller::instance()->content->library->get_layout_name();
}

/**
 * @return int|bool
 */
function wpep_content_primary_has_categories() {
  return WPEP_Controller::instance()->content->library->has_categories();
}

function wpep_content_primary_available_post_types() {
  return wpep_content_type()->library_post_types();
}

/**
 * @return int|bool
 */
function wpep_content_primary_has_categories_navigation() {
  return WPEP_Controller::instance()->content->library->has_categories_navigation();
}

/**
 * @return int|bool
 */
function wpep_content_primary_has_all_categories_navigation() {
  return WPEP_Controller::instance()->content->library->has_all_categories_navigation();
}

function wpep_content_primary_has_category_card_labels() {
  return wpep_get_setting_flag( 'index-page-category-presentation-labels' );
}

/**
 * @return int|bool
 */
function wpep_content_primary_course_grid_has_progress_bar() {
  if( !isset( WPEP_Controller::instance()->content->library->gridItem ) )
    return false;

  return WPEP_Controller::instance()->content->library->gridItem->course_grid_has_progress_bar();
}

function wpep_content_primary_order_options() {
  $order_options = [
    'ASC'   => __( 'Default - Ascending', 'wpep' ),
    'DESC'  => __( 'Descending', 'wpep' )
  ];

  return apply_filters( 'wpep_index_page_post_order_options', $order_options );
}

function wpep_content_primary_order_by_options() {
  $order_options = [
    'meta_value' => __( 'Default - Manual Order', 'wpep' ),
    'title'      => __( 'Title', 'wpep' ),
    'date'       => __( 'Publish Date', 'wpep' ),
    'modified'   => __( 'Last Updated Date', 'wpep' ),
    'post__in'   => __( "Advanced : Order in which you've selected specific items.", "wpep" )
  ];

  return apply_filters( 'wpep_index_page_post_order_by_options', $order_options );
}

/**
 * @param null $post_type
 * @return array
 */
function wpep_content_primary_row_options( $post_type = null ) {
  $index_page_row_options = [
    1 => sprintf( __( '%s Per Row', 'wpep' ), '1' ),
    2 => sprintf( __( '%s Per Row', 'wpep' ), '2' ),
    3 => sprintf( __( '%s Per Row', 'wpep' ), '3' ),
    4 => sprintf( __( '%s Per Row', 'wpep' ), '4' ),
  ];

  if( $post_type === WPEP_POST_TYPE_COURSE )
    $index_page_row_options = [ WPEP_ITEMS_PER_ROW_DYNAMIC => __( "Dynamic Course Tab Placement", "wpep" ) ] + $index_page_row_options;

  return apply_filters( 'wpep_index_page_items_per_row', $index_page_row_options );
}

/**
 * @return array
 */
function wpep_content_categories_row_options() {
  $courses_per_row = [
    0 => __( "Autodetect", 'wpep' ),
    1 => __( "Disable Category Navigation", "wpep" ),
    2 => sprintf( __( '%s Per Row', 'wpep' ), '2' ),
    3 => sprintf( __( '%s Per Row', 'wpep' ), '3' ),
    4 => sprintf( __( '%s Per Row', 'wpep' ), '4' ),
    5 => sprintf( __( '%s Per Row', 'wpep' ), '5' ),
    6 => sprintf( __( '%s Per Row', 'wpep' ), '6' ),
    7 => sprintf( __( '%s Per Row', 'wpep' ), '7' ),
    8 => sprintf( __( '%s Per Row', 'wpep' ), '8' ),
  ];

  return apply_filters( 'wpep_index_page_categories_per_row', $courses_per_row );
}

/**
 * @param string|array $course_list_ids
 * @param array $options
 * @return string
 */
function wpep_content_primary_course_list_by_ids( $course_list_ids, $options = [] ) {
  if( empty( $course_list_ids ) )
    return '';

  return do_shortcode( wpep_content_primary_course_list_shortcode_by_ids( $course_list_ids, $options ) );
}

/**
 * @param $course_list_ids
 * @param array $options
 * @return string
 */
function wpep_content_primary_course_list_shortcode_by_ids( $course_list_ids, $options = [] ) {
  if( empty( $course_list_ids ) )
    return '';

  $course_list_ids = ( is_array( $course_list_ids ) ? implode( ",", $course_list_ids ) : $course_list_ids );

  $options = array_merge( [
    'verify_show_flag'    => 0,
    'category_navigation' => 0,
    'display_header'      => 0,
    'courses_per_row'     => 3,
    'videos_per_row'      => 0,
    'ebooks_per_row'      => 0,
    'offers_per_row'      => 0,
    'list_items_per_row'  => 0
  ], $options );

  $response = '[' . WPEP_SHORT_CODE_PREFIX . 'index courses="' . $course_list_ids . '"';

  foreach( $options as $option_key => $option_value )
    $response .= ' ' . $option_key . '="' . $option_value . '"';

  $response .= ']';

  return $response;
}

/**
 * @param string|array $post_ids
 * @param array $options
 * @return string
 */
function wpep_content_primary_list_by_id_map( $post_ids, $options = [] ) {
  if( empty( $post_ids ) )
    return '';

  return do_shortcode( wpep_content_primary_list_shortcode_by_id_map( $post_ids, $options ) );
}

/**
 * @param $post_ids
 * @param array $options
 * @return string
 */
function wpep_content_primary_list_shortcode_by_id_map( $post_ids, $options = [] ) {
  if( empty( $post_ids ) )
    return '';

  $post_ids = ( is_array( $post_ids ) ? implode( ",", $post_ids ) : $post_ids );

  $options = array_merge( [
    'verify_show_flag'    => 0,
    'category_navigation' => 0,
    'display_header'      => 0,
  ], $options );

  $response = '[' . WPEP_SHORT_CODE_PREFIX . 'index post_ids="' . $post_ids . '"';

  foreach( $options as $option_key => $option_value )
    $response .= ' ' . $option_key . '="' . ( is_array( $option_value ) ? esc_attr( json_encode( $option_value ) ) : $option_value ) . '"';

  $response .= ']';

  return $response;
}