<?php

/**
 * @param $user_id
 * @param string $type
 * @return bool
 */
function wpep_user_has_notification( $user_id, $type = 'general' ) {
  if( in_array( $type, [ 'auth', 'critical' ] ) )
    return true;

  if( intval( get_user_meta( $user_id, 'wpep_disable_email_notifications', true ) ) )
    return false;

  if( $type === 'weekly_recap'
      && intval( get_user_meta( $user_id, 'wpep_disable_email_weekly_recap', true ) ) )
    return false;

  return true;
}

function wpep_is_user_agreement_required( $user_id ) {
  if( $user_id == 0 )
    return false;

  if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) == false
      && wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) == false )
    return false;

  if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) ) {
    $user_agreement_date = get_user_meta( $user_id, WPEP_USER_META_AGREEMENT_TOS_DATE, true );

    if( $user_agreement_date == '' )
      return true;

    $last_agreement_change = wpep_get_setting( 'user-service-agreement-last-change' );

    if( $last_agreement_change !== '' ) {
      if( strtotime( $user_agreement_date ) < strtotime($last_agreement_change ) )
        return true;
    }

  }

  if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) ) {
    $user_agreement_date = get_user_meta( $user_id, WPEP_USER_META_AGREEMENT_PRIVACY_DATE, true );

    if( $user_agreement_date == '' )
      return true;

    $last_agreement_change = wpep_get_setting( 'user-privacy-agreement-last-change' );

    if( $last_agreement_change !== '' ) {
      if( strtotime( $user_agreement_date ) < strtotime($last_agreement_change ) )
        return true;
    }
  }

  return false;
}

function wpep_user_content_allowed_html_tags() {
  return [
    'a'      => [ 'href' => [], 'target' => [], 'alt' => [], 'data-wpep-mention-user-id' => [] ],
    'br'     => [],
    'video'  => [ 'width' => [], 'height' => [] ],
    'source' => [ 'src' => [], 'type' => [] ],
    'strong' => [ 'style' => [] ],
    'sub'    => [ 'style' => [] ],
    'sup'    => [ 'style' => [] ],
    's'      => [ 'style' => [] ],
    'i'      => [ 'style' => [] ],
    'u'      => [ 'style' => [] ],
    'span'   => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [], 'data-wpep-mention-user-id' => [] ],
    'h1'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'h2'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'h3'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'ol'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'ul'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'li'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'em'     => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [] ],
    'hr'     => [],
    'p'      => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'data' => [], 'data-wpep-mention-user-id' => [] ],
    'img'    => [ 'align' => [], 'class' => [], 'type' => [], 'id' => [], 'style' => [], 'src' => [], 'alt' => [], 'href' => [], 'rel' => [], 'target' => [], 'value' => [], 'name' => [], 'width' => [], 'height' => [], 'data' => [], 'title' => [] ]
  ];
}

function wpep_user_update_student_supervisor( $user_id, $new_information ) {
  if( empty( $new_information ) ) {
    delete_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID );

    return;
  }

  $new_information     = is_array( $new_information ) ? $new_information : [ $new_information ];
  $current_information = get_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, false );

  if( !empty( $current_information ) )
    foreach( $current_information as $existent_supervisor_user_id )
      if( !in_array( $existent_supervisor_user_id, $new_information ) )
        delete_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, $existent_supervisor_user_id );

  foreach( $new_information as $supervisor_user_id )
    if( !in_array( $supervisor_user_id, $current_information ) )
      add_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, intval( $supervisor_user_id ) );
}

function wpep_get_user_role_list_array_assoc( $exclude_roles = [ 'administrator' ] ) {
  if ( ! function_exists( 'get_editable_roles' ) ) {
    require_once ABSPATH . 'wp-admin/includes/user.php';
  }

  $response = [];
  $user_role_list = get_editable_roles();

  foreach ( $user_role_list as $role => $details )
    $response[ $role ] = translate_user_role($details['name'] );

  if( is_array( $exclude_roles ) && !empty( $exclude_roles ) ) {
    foreach( $exclude_roles as $exclude_role )
      unset( $response[ $exclude_role ] );
  }

  asort( $response );

  return $response;
}

function wpep_get_users_by_role( $role ) {
  $args = [ 'number' => -1, ];

  if( is_array( $role ) )
    $args['role__in'] = $role;
  else
    $args['role'] = $role;

  return get_users( $args );
}

function wpep_get_users_by_role_as_array_assoc( $role, $array_key_param, $array_value_param ) {
  $user_list = wpep_get_users_by_role( $role );

  if( empty( $user_list ) )
    return [];

  $response = [];

  foreach( $user_list as $user )
    $response[ $user->$array_key_param ] = $user->$array_value_param;

  return $response;
}

function wpep_search_users_by_args( $args ) {
  if( !isset( $args[ 'search'] ) )
    return [];

  $search_term     = sanitize_text_field( $args[ 'search' ] );
  $match_ids       = ( isset( $args['match_ids'] ) && !empty( $args['match_ids'] ) ? array_map( 'intval', $args['match_ids'] ) : [] );
  $limit           = ( isset( $args[ 'limit' ] ) ? $args[ 'limit'] : 15 );

  $sql = 'SELECT ID FROM ' . wpep_wpdb()->users . ' users_table
               LEFT JOIN ' . wpep_wpdb()->usermeta . ' user_meta_fn_table 
                      ON user_meta_fn_table.user_id = users_table.ID
                     AND user_meta_fn_table.meta_key = "first_name"
               LEFT JOIN ' . wpep_wpdb()->usermeta . ' user_meta_ln_table 
                      ON user_meta_ln_table.user_id = users_table.ID
                     AND user_meta_ln_table.meta_key = "last_name"
                   WHERE ( 
                         users_table.user_login LIKE "%' . $search_term . '%"
                      OR users_table.user_nicename LIKE "%' . $search_term . '%"
                      OR users_table.user_email LIKE "%' . $search_term . '%"
                      OR users_table.display_name LIKE "%' . $search_term . '%"
                      OR user_meta_fn_table.meta_value LIKE "%' . $search_term . '%"
                      OR user_meta_ln_table.meta_value LIKE "%' . $search_term . '%"
                    ';

  if( strpos( $search_term, ' ' ) !== false ) {
    foreach( explode( ' ', $search_term ) as $search_term_inner ) {
      $sql .= ' OR user_meta_fn_table.meta_value LIKE "%' . $search_term_inner . '%"';
      $sql .= ' OR user_meta_ln_table.meta_value LIKE "%' . $search_term_inner . '%"';
    }
  }

  $sql .= ')';

  if( !empty( $match_ids ) )
    $sql .= ' AND users_table.ID IN (' . implode( ',', $match_ids ) . ')';

  $user_ids = wpep_wpdb()->get_col( $sql );

  if( empty( $user_ids ) )
    return [];

  if( count( $user_ids ) > $limit )
    $user_ids = array_slice( $user_ids, 0, $limit );

  return get_users( [ 'paged' => -1, 'include' => $user_ids ] );
}

function wpep_get_user_meta_information_list_keys() {
  return apply_filters( 'wpep_user_meta_information_list_keys', [
    WPEP_USER_META_COVER_PICTURE,
    WPEP_USER_META_PROFILE_PICTURE,
    WPEP_USER_META_PROFILE_PRIVACY_TYPE,
    WPEP_USER_META_SOCIAL_NETWORK_PROFILES,
    WPEP_USER_META_STUDENT_IDENTIFICATION
  ] );
}