<?php

abstract class WPEP_Data_Third_Party_Import_Integration {

  public $import_entity_type = "post"; // Or taxonomy.
  public $import_entity_sub_type          = false;
  public $import_entity_sub_type_meta_key = false;

  public $taxonomy_term_map = [];

  private $_migration_option_name = 'wpep_ds_tpi_';
  protected $_entity_import_status = 'pending';
  protected $_entity_import_que = [];
  protected $_entity_import_map  = [];
  protected $_misc_import_map  = [];
  protected $_terms_import_map = [];
  public $preferences_import_map = [];

  public function setup() {
    $this->_migration_option_name = $this->_migration_option_name . $this->get_alias();

    $data = get_option( $this->_migration_option_name, [
      'status'      => 'pending',
      'que'         => [],
      'post'        => [],
      'misc'        => [],
      'terms'       => [],
      'preferences' => $this->get_default_preferences()
    ] );

    $this->_entity_import_status  = $data[ 'status' ];
    $this->_entity_import_que     = $data[ 'que' ];
    $this->_entity_import_map     = $data[ 'post' ];
    $this->_misc_import_map       = $data[ 'misc' ];
    $this->_terms_import_map      = $data[ 'terms' ];
    $this->preferences_import_map = $data[ 'preferences' ];
  }

  public function sync_settings() {
    update_option( $this->_migration_option_name, [
      'status'      => $this->_entity_import_status,
      'que'         => $this->_entity_import_que,
      'post'        => $this->_entity_import_map,
      'misc'        => $this->_misc_import_map,
      'terms'       => $this->_terms_import_map,
      'preferences' => $this->preferences_import_map
    ] );
  }

  abstract public function get_name();

  abstract public function get_alias();

  abstract public function get_description();

  abstract public function get_default_preferences();

  abstract public function get_preferences_field_map();

  abstract public function get_content_types_list();

  public function set_preferences_field_map( $preferences_import_map ) {
    $this->preferences_import_map = $preferences_import_map;
  }

  public function get_que() {
    return $this->_entity_import_que;
  }

  public function get_import_map() {
    return $this->_entity_import_map;
  }

  public function get_import_status() {
    return $this->_entity_import_status;
  }

  public function import_available() {
    $entity_ids = $this->implementation_get_import_entity_ids();

    if( !is_array( $entity_ids ) )
      return false;

    return count( $entity_ids ) >= 1;
  }

  public function import_start() {
    if( $this->_entity_import_status != 'pending' )
      $this->import_end();

    $this->_entity_import_status = 'started';
    $this->_entity_import_que = $this->implementation_get_import_entity_ids();

    $this->sync_settings();

    return true;
  }

  public function import_process() {
    if( !in_array( $this->_entity_import_status, [ 'started', 'processing' ] ) )
      return false;

    if( empty( $this->_entity_import_que ) ) {
      $this->_entity_import_status = 'completed';
      $this->sync_settings();

      return true;
    }

    $i = 1; foreach( $this->_entity_import_que as $key => $import_entity_id ) {
      if( !is_numeric( $import_entity_id ) )
        continue;

      if( $i >= 5 )
        break;

      $wpep_post_id = $this->implementation_import_base_entity( $import_entity_id );

      if( $wpep_post_id === false ) {
        unset( $this->_entity_import_que[ $key ] );
        $i++;

        continue;
      }

      $this->implementation_import_single( $import_entity_id, $wpep_post_id );

      $this->_entity_import_map[ $import_entity_id ] = $wpep_post_id;

      unset( $this->_entity_import_que[ $key ] );
      $i++;
    }

    $this->_entity_import_status = ( empty( $this->_entity_import_que ) ? 'completed' : 'processing' );
    $this->sync_settings();

    return true;
  }

  public function import_cleanup() {
    if( !in_array( $this->_entity_import_status, [ 'cleanup_start', 'cleanup_processing' ] ) )
      return false;

    if( $this->_entity_import_status === 'cleanup_start' ) {
      $this->_entity_import_que = $this->implementation_get_import_entity_ids();
    } elseif( empty( $this->_entity_import_que ) ) {
      $this->_entity_import_status = 'cleanup_completed';

      $this->sync_settings();

      return true;
    } else {
      $this->_entity_import_status = 'cleanup_processing';
    }

    $i = 1; foreach( $this->_entity_import_que as $key => $post_id ) {
      if( $i >= 5 ) {
        $this->sync_settings();

        return true;
      }

      $this->implementation_import_single_remove( $post_id );

      unset( $this->_entity_import_que[ $key ] );
    }

    if( empty( $this->_entity_import_que ) )
      $this->_entity_import_que = 'cleanup_completed';

    $this->sync_settings();

    return true;
  }

  abstract public function implementation_import_confirmation_content();

  /**
   * @param $importer_map
   * @param $importer_que
   * @return string
   */
  abstract public function implementation_import_progressing_notification_text( $importer_map, $importer_que );

  abstract public function implementation_import_success_preview_table_data( $importer_map );

  public function import_end() {
    delete_option( $this->_migration_option_name );
  }

  abstract public function implementation_import_base_entity( $import_entity_id );

  abstract public function implementation_get_import_entity_ids();

  abstract public function implementation_import_single( $import_entity_id, $wpep_post_id );

  abstract public function implementation_import_single_remove( $import_entity_id );

}