<?php

use WPEP\Container\LicenseAndUpdates as WPEP_Coordinator_LicenseAndUpdates;

class WPEP_Integration_License_And_Updates extends WPEP_Coordinator_LicenseAndUpdates {

  public $product_name;
  public $product_id;
  public $plugin_alias;
  public $plugin_version;
  public $options_prefix;

  public $_top_level = false;

  public function __construct(
    $product_name,
    $product_id,
    $plugin_alias,
    $plugin_version,
    $options_prefix
  ) {
    $this->product_name   = $product_name;
    $this->product_id     = $product_id;
    $this->plugin_alias   = $plugin_alias;
    $this->plugin_version = $plugin_version;
    $this->options_prefix = $options_prefix;

    parent::__construct();
  }

  public function get_product_name() {
    return $this->product_name;
  }

  public function get_product_id() {
    return $this->product_id;
  }

  public function get_plugin_alias() {
    return $this->plugin_alias;
  }

  public function get_plugin_version() {
    return $this->plugin_version;
  }

  public function get_options_prefix() {
    return $this->options_prefix;
  }

  public function get_license() {
    if( $this->in_bundled_license() )
      return wpep_controller()->adminController->license->get_license();

    return get_option( $this->get_option_license_key(), '' );
  }

  public function prepare_license_status( $licenseStatus ) {
    return $licenseStatus;
  }

  public function is_valid_license() {
    if( $this->in_bundled_license() )
      return wpep_controller()->adminController->license->is_valid_license();

    if( $this->get_license() == '' )
      return false;

    $status = get_option( $this->get_option_license_key_status(), null );

    if( $status === 'valid' )
      return true;

    if( intval( $status ) === 1 )
      return true;

    return false;
  }

  public function get_option_license_key() {
    return $this->get_options_prefix() . 'license';
  }

  public function get_option_license_key_status() {
    return $this->get_options_prefix() . 'license_status';
  }

  public function get_license_status_text() {
    if( $this->is_valid_license() )
      return '<p style="color:#2ecc71">' . __( "Your License is Valid", "wpep" ) . '</p>';

    if( !in_array( get_option( $this->get_option_license_key_status(), null ), [ 'valid', 'invalid', null ] ) )
      return '<p style="color:#e74c3c">' . sprintf( __( "This License key couldn't be verified. Error : %s", "wpep" ), get_option( $this->get_option_license_key_status(), '???' ) ) . '</p>';

    if( $this->get_license() !== '' )
      return '<p style="color:#e74c3c">' . __( "Your License is invalid", "wpep" ) . '</p>';

    return '<p style="color:#4aa3df">' . __( "Please enter your license in order to receive automatic updates and support", 'wpep' ) . '</p>';
  }

  /**
   * @deprecated
   * @return array
   */
  public function get_license_key_field_as_array() {
    return [];
  }

  /**
   * @deprecated
   * @param bool $separator_text
   * @param string $field_key
   * @return array
   */
  public function get_license_key_field_as_indexed_array( $separator_text = false, $field_key = 'license' ) {
    return [];
  }

  public function set_license($license) {
    if( $this->in_bundled_license() )
      return $this;

    return parent::set_license($license);
  }

  public function in_bundled_license() {
    if( wpep_controller()->adminController->license === null )
      return false;

    return in_array( $this->get_product_id(), wpep_controller()->adminController->license->get_license_bundled_products() );
  }

}